<?php
/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Csd
 * @subpackage Csd/admin
 * @author     ORION <support@orionorigin.com>
 */

/**
 * Admin class
 */
class Csd_Admin {

    /**
     * The ID of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string    $plugin_name    The ID of this plugin.
     */
    private $plugin_name;

    /**
     * The version of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string    $version    The current version of this plugin.
     */
    private $version;

    /**
     * Initialize the class and set its properties.
     *
     * @since    1.0.0
     * @param      string $plugin_name       The name of this plugin.
     * @param      string $version    The version of this plugin.
     */
    public function __construct($plugin_name, $version) {

        $this->plugin_name = $plugin_name;
        $this->version = $version;
    }

    /**
     * Register the stylesheets for the admin area.
     *
     * @since    1.0.0
     */
    public function enqueue_styles() {
        wp_enqueue_style($this->plugin_name, CSD_URL . 'admin/css/csd-admin.css', array(), $this->version, 'all');
        if (is_csd_admin_screen()) {
            wp_enqueue_style('csd-simplegrid', CSD_URL . 'admin/css/simplegrid.min.css', array(), $this->version, 'all');
            wp_enqueue_style('csd-colorpicker-css', CSD_URL . 'admin/js/colorpicker/css/colorpicker.css', array(), $this->version, 'all');
            //wp_enqueue_style('csd-tooltip-css', CSD_URL . 'admin/css/tooltip.min.css', array(), $this->version, 'all');
            wp_enqueue_style('csd-o-ui', CSD_URL . 'admin/css/UIs.css', array(), $this->version, 'all');
            wp_enqueue_style('csd-select2-css', CSD_URL . 'admin/css/select2.min.css', array(), $this->version, 'all');
            wp_enqueue_style('o-csd-flexgrid', CSD_URL . 'admin/css/flexiblegs.css', array(), $this->version, 'all');
            wp_enqueue_style('csd-bs-modal-admin-css', CSD_URL . 'public/js/modal/modal.min.css', array(), $this->version, 'all');
        }
    }

    /**
     * Register the JavaScript for the admin area.
     *
     * @since    1.0.0
     */
    public function enqueue_scripts() {
        if (is_csd_admin_screen()) {
            wp_enqueue_script('csd-tabs-js', CSD_URL . 'admin/js/SpryTabbedPanels.min.js', array('jquery'), $this->version, false);
            // wp_enqueue_script('csd-tooltip-js', CSD_URL . 'admin/js/tooltip.js', array('jquery'), $this->version, false);
            wp_enqueue_script('csd-colorpicker-js', CSD_URL . 'admin/js/colorpicker/js/colorpicker.js', array('jquery'), $this->version, false);
            wp_enqueue_script('csd-jquery-cookie-js', CSD_URL . 'admin/js/jquery.cookie.min.js', array('jquery'), $this->version, false);
            wp_enqueue_script('o-admin', CSD_URL . 'admin/js/o-admin.js', array('jquery', 'jquery-ui-sortable'), $this->version, false);
            wp_localize_script( 'o-admin', 'home_url', array( 'o_admin_ajax_url' => home_url( '/' ) ) );
            wp_enqueue_script('select2-js', CSD_URL . 'admin/js/select2.min.js', array('jquery'), $this->version, false);
            wp_enqueue_script('csd-bs-modal-admin-js', CSD_URL . 'public/js/modal/csd-modal.min.js', array('jquery'), $this->version, false);
            wp_enqueue_script($this->plugin_name, CSD_URL . 'admin/js/csd-admin.js', array('jquery', 'select2-js', 'csd-bs-modal-admin-js'), $this->version, false);
            wp_localize_script($this->plugin_name, 'csd_ajax_object', array('csd_ajax_url' => admin_url('admin-ajax.php')));
        }
    }

    /**
     * Newsletter subscription
     */
    public function csd_subscribe() {
        if (isset($_POST['email']) && wp_verify_nonce($_POST['csd_subscription_nonce'], basename(__FILE__))) {// Input var okay.
            $email = sanitize_text_field(wp_unslash($_POST['email'])); // Input var okay.
            if (preg_match('#^[\w.-]+@[\w.-]+\.[a-z]{2,6}$#i', $email)) {
                $url = 'https://orionorigin.com/service/osubscribe/v1/subscribe/?email=' . $email;
                $args = array('timeout' => 120);
                $response = wp_remote_get($url, $args);

                if (is_wp_error($response)) {
                    $error_message = $response->get_error_message();
                    echo esc_html__("Something went wrong: $error_message");
                    die();
                }
                if (isset($response['body'])) {
                    $answer = $response['body'];
                    if ('true' === $answer) {
                        update_option('o-csd-subscribe', 'subscribed');
                        echo esc_html__($answer);
                    } else {
                        echo esc_html__($answer);
                    }

                    die();
                }
            } else {
                echo 'Please enter a valid email address';
                die();
            }
        } else {
            echo 'Nonce not verified.';
            die();
        }
    }

    /**
     * Displays the subscription notice
     */
    public function csd_get_subscription_notice() {
        if (!get_option('o-csd-subscribe') && get_transient('csd-hide-notice') !== 'hide' && is_csd_admin_screen()) {
            wp_nonce_field(basename(__FILE__), 'csd_subscription_nonce');
            ?>
            <div id="csd-subscription-notice" class="notice notice-info">

                <div id="csd-plug-logo-text" >
                    <img id="plug-logo" style="height:50px; width: 50px"src="<?php echo esc_url(CSD_URL); ?>/admin/images/csd.png">
                    <p> 
                        <strong>Custom Signs Designer for WooCommerce</strong>: <?php esc_html_e(' Sign up now to receive new releases notices and important bugs fixes directly', 'csd'); ?>
                        <br> <?php esc_html_e('into your inbox!', 'csd'); ?>
                    </p>

                </div>

                <div id="csd-plug-sucribe-form">
                    <input type="email" id="o_user_email" name="usermail" placeholder="Your email here">
                    <img id="csd-subscribe-loader" style="display:none;" src="<?php echo esc_url(CSD_URL); ?>/admin/images/loader.gif" >
                    <button id="csd-subscribe" class="button button-primary"><?php esc_attr_e('Subscribe', 'csd'); ?></button>
                    <a id="csd-dismiss"><?php esc_attr_e('Not now', 'csd'); ?></a>
                </div>
            </div>
            <?php
        }
        ?>
        <div id="csd-subscription-success-notice" class="notice notice-info is-dismissible" style="display:none;">
            <img src="<?php echo esc_url(CSD_URL); ?>/admin/images/csd.png">
            <div> <strong>Custom Signs Designer for WooCommerce</strong>: <?php esc_html_e('Thank you for subscribing! ', 'csd'); ?></div>
        </div>
        <?php
    }

    /**
     * hide the notice for two weeks
     */
    public function csd_hide_notice() {
        set_transient('csd-hide-notice', 'hide', 2 * WEEK_IN_SECONDS);
        echo 'ok';
        die();
    }

    /**
     * Builds all the plugin menu and submenu
     */
    public function csd_add_submenu() {
        if (class_exists('WooCommerce') /* && class_exists("Imagick") */) {
            global $submenu;
            $icon = CSD_URL . 'admin/images/csd-dashicon.png';
            add_menu_page('Custom Signs Designer for WooCommerce', 'CSD', 'manage_product_terms', 'csd-manage-fonts', array($this, 'csd_get_fonts_page'), $icon);
            add_submenu_page('csd-manage-fonts', esc_html__('Fonts', 'csd'), esc_html__('Fonts', 'csd'), 'manage_product_terms', 'csd-manage-fonts', array($this, 'csd_get_fonts_page'));
            add_submenu_page('csd-manage-fonts', esc_html__('Cliparts', 'csd'), esc_html__('Cliparts', 'csd'), 'manage_product_terms', 'edit.php?post_type=csd-cliparts', false);
            add_submenu_page('csd-manage-fonts', esc_html__('Configurations', 'csd'), esc_html__('Configurations', 'csd'), 'manage_product_terms', 'edit.php?post_type=csd-config', false);
            if (class_exists('Ofb')) {
                add_submenu_page('csd-manage-fonts', __('Forms', 'csd'), __('Forms', 'csd'), 'manage_product_terms', 'edit.php?post_type=ofb', false);
            }
            add_submenu_page('csd-manage-fonts', esc_html__('Settings', 'csd'), esc_html__('Settings', 'csd'), 'manage_product_terms', 'csd-manage-settings', array($this, 'csd_get_settings_page'));
            //add_submenu_page('csd-manage-fonts', esc_html__('Request a feature', 'csd'), esc_html__('Request a feature', 'csd'), 'manage_product_terms', 'csd-request-feature', array($this, 'csd_get_request_page'));
            add_submenu_page('csd-manage-fonts', esc_html__('Getting Started', 'csd'), esc_html__('Getting Started', 'csd'), 'manage_product_terms', 'csd-about', array($this, "csd_get_about_page"));

            $submenu['csd-manage-fonts'][] = array('<div id="user-manual">User Manual</div>', 'manage_product_terms', 'https://orionorigin.com/documentation/seme-custom-signs-designer/getting-started-seme-custom-signs-designer/how-to-set-my-license-in-seme/?utm_source=Seme%20Pro&utm_medium=user%20manual%20submenu&utm_campaign=Seme');
            $submenu['csd-manage-fonts'][] = array('<div id="submit-a-ticket">Submit a ticket</div>', 'manage_product_terms', 'https://orionorigin.com/contact/?utm_source=Seme%20Pro&utm_medium=get%20support%20submenu&utm_campaign=Seme');
        }
    }

    /**
     * Builds the fonts management page
     */
    public function csd_get_fonts_page() {
        include_once CSD_DIR . '/includes/csd-add-fonts.php';
        csd_manage_fonts();
    }

    /**
     * redirect user to the request systeme.
     */
    public function csd_get_request_page() {
        return false;
    }

    /**
     * Redirects the plugin to the about page after the activation
     */
    public function csd_redirect() {
        if (class_exists('WooCommerce')) {
            if (get_option('csd_do_activation_redirect', false)) {
                delete_option('csd_do_activation_redirect');
                wp_safe_redirect(admin_url('admin.php?page=csd-about'));
            }
        }
    }

    /**
     * Notice user about fonts mangement
     */
    public function csd_get_help_notices() {
        $screen = get_current_screen();
        if (is_csd_admin_screen() && strpos($screen->base, 'csd-manage-fonts') !== false) {
            echo '<div class="csd-info">
                   <p><b>Custom Signs Designer for WooCommerce: </b>' . esc_html__('Learn more about fonts management', 'csd') . ' <a class="button" href="https://orionorigin.com/tutorials/woocommerce-product-designer-tutorials/how-to-manage-fonts/" target="_blank">' . esc_html__('here', 'csd') . '</a></p>
                </div>';
        }

        if (!class_exists('Imagick') && is_csd_admin_screen()) {
            if (is_csd_admin_screen()) {
                ?>
                <div id="csd-subscription-success-notice" class="notice notice-info">
                <?php esc_html_e('The required imagick classes are not found.', 'csd'); ?> 
                    <a target="<?php esc_attr_e('_blank', 'csd'); ?>" href="<?php echo esc_url('http://php.net/manual/en/imagick.setup.php', 'csd'); ?>"> <?php esc_html_e('Click here', 'csd'); ?> </a>
                <?php esc_html_e('to learn how to install it. ', 'csd'); ?>
                </div>  
                    <?php
                }
            }
        }

        /**
         * add new upload types
         */
        public function csd_add_custom_mime_types($mimes) {
            return array_merge(
                    $mimes, array(
                'svg' => 'image/svg+xml',
                'ttf' => 'application/x-font-ttf',
                    )
            );
        }

        /**
         * add new upload types
         */
        public function csd_get_max_input_vars_php_ini() {
            $total_max_normal = ini_get('max_input_vars');
            $msg = 'Your max input var is <strong>' . $total_max_normal . '</strong> but this page contains <strong>{nb}</strong> fields. You may experience a lost of data after saving. In order to fix this issue, please increase <strong>the max_input_vars</strong> value in your php.ini file.';
            ?>
        <script type="text/javascript">
            var o_max_input_vars = <?php echo esc_attr($total_max_normal); ?>;
            var o_max_input_msg = "<?php echo esc_attr__($msg, 'csd'); ?>";
        </script>         
        <?php
    }

    /**
     * Alerts the administrator if the minimum requirements are not met
     */
    public function notify_minmimum_required_parameters() {
        global $csd_settings;
        $general_options = get_proper_value($csd_settings, 'csd-general-options');
        $hide_notices = get_proper_value($general_options, 'hide-requirements-notices', false);
        if ($hide_notices) {
            return;
        }
        $message = '';
        $minimum_required_parameters = array(
            'memory_limit' => array(128, 'M'),
            'post_max_size' => array(8, 'M'),
            'upload_max_filesize' => array(32, 'M'),
        );
        foreach ($minimum_required_parameters as $key => $min_arr) {
            $defined_value = ini_get($key);
            if (!stristr($defined_value, 'G', true)) {
                $defined_value_int = str_replace($min_arr[1], '', $defined_value);
                if ($defined_value_int < $min_arr[0]) {
                    $message .= "Your PHP setting <b>$key</b> is currently set to <b>$defined_value</b>. We recommand to set this value at least to <b>" . implode('', $min_arr) . '</b> to avoid any issue with our plugin.<br>';
                }
            }
        }

        $edit_msg = __('How to fix this: You can edit your php.ini file to increase the specified variables to the recommanded values or you can ask your hosting company to make the changes for you.', 'csd');

        if (!empty($message)) {
            echo '<div class="error">
                   <p><b>CSD The Ultimate WooCommerce Signs Plugin: </b><br>' . $message . '<br>
                       <b>' . esc_html($edit_msg) . '</b></p>
                </div>';
        }

        $permalinks_structure = get_option('permalink_structure');
        if (strpos($permalinks_structure, 'index.php') !== false) {
            $message .= 'Your <a href="' . esc_url(admin_url() . 'options-permalink.php') . '">permalinks</a> structure is currently set to <b>custom</b> with index.php present in the structure. We recommand to set this value to <b>Post name</b> to avoid any issue with our plugin.<br>';
        }
        if (!empty($message)) {
            echo '<div class="error"><p><b>CSD The Ultimate WooCommerce Signs Plugin:</b><br>' . $message . '</p></div>';
        }
    }

    /**
     * Builds the settings page
     */
    public function csd_get_settings_page() {
        csd_remove_transients();
        if (isset($_POST) && !empty($_POST) && wp_verify_nonce($_POST['csd_settings_nonce'], basename(__FILE__))) {
            $this->csd_save_tab_options();
            global $wp_rewrite;
            $wp_rewrite->flush_rules(false);
        }
        wp_enqueue_media();
        ?>
        <form method="POST">
            <div id="csd-settings">
        <?php wp_nonce_field(basename(__FILE__), 'csd_settings_nonce'); ?>
                <div class="wrap">
                    <h2><?php esc_attr_e('Custom Signs Designer for WooCommerce Settings', 'csd'); ?></h2>
                    <br>
                </div>
                <div id="TabbedPanels1" class="TabbedPanels">
                    <ul class="TabbedPanelsTabGroup ">
                        <li class="TabbedPanelsTab " tabindex="1"><span><?php esc_attr_e('General', 'csd'); ?></span> </li>
                        <li class="TabbedPanelsTab" tabindex="2"><span><?php esc_attr_e('Output', 'csd'); ?></span></li>
                        <li class="TabbedPanelsTab" tabindex="3"><span><?php esc_attr_e('License', 'csd'); ?></span></li>
                        <li class="TabbedPanelsTab" tabindex="4"><span><?php esc_attr_e('Upload', 'csd'); ?></span></li>
                    </ul>

                    <div class="TabbedPanelsContentGroup">

                        <div class="TabbedPanelsContent">
                            <div class='csd-grid csd-grid-pad'>
        <?php $this->csd_get_general_settings(); ?>                           
                            </div>
                        </div>

                        <div class="TabbedPanelsContent">
                            <div class="csd-grid csd-grid-pad">
        <?php
        $this->csd_get_output_settings();
        ?>
                            </div> 
                        </div>

                        <div class="TabbedPanelsContent">
                            <div class="csd-grid csd-grid-pad">
        <?php
        $this->csd_get_license_settings();
        ?>
                            </div> 
                        </div>

                        <div class="TabbedPanelsContent">
                            <div class="csd-grid csd-grid-pad">
        <?php
        $this->csd_get_upload_settings();
        ?>
                            </div> 
                        </div>

                    </div>
                </div>
            </div>
            <input type="submit" value="<?php esc_attr_e('Save', 'csd'); ?>" class="button button-primary button-large mg-top-10-i">
        </form>
        <?php
    }

    /**
     * Save the settings
     */
    private function csd_save_tab_options() {
        if (isset($_POST) && !empty($_POST) && wp_verify_nonce($_POST['csd_settings_nonce'], basename(__FILE__))) {
            if (isset($_POST['csd-general-options'])) {
                update_option('csd-general-options', $_POST['csd-general-options']);
            }

            if (isset($_POST['csd-output-options'])) {
                update_option('csd-output-options', $_POST['csd-output-options']);
            }

            if (isset($_POST['csd-license-options'])) {
                update_option('csd-license-options', $_POST['csd-license-options']);
            }

            if (isset($_POST['csd-upload-options'])) {
                update_option('csd-upload-options', $_POST['csd-upload-options']);
            }

            $this->csd_init_globals();
            ?>
            <div id="message" class="updated below-h2"><p><?php echo esc_attr_e('Settings successfully saved.', 'csd'); ?></p></div>
            <?php
        }
    }

    /**
     * init the global $csd_settings variable
     *
     * @return array Settings
     */
    public function csd_init_globals() {
        global $csd_settings;
        $csd_settings['csd-general-options'] = get_option('csd-general-options');
        $csd_settings['csd-output-options'] = get_option('csd-output-options');
        $csd_settings['csd-license'] = get_option('csd-license');
        $csd_settings['csd-upload'] = get_option('csd-upload');
    }

    /**
     * Builds the skin settings options
     */
    private function csd_get_skins_settings() {
        $skin_begin = array(
            'type' => 'sectionbegin',
            'id' => 'csd-skin-container',
            'table' => 'options',
        );
        $skins_arr = apply_filters(
                'csd_configuration_skins', array(
            'CSD_Default_Skin' => esc_html__('Default', 'csd'),
                )
        );

        $skins = array(
            'title' => esc_html__('Skin', 'csd'),
            'name' => 'csd-ui-options[skin]',
            'type' => 'select',
            'options' => $skins_arr,
            'class' => 'chosen_select_nostd csd-config-skin',
            'desc' => esc_html__('Editor look and feel.', 'csd'),
        );

        $skin_end = array('type' => 'sectionend');
        $skin_settings = apply_filters(
                'csd_skins_settings', array(
            $skin_begin,
            $skins,
            $skin_end,
                )
        );
        $allowed_tags = csd_allowed_tags();
        $raw_output = o_admin_fields($skin_settings);
        echo wp_kses($raw_output, $allowed_tags);
    }

    /**
     * Get Woocommerce Page.
     * @return array
     */
    private function get_woocommerce_page() {
        $woocommerce_page = array();
        array_push($woocommerce_page, get_option('woocommerce_shop_page_id'));
        array_push($woocommerce_page, get_option('woocommerce_cart_page_id'));
        array_push($woocommerce_page, get_option('woocommerce_checkout_page_id'));
        array_push($woocommerce_page, get_option('woocommerce_myaccount_page_id'));
        array_push($woocommerce_page, get_option('woocommerce_terms_page_id'));
        return $woocommerce_page;
    }

    /**
     * Builds the general settings options
     *
     * @return array Settings
     */
    private function csd_get_general_settings() {
        $options = array();

        $general_options_begin = array(
            'type' => 'sectionbegin',
            'id' => 'csd-general-options',
            'table' => 'options',
            'title' => esc_html__('General Settings', 'csd'),
        );

        $args = array(
            'post_type' => 'page',
            'nopaging' => true,
            'exclude' => $this->get_woocommerce_page()
        );

        $customizer_page = array(
            'title' => esc_html__('Design Page', 'csd'),
            'desc' => esc_html__('This setting allows the plugin to locate the page where customizations are made. Please note that this page can only be accessed by our plugin and should not appear in any menu.', 'csd'),
            'name' => 'csd-general-options[csd_page_id]',
            'type' => 'post-type',
            'default' => '',
            'class' => 'chosen_select_nostd',
            'args' => $args,
        );

        $content_filter = array(
            'title' => esc_html__('Automatically append canvas to the design page.', 'csd'),
            'name' => 'csd-general-options[csd-content-filter]',
            'default' => '1',
            'type' => 'radio',
            'desc' => esc_html__('This option allows you to define whether or not you want to use a shortcode to display the the customizer in the selected page. If this setting is set to No, you will need to manually put the shortcode [csd-editor] in the selected customization page.', 'csd'),
            'options' => array(
                '1' => esc_html__('Yes', 'csd'),
                '0' => esc_html__('No', 'csd'),
            ),
            'class' => 'chosen_select_nostd',
        );

        $customizer_cart_display = array(
            'title' => esc_html__('Parts position in cart', 'csd'),
            'name' => 'csd-general-options[csd-parts-position-cart]',
            'default' => 'thumbnail',
            'type' => 'radio',
            'desc' => esc_html__('This option allows you to set where to show your customized products parts on the cart page', 'csd'),
            'options' => array(
                'thumbnail' => esc_html__('Thumbnail column', 'csd'),
                'name' => esc_html__('Name column', 'csd'),
            ),
            'class' => 'chosen_select_nostd',
        );

        $download_button = array(
            'title' => __('Download design', 'csd'),
            'name' => 'csd-general-options[csd-download-btn]',
            'default' => '1',
            'type' => 'radio',
            'desc' => __('This option allows you to show/hide the download button on the customization page', 'csd'),
            'options' => array(
                '1' => __('Yes', 'csd'),
                '0' => __('No', 'csd'),
            ),
            'class' => 'chosen_select_nostd',
        );

        $preview_button = array(
            'title' => __('Preview design', 'csd'),
            'name' => 'csd-general-options[csd-preview-btn]',
            'default' => '1',
            'type' => 'radio',
            'desc' => __('This option allows you to show/hide the preview button on the customization page', 'csd'),
            'options' => array(
                '1' => __('Yes', 'csd'),
                '0' => __('No', 'csd'),
            ),
            'class' => 'chosen_select_nostd',
        );

        $user_account_download_button = array(
            'title' => esc_html__('Download design from user account page', 'csd'),
            'name' => 'csd-general-options[csd-user-account-download-btn]',
            'default' => '1',
            'type' => 'radio',
            'desc' => esc_html__('This option allows you to show/hide the download button on user account page', 'csd'),
            'options' => array(
                '1' => esc_html__('Yes', 'csd'),
                '0' => esc_html__('No', 'csd'),
            ),
            'class' => 'chosen_select_nostd',
        );

        $send_attachments = array(
            'title' => esc_html__('Send the design as an attachment', 'csd'),
            'name' => 'csd-general-options[csd-send-design-mail]',
            'default' => '1',
            'type' => 'radio',
            'desc' => esc_html__('This option allows you to send or not the design by mail after checkout', 'csd'),
            'options' => array(
                '1' => esc_html__('Yes', 'csd'),
                '0' => esc_html__('No', 'csd'),
            ),
            'class' => 'chosen_select_nostd',
        );

        $add_to_cart_action = array(
            'title' => esc_html__('Redirect after adding a custom design to the cart?', 'csd'),
            'name' => 'csd-general-options[csd-redirect-after-cart]',
            'default' => '0',
            'type' => 'radio',
            'desc' => esc_html__('This option allows you to define what to do after adding a design to the cart', 'csd'),
            'options' => array(
                '1' => esc_html__('Yes', 'csd'),
                '0' => esc_html__('No', 'csd'),
            ),
            'class' => 'chosen_select_nostd',
        );

        $hide_design_buttons_cart_page = array(
            'title' => esc_html__('Hide design buttons on shop page', 'csd'),
            'name' => 'csd-general-options[csd-hide-btn-shop-pages]',
            'default' => '0',
            'type' => 'radio',
            'desc' => esc_html__('This option allows you to show/hide the design button on the shop page', 'csd'),
            'options' => array(
                '1' => esc_html__('Yes', 'csd'),
                '0' => esc_html__('No', 'csd'),
            ),
            'class' => 'chosen_select_nostd',
        );

        $hide_cart_button_for_custom_products = array(
            'title' => esc_html__('Hide Add to cart button on single product page', 'csd'),
            'name' => 'csd-general-options[csd-hide-cart-button]',
            'default' => '0',
            'type' => 'radio',
            'desc' => esc_html__('This option allows you to define whether or not you want to hide the add to cart button for custom products on the single products page.', 'csd'),
            'options' => array(
                '1' => esc_html__('Yes', 'csd'),
                '0' => esc_html__('No', 'csd'),
            ),
            'class' => 'chosen_select_nostd',
        );

        $hide_cart_button_shop_page = array(
            'title' => esc_html__('Hide Add to cart button on shop page', 'csd'),
            'name' => 'csd-general-options[csd-hide-cart-shop-pages]',
            'default' => '0',
            'type' => 'radio',
            'desc' => esc_html__('This option allows you to show/hide the Add to cart button for customizable products on the shop page', 'csd'),
            'options' => array(
                '1' => esc_html__('Yes', 'csd'),
                '0' => esc_html__('No', 'csd'),
            ),
            'class' => 'chosen_select_nostd',
        );

        $hide_preview_button = array(
            'title' => esc_html__('Hide preview button on designer page', 'csd'),
            'name' => 'csd-general-options[csd-hide-preview-button]',
            'default' => 'yes',
            'type' => 'radio',
            'desc' => esc_html__('This option allows you to show/hide the preview buttons on designer page', 'csd'),
            'options' => array(
                'yes' => esc_html__('Yes', 'csd'),
                'no' => esc_html__('No', 'csd'),
            ),
            'class' => 'chosen_select_nostd',
        );

        $hide_requirements_notices = array(
            'title' => __('Hide requirements notices', 'csd'),
            'name' => 'csd-general-options[hide-requirements-notices]',
            'default' => '0',
            'type' => 'radio',
            'desc' => __('This option allows you to define whether or not you want to hide the requirement notice.', 'csd'),
            'options' => array(
                '0' => __('No', 'csd'),
                '1' => __('Yes', 'csd'),
            ),
            'row_class' => 'csd_hide_requirements',
            'class' => 'chosen_select_nostd',
        );



        $general_options_end = array('type' => 'sectionend');

        array_push($options, $general_options_begin);
        array_push($options, $customizer_page);
        array_push($options, $content_filter);
        array_push($options, $customizer_cart_display);
        array_push($options, $download_button);
        array_push($options, $preview_button);
        array_push($options, $user_account_download_button);
        array_push($options, $send_attachments);
        array_push($options, $add_to_cart_action);
        array_push($options, $hide_cart_button_for_custom_products);
        array_push($options, $hide_design_buttons_cart_page);
        array_push($options, $hide_cart_button_shop_page);
        array_push($options, $hide_preview_button);
        array_push($options, $general_options_end);

        $options = apply_filters('csd_general_options', $options);
        $allowed_tags = csd_allowed_tags();
        $raw_output = o_admin_fields($options);
        echo wp_kses($raw_output, $allowed_tags);
    }

    /**
     * Builds the output settings options
     *
     * @return array Settings
     */
    private function csd_get_output_settings() {
        $options = array();

        $cmyk_attr = array();
        if (!class_exists('Imagick')) {
            $cmyk_attr = array(
                'disabled' => '',
            );
        }
        $cmyk_profil = array(
            'title' => esc_html__('CMYK profile (when CMYK mode is enabled)', 'csd'),
            'name' => 'csd-output-options[csd-cmyk-profil]',
            'type' => 'file',
            'desc' => esc_html__('This option allows you to set your own CMYK profile to use during the output conversion.', 'csd') . '<br>' .
            esc_html__('Disabled if the Imagemagick extension is not installed and active.', 'csd'),
            'custom_attributes' => array_merge($cmyk_attr, array('alt' => 'color profil')),
            'set' => esc_html__('Set profile', 'csd'),
            'remove' => esc_html__('Remove profile', 'csd'),
        );
        $design_composition = array(
            'title' => esc_html__('Design Composition', 'csd'),
            'name' => 'csd-output-options[design-composition]',
            'default' => 'no',
            'type' => 'radio',
            'desc' => esc_html__('This option allows you to display or not design composition in the order ', 'csd'),
            'options' => array(
                'no' => esc_html__('No', 'csd'),
                'yes' => esc_html__('Yes', 'csd'),
            ),
            'class' => 'chosen_select_nostd',
        );
        $use_order_id = array(
            'title' => esc_html__('Use order ID as zip name', 'csd'),
            'name' => 'csd-output-options[use-order-id-as-zip-name]',
            'default' => 'no',
            'type' => 'radio',
            'desc' => esc_html__('This option will name each output archive using the order ID.', 'csd'),
            'options' => array(
                'no' => esc_html__('No', 'csd'),
                'yes' => esc_html__('Yes', 'csd'),
            ),
            'class' => 'chosen_select_nostd',
        );

        $output_options_begin = array(
            'type' => 'sectionbegin',
            'id' => 'csd-output-options',
            'title' => esc_html__('Output Settings', 'csd'),
            'table' => 'options',
        );

        $output_options_end = array(
            'type' => 'sectionend',
            'id' => 'csd-output-options',
        );

        array_push($options, $output_options_begin);
        //array_push($options, $cmyk_profil);
        array_push($options, $design_composition);
        array_push($options, $use_order_id);
        array_push($options, $output_options_end);

        $allowed_tags = csd_allowed_tags();
        $raw_output = o_admin_fields($options);
        echo wp_kses($raw_output, $allowed_tags);
    }

    public function notify_customization_page_missing() {
        global $csd_settings;
        $options = $csd_settings['csd-general-options'];
        $hide_notices = get_proper_value($options, 'hide-requirements-notices', false);
        $csd_page_id = $options['csd_page_id'];
        $settings_url = get_bloginfo('url') . '/wp-admin/admin.php?page=csd-manage-settings';
        if (!class_exists('WooCommerce')) {
            echo '<div class="error">
                   <p><b>CSD- The ultimate woocommerce Signs Designers: </b>WooCommerce is not installed on your website. You will not be able to use the features of the plugin.</p>
                </div>';
        } elseif (!class_exists('Imagick')) {
            echo '<div class="error">
                   <p><b>CSD- The ultimate woocommerce Signs Designers: </b>Imagick is not installed on your website. You will not be able to use the features of the plugin.</p>
                </div>';
        } else {
            if (empty($csd_page_id)) {
                echo '<div class="error"> 
                   <p><b>CSD - The ultimate woocommerce Signs Designers: </b>The design page is not defined. Please configure it in <a href="' . esc_html($settings_url) . '">plugin settings page</a>: .</p>
                 </div>';
            }
            if (!extension_loaded('zip') && !$hide_notices) {
                echo '<div class="error">
                   <p><b>CSD - The ultimate woocommerce Signs Designers: </b>ZIP extension not loaded on this server. You won\'t be able to generate zip outputs.</p>
                </div>';
            }
        }
    }

    /**
     * Builds the social networks settings options
     *
     * @return array Settings
     */
    private function csd_get_license_settings() {
        $options = array();
        $license_begin = array(
            'type' => 'sectionbegin',
            'id' => 'csd-license',
            'title' => esc_html__('License Settings', 'csd'),
            'table' => 'options',
        );

        $license_end = array(
            'type' => 'sectionend',
            'id' => 'csd-license',
        );
        $license_key = array(
            'title' => esc_html__('License Key', 'csd'),
            'desc' => __('The license key can be found in the order receipt email received after your purchase or <a href="https://orionorigin.com/my-account/orders/" target="_blank">here</a>.', 'csd'),
            'name' => 'csd-license-options[license-key]',
            'type' => 'text',
            'default' => '',
        );

        array_push($options, $license_begin);
        array_push($options, $license_key);
        array_push($options, $license_end);

        $allowed_tags = csd_allowed_tags();
        $raw_output = o_admin_fields($options);
        echo wp_kses($raw_output, $allowed_tags);
    }

    /**
     * 
     *
     * @return array Settings
     */
    private function csd_get_upload_settings() {
        $options = array();
        $upload_begin = array(
            'type' => 'sectionbegin',
            'id' => 'csd-upload',
            'title' => esc_html__('Upload Settings', 'csd'),
            'table' => 'options',
        );

        $upload_end = array(
            'type' => 'sectionend',
            'id' => 'csd-upload',
        );
        $min_upload_w = array(
            'title' => __('Uploads min width (px)', 'csd'),
            'desc' => __('Uploaded images minimum width', 'csd'),
            'name' => 'csd-upload-options[csd-min-upload-width]',
            'type' => 'text',
            'default' => '',
        );

        $min_upload_h = array(
            'title' => __('Uploads min height (px)', 'csd'),
            'desc' => __('Uploaded images minimum height', 'csd'),
            'name' => 'csd-upload-options[csd-min-upload-height]',
            'type' => 'text',
            'default' => '',
        );

        $upl_extensions = array(
            'title' => __('Allowed uploads extensions', 'csd'),
            'name' => 'csd-upload-options[csd-upl-extensions]',
            'default' => array('jpg', 'jpeg', 'png', 'gif', 'bmp', 'svg', 'psd', 'eps', 'pdf', 'webp'),
            'type' => 'multiselect',
            'desc' => __('Allowed extensions for uploads', 'csd'),
            'options' => array(
                'jpg' => __('jpg', 'csd'),
                'jpeg' => __('jpeg', 'csd'),
                'png' => __('png', 'csd'),
                'gif' => __('gif', 'csd'),
                'bmp' => __('bmp', 'csd'),
                'svg' => __('svg', 'csd'),
                'psd' => __('psd', 'csd'),
                'eps' => __('eps', 'csd'),
                'pdf' => __('pdf', 'csd'),
                'webp' => __('webp', 'csd'),
            ),
        );

        array_push($options, $upload_begin);
        array_push($options, $min_upload_w);
        array_push($options, $min_upload_h);
        array_push($options, $upl_extensions);
        array_push($options, $upload_end);

        $allowed_tags = csd_allowed_tags();
        $raw_output = o_admin_fields($options);
        echo wp_kses($raw_output, $allowed_tags);
    }

    //Afficher la notice d'activation de la license.
    function csd_get_license_activation_notice() {
        $options = get_option('csd-license-options');
        if (!isset($options['license-key']) || '' === $options['license-key'] || is_null($options['license-key'])) {
            $url = admin_url('admin.php?page=csd-manage-settings');
            ?>
            <div class="notice notice-error">
                <p><?php _e('<strong>Custom Signs Designer for WooCommerce</strong>: No license key provided. Please, set your license key in the "license" tab <a href="' . $url . '"> here</a>.', 'csd'); ?></p>
                <p></p>
            </div>
            <?php
        }

        if ($_SERVER['REMOTE_ADDR'] != '127.0.0.1' && $_SERVER['REMOTE_ADDR'] != '::1') {
            if (!get_option('csd-license-key') && is_csd_admin_screen()) {
                ?>
                <div class="notice notice-error">
                    <p><?php _e('<strong>Custom Signs Designer for WooCommerce</strong>: You have not activated your license yet. Please activate it in order to get the plugin working.', 'csd'); ?></p>
                    <a class="button" id="csd-activate"><?php _e("Activate", "csd"); ?></a><img style="display:none;" id="csd-spinner" src="<?php echo CSD_URL . 'admin/images/spinner.gif'; ?> ">
                    <p></p>
                    <div id="csd-license-message"></div>
                </div>
                <?php
            }
        }
    }

//Fonction chargée d'activer la license.
    function csd_activate_license() {
        $options = get_option('csd-license-options');
        if (isset($options['license-key']) && '' !== $options['license-key']) {
            $license_key = $options['license-key'];
            $site_url = get_site_url();
            $code = $_POST['code'];
            $item_id = CSD_PLUGIN_ID;
            $url = "https://orionorigin.com/service/olicenses/v1/license/?license-key=" . urlencode($license_key) . "&siteurl=" . urlencode($site_url) . "&id=" . $item_id . "&code=" . $code;
            $args = array('timeout' => 60);
            $response = wp_remote_get($url, $args);
            if (is_wp_error($response)) {
                $error_message = $response->get_error_message();
                echo "Something went wrong: $error_message";
                die();
            }
            if (isset($response["body"])) {
                $answer = $response["body"];
            }

            if (is_array(json_decode($answer, true))) {
                $data = json_decode($answer, true);
                $success = update_option('csd-license-key', $data['key']);
                echo "200";
            } else {
                echo $answer;
            }
        } else {
            $url = admin_url('admin.php?page=csd-manage-settings');
            echo ("License key not found. Please, set your license key in the 'license' tab <a href=" . $url . ">here</a>. ");
        }

        die();
    }

    /**
     * Function to verify license validity
     */
    public function csd_verify_validity() {
        $options = get_option('csd-license-options');
        if (isset($options['license-key']) && $options['license-key'] != "") {
            $license_key = $options['license-key'];
            $site_url = get_site_url();
            $item_id = CSD_PLUGIN_ID;
            $url = "https://orionorigin.com/service/olicenses/v1/checking/?license-key=" . urlencode($license_key) . "&siteurl=" . urlencode($site_url) . "&id=" . $item_id;
            $args = array('timeout' => 60);
            $response = wp_remote_get($url, $args);

            if (!is_wp_error($response)) {
                if (isset($response["body"]) && intval($response["body"]) != 200) {
                    delete_option("csd-license-key");
                }
            }
        } else {
            if (get_option("csd-license-key")) {
                delete_option("csd-license-key");
            }
        }
    }

    /**
     * Build the about page
     */
    public function csd_get_about_page() {
        ?>
        <div id='csd-about-page'>
            <div class="wrap">
                <div id="features-wrap">
                    <h2 class="feature-h2"><?php _e('Getting Started', 'csd'); ?></h2>
                    <div class="list-posts-content">
                        <div class="o-wrap o-features xl-gutter-8">
                            <div class="o-col col xl-1-3">
                                <img class="vc_single_image-img " src="<?php echo CSD_URL . "admin/images/features/" ?>custom-product.svg">
                                <h3 class="csd-title"><?php _e('CREATE A CUSTOM PRODUCT', 'csd'); ?></h3>
                                <p><?php _e('This tutorial is a step by step guide to teach you how to create a custom product on a woocommerce store using the Custom Signs Designer for WooCommerce plugin.', 'csd'); ?></p>
                                <a href="https://orionorigin.com/lettering-and-signs-designer/tutorial/how-to-create-a-woocommerce-custom-sign-or-lettering-product/" class="button" target="_blank"><?php _e('Learn More', 'csd'); ?></a>
                            </div>
                            <div class="o-col col xl-1-3">
                                <img class="vc_single_image-img " src="<?php echo CSD_URL . "admin/images/features/" ?>pricing-csd-01.svg">
                                <h3 class="csd-title"><?php _e('CREATE A CUSTOM PRICING STRATEGY', 'csd'); ?></h3>
                                <p><?php _e('The goal of this tutorial is to show you how you can create a custom pricing strategy for your products using the lettering and signs plugin.', 'csd'); ?></p>
                                <a href="https://orionorigin.com/lettering-and-signs-designer/tutorial/how-to-setup-a-custom-pricing-strategy-for-the-lettering-and-signs-plugin-for-woocommerce/" class="button" target="_blank"><?php _e('Learn More', 'csd'); ?></a>
                            </div>
                            <div class="o-col col xl-1-3">
                                <img class="vc_single_image-img " src="<?php echo CSD_URL . "admin/images/features/" ?>add-fonts.svg">
                                <h3 class="csd-title"><?php _e('HOW TO MANAGE FONTS', 'csd'); ?></h3>
                                <p><?php _e('Fonts are one of the most basic features in online product customization and yet one of the most critical ones. CSD does not only allow you to use google fonts.', 'csd'); ?></p>
                                <a href="https://orionorigin.com/lettering-and-signs-designer/tutorial/how-to-manage-fonts-in-the-lettering-and-signs-plugin-for-woocommerce/" class="button" target="_blank"><?php _e('Learn More', 'csd'); ?></a>
                            </div>
                        </div>
                    </div>
                </div>

            </div>  
        </div>
        <?php
    }

    /**
     * Runs the new version check and upgrade process
     * @return \CSD_Updater
     */
    function csd_get_updater() {
        do_action('csd_before_init_updater');
        require_once( CSD_DIR . DIRECTORY_SEPARATOR . 'includes' . DIRECTORY_SEPARATOR . 'updaters' . DIRECTORY_SEPARATOR . 'class-csd-updater.php' );
        $updater = new CSD_Updater();
        $updater->init();
        require_once( CSD_DIR . DIRECTORY_SEPARATOR . 'includes' . DIRECTORY_SEPARATOR . 'updaters' . DIRECTORY_SEPARATOR . 'class-csd-updating-manager.php' );
        $updater->setUpdateManager(new CSD_Updating_Manager(CSD_VERSION, $updater->versionUrl(), CSD_MAIN_FILE));
        do_action('csd_after_init_updater');
        return $updater;
    }

}
