<?php
/**
 * Class csd config.
 *
 * @link       https://designersuiteforwp.com
 * @since      1.0.0
 *
 * @package    Csd
 * @subpackage Csd/public
 */

/**
 * The
 *
 * @package    Csd
 * @subpackage Csd/public
 * @author     ORION <support@designersuiteforwp.com>
 */
class CSD_Clipart {

    public $id;
    public $settings;

    public function __construct( $clipart_id = false ) {
        if ( $clipart_id ) {
            $this->id       = $clipart_id;
            $this->settings = get_post_meta( $clipart_id, 'csd-metas', true );
        }
    }

   /**
	 * Register the csd-cliparts postype.
	 */
	public function csd_register_cpt_cliparts() {

		$labels = array(
			'name'               => __( 'Cliparts', 'csd' ),
			'singular_name'      => __( 'Cliparts', 'csd' ),
			'add_new'            => __( 'New cliparts group', 'csd' ),
			'add_new_item'       => __( 'New cliparts group', 'csd' ),
			'edit_item'          => __( 'Edit cliparts group', 'csd' ),
			'new_item'           => __( 'New cliparts group', 'csd' ),
			'view_item'          => __( 'View group', 'csd' ),
			'not_found'          => __( 'No cliparts group found', 'csd' ),
			'not_found_in_trash' => __( 'No cliparts group in the trash', 'csd' ),
			'menu_name'          => __( 'Cliparts', 'csd' ),
		);

		$args = array(
			'labels'              => $labels,
			'hierarchical'        => false,
			'description'         => 'Cliparts for the product designer',
			'supports'            => array( 'title' ),
			'public'              => false,
			'menu_icon'           => 'dashicons-images-alt',
			'show_ui'             => true,
			'show_in_menu'        => false,
			'show_in_nav_menus'   => false,
			'publicly_queryable'  => false,
			'exclude_from_search' => true,
			'has_archive'         => false,
			'query_var'           => false,
			'can_export'          => true,
		);

		register_post_type( 'csd-cliparts', $args );
	}

	/**
	 * Create the csd-cliparts metabox.
	 */
	public function csd_get_cliparts_metabox() {

		$screens = array( 'csd-cliparts' );

		foreach ( $screens as $screen ) {

			add_meta_box(
				'csd-cliparts-box',
				__( 'Cliparts', 'csd' ),
				array( $this, 'get_cliparts_page' ),
				$screen
			);
		}
	}

	/**
	 * Get cliparts page.
	 */
	public function get_cliparts_page() {
		wp_enqueue_media();
		?>
		<div class='block-form'>
		<?php
		$begin = array(
			'type' => 'sectionbegin',
			'id'   => 'cliparts-container',
		);

		$price = array(
			'title' => __( 'Price', 'csd' ),
			'name'  => 'price',
			'type'  => 'number',
		);

		$name = array(
			'title' => __( 'Name', 'csd' ),
			'name'  => 'name',
			'type'  => 'text',
		);

		$c_image = array(
			'title'  => __( 'Icon', 'csd' ),
			'name'   => 'id',
			'type'   => 'image',
			'set'    => 'Set',
			'remove' => 'Remove',
			'desc'   => __( 'Component icon', 'csd' ),
		);

		$cliparts = array(
			'title'           => __( 'Components', 'csd' ),
			'name'            => 'csd-cliparts-data',
			'type'            => 'repeatable-fields',
			'fields'          => array( $c_image, $name, $price ),
			'desc'            => __( 'Cliparts', 'csd' ),
			'ignore_desc_col' => true,
			'class'           => 'striped',
			'add_btn_label'   => __( 'Add clipart', 'csd' ),
		);

		$end      = array( 'type' => 'sectionend' );
		$settings = apply_filters(
			'csd_cliparts_settings',
			array(
				$begin,
				$cliparts,
				$end,
			)
		);
		echo o_admin_fields( $settings );
		global $o_row_templates;
		?>
		</div>
		<input type="hidden" name="securite_nonce" value="<?php echo esc_html( wp_create_nonce( 'securite-nonce' ) ); ?>"/>
		<button class="button csd-add-cliparts"><?php esc_html_e( 'Add multiple cliparts', 'csd' ); ?></button>
		<br><br><strong><?php esc_html_e( 'Note: ', 'csd' ); ?></strong><?php esc_html_e( 'Please hold Ctrl or Cmd to select multiple cliparts at once from the medias popup.', 'csd' ); ?> 
		<script>
			var o_rows_tpl =<?php echo wp_json_encode( $o_row_templates ); ?>;
		</script>
		<?php
	}

	/**
     * Re order the metaboxes
     *
     * @param array $order Array of ordered elements.
     * @return string
     */
    public function csd_get_metabox_order( $order ) {
        $order['advanced'] = 'csd-cliparts-box,submitdiv';
        return $order;
    }


   	/**
     *  Force the layout in the config to be one column
     *
     * @param array $columns Array of columns number.
     * @return int
     */
    public function csd_get_layout_columns( $columns ) {
        $columns['csd-config'] = 1;
        return $columns;
    }

   /**
	 * Save the clipart.
	 *
	 * @param int $post_id The post id.
	 */
	public function csd_save_cliparts( $post_id ) {
		if ( isset( $_POST['securite_nonce'] ) ) {
			if ( wp_verify_nonce( wp_unslash( sanitize_key( $_POST['securite_nonce'] ) ), 'securite-nonce' ) ) {
				if ( isset( $_POST['csd-cliparts-data'] ) ) {
					update_post_meta( $post_id, 'csd-cliparts-data', wp_unslash( $_POST['csd-cliparts-data'] ) );
				}
			}
		}
	}

	/**
	 * Set default category when no category is chosen.
	 *
	 * @param type $post_id the post id.
	 * @param type $post the post.
	 */
	public function set_default_object_terms( $post_id, $post ) {
		if ( 'publish' === $post->post_status ) {
			$defaults   = array( 'default', 'default' );
			$taxonomies = get_object_taxonomies( $post->post_type );
			foreach ( (array) $taxonomies as $taxonomy ) {
				if ( 'csd-categorie-cliparts' === $taxonomy ) {
					$terms = wp_get_post_terms( $post_id, $taxonomy );
					if ( empty( $terms ) ) {
						wp_set_object_terms( $post_id, $defaults, $taxonomy );
					}
				}
			}
		}
	}

}
