<?php
/**
 * Class csd config.
 *
 * @link       https://designersuiteforwp.com
 * @since      1.0.0
 *
 * @package    Csd
 * @subpackage Csd/public
 */

/**
 * The
 *
 * @package    Csd
 * @subpackage Csd/public
 * @author     ORION <support@designersuiteforwp.com>
 */
class CSD_Config {

    public $id;
    public $settings;

    public function __construct( $config_id = false ) {
        if ( $config_id ) {
            $this->id       = $config_id;
            $this->settings = get_post_meta( $config_id, 'csd-metas', true );
        }
    }

    /**
     * Register le custom post type
     */
    public function csd_register_cpt_config() {

        $labels = array(
            'name'               => _x( 'Configurations', 'csd' ),
            'singular_name'      => _x( 'Configurations', 'csd' ),
            'add_new'            => _x( 'New configuration', 'csd' ),
            'add_new_item'       => _x( 'New configuration', 'csd' ),
            'edit_item'          => _x( 'Edit configuration', 'csd' ),
            'new_item'           => _x( 'New configuration', 'csd' ),
            'view_item'          => _x( 'View configuration', 'csd' ),
            'not_found'          => _x( 'No configuration found', 'csd' ),
            'not_found_in_trash' => _x( 'No configuration in the trash', 'csd' ),
            'menu_name'          => _x( 'Product Designer', 'csd' ),
            'all_items'          => _x( 'Configurations', 'csd' ),
        );

        $args = array(
            'labels'              => $labels,
            'hierarchical'        => false,
            'description'         => 'Configurations',
            'supports'            => array( 'title' ),
            'public'              => false,
            'show_ui'             => true,
            'show_in_menu'        => false,
            'show_in_nav_menus'   => false,
            'publicly_queryable'  => false,
            'exclude_from_search' => true,
            'has_archive'         => false,
            'query_var'           => false,
            'can_export'          => true,
        );

        register_post_type( 'csd-config', $args );
    }

    /**
     * Builds the cpt metaboxes
     */
    public function csd_get_config_metabox() {

        $screens = array( 'csd-config' );

        foreach ( $screens as $screen ) {

            add_meta_box(
                'csd-metas-skin',
                esc_html__( 'Skin', 'csd' ),
                array( $this, 'csd_get_config_skin_metabox' ),
                $screen
            );

            add_meta_box(
                'csd-metas-canvas',
                esc_html__( 'Canvas', 'csd' ),
                array( $this, 'csd_get_config_canvas_metabox' ),
                $screen
            );

            add_meta_box(
                'csd-metas-text',
                esc_html__( 'Text', 'csd' ),
                array( $this, 'csd_get_config_text_metabox' ),
                $screen
            );

            add_meta_box(
                'csd-metas-fonts',
                esc_html__( 'Fonts', 'csd' ),
                array( $this, 'csd_get_config_fonts_metabox' ),
                $screen
            );

            add_meta_box(
                'csd-metas-clipart',
                esc_html__( 'Cliparts', 'csd' ),
                array( $this, 'csd_get_config_cliparts' ),
                $screen
            );

            add_meta_box(
                'csd-metas-output',
                esc_html__( 'Output settings', 'csd' ),
                array( $this, 'csd_get_config_output_metabox' ),
                $screen
            );

            add_meta_box(
                'csd-metas-pricing-rules',
                esc_html__( 'Pricing rules', 'csd' ),
                array( $this, 'csd_get_config_pricing_metabox' ),
                $screen
            );

            if ( class_exists( 'Ofb' ) ) {
                add_meta_box(
                    'csd-metas-form-builder',
                    __( 'Form Builder', 'csd' ),
                    array( $this, 'csd_get_config_form_builder' ),
                    $screen
                );
            }
            
            apply_filters("add-scoth-cj-config-metabox", $screen);
        }
    }

    public function csd_config_order_metabox( $order ) {
        return array(
            'normal' => join(
                ',',
                array(
                    'csd-metas-skin',
                    'csd-metas-canvas',
                    'csd-metas-text',
                    'csd-metas-fonts',
                    'csd-metas-clipart',
                    'csd-metas-output',
                    'csd-metas-pricing-rules',
                    'csd-metas-form-builder',
                    'submitdiv',
                )
            ),
        );
    }

    /**
     * Config skin settings
     */
    public function csd_get_config_skin_metabox() {
        $begin     = array(
            'type' => 'sectionbegin',
            'id'   => 'csd-skin-container',
        );
        $skins_arr = apply_filters(
            'csd_configuration_skins',
            array(
                'CSD_Default_Skin' => __( 'Default', 'csd' ),
            )
        );

        $skins    = array(
            'title'   => __( 'Skin', 'csd' ),
            'name'    => 'csd-metas[skin]',
            'type'    => 'select',
            'options' => $skins_arr,
            'default' => 'CSD_Default_Skin',
            'desc'    => __( 'Editor look and feel.', 'csd' ),
        );
        $end      = array( 'type' => 'sectionend' );
        $settings = array(
            $begin,
            $skins,
            $end,
        );
        // Escaping output.
        $allowed_tags = csd_allowed_tags();
        $raw_output   = o_admin_fields( $settings );
        echo wp_kses( $raw_output, $allowed_tags );
        global $o_row_templates;
        ?>
        <script>
            var o_rows_tpl =<?php echo wp_json_encode( $o_row_templates ); ?>;
        </script>
        <?php
    }

    /**
     * Builds the canvas meta box
     */
    public function csd_get_config_canvas_metabox() {
        wp_enqueue_media();
        wp_nonce_field( basename( __FILE__ ), 'csd_config_nonce' );
        $args  = array( 'post_status' => 'publish' );
        $pages = get_pages( $args );
        // Escaping output.
        $allowed_tags = csd_allowed_tags();
        $pages_arr    = array();

        foreach ( $pages as $page ) {
            $pages_arr[ $page->ID ] = $page->post_title;
        }
        $begin = array(
            'type' => 'sectionbegin',
            'id'   => 'csd-canvas-settings',
        );

        $default_unit = apply_filters("add-scoth-cj-config-add-unit",
                array(
                    'title'     => esc_html__( 'Default Unit', 'csd' ),
                    'name'      => 'csd-metas[default-unit]',
                    'id' => 'default_unit',
                    'desc'      => esc_html__( 'Choose the base unit in which the dimensions of the canvas and the size of the text will be converted when displayed on the personalization page.', 'csd' ),
                    'type'      => 'select',
                    'options' => array(
                        'Feet'   => esc_html__( 'Feet', 'csd' ),
                        'Cm'   => esc_html__( 'Centimeter', 'csd' ),
                        'Px'   => esc_html__( 'Pixels', 'csd' ),
                        'Inch' => esc_html__( 'Inch', 'csd' ),
                    ),
                    'default' => 'Px',
                )
        );

        $max_width  = array(
            'title'   => esc_html__( 'Default width', 'csd' ),
            'name'    => 'csd-metas[canvas-w]',
            'type'    => 'number',
            'desc'    => esc_html__( 'in pixels. Canvas max width. Leave this field empty to use the value defined in the global settings.', 'csd' ),
            'id' => 'canvas_w',
            'default' => 30,
        );
        $max_height = array(
            'title'   => esc_html__( ' Default height', 'csd' ),
            'name'    => 'csd-metas[canvas-h]',
            'type'    => 'number',
            'desc'    => esc_html__( 'in pixels. Canvas max width. Leave this field empty to use the value defined in the global settings.', 'csd' ),
            'id' => 'canvas_h',
            'default' => 30,
        );

        $default_canvas_dimension = array(
            'title'     => esc_html__( 'Default signs dimensions', 'csd' ),
            'type'      => 'groupedfields',
            'desc'      => __( 'Width and Height of the design area in the browser. <span style="color: red">It can be smaller than the expected output files in order to fit inside the website pages but need to be a scaled version of the expected output.</span>', 'csd' ),
            'fields'    => array( $max_width, $max_height),
            'row_class' => 'canvas_dimensions',
        );

        $text_min = array(
            'title'   => esc_html__( 'Min font size', 'csd' ),
            'name'    => 'csd-metas[min-size]',
            'type'    => 'number',
            'desc'    => esc_html__( 'The minimum font size that the user can define.', 'csd' ),
            'default' => 12,
        );
        $text_max = array(
            'title'   => esc_html__( 'Max font size', 'csd' ),
            'name'    => 'csd-metas[max-size]',
            'type'    => 'number',
            'desc'    => esc_html__( 'The maximum font size that the user can define.', 'csd' ),
            'default' => 60,
        );

        $text_dimensions = array(
            'title'     => esc_html__( 'Text dimensions limits', 'csd' ),
            'type'      => 'groupedfields',
            'desc'      => esc_attr_e( 'min and max values that customer can define as text font size. ', 'csd' ),
            'fields'    => array( $text_min, $text_max),
            'row_class' => 'dimensions-font',
        );

        $bg_type = array(
            'title'     => esc_html__( 'Background color', 'csd' ),
            'name'      => 'csd-metas[bg-type]',
            'default'   => 'colors',
            'type'      => 'radio',
            'desc'      => esc_html__( 'How the customer will set the canvas background color.', 'csd' ),
            'options'   => array(
                'colors'      => esc_html__( 'Colors Palette', 'csd' ),
                'colorpicker' => esc_html__( 'Colors Picker', 'csd' ),
            ),
            'row_class' => 'csd-bg-type',
        );

        $price = array(
            'title' => esc_html__( 'Price', 'csd' ),
            'name'  => 'price',
            'type'  => 'number',
        );

        $hex_code = array(
            'title' => esc_html__( 'HEX color code', 'csd' ),
            'name'  => 'hex',
            'type'  => 'color',
        );

        $hex_name = array(
            'title' => esc_html__( 'HEX color name', 'csd' ),
            'name'  => 'name',
            'type'  => 'text',
        );

        $default_hex_code = array(
            'title'     => esc_html__( 'Default color', 'csd' ),
            'name'      => 'csd-metas[default-hex-code]',
            'type'      => 'color',
            'default'   => '#000000',
            'row_class' => 'show-if-canvas-colorpicker',
        );

        $bg_colors = array(
            'title'         => esc_html__( 'Palette', 'csd' ),
            'name'          => 'csd-metas[bg-colors-palette]',
            'type'          => 'repeatable-fields',
            'fields'        => array( $hex_code, $price, $hex_name ),
            'desc'          => esc_html__( 'Background color palette (if you are not sure about the color name, leave the Hex color name field empty. )', 'csd' ),
            'class'         => 'striped',
            'add_btn_label' => esc_html__( 'Add color', 'csd' ),
            'row_class'     => 'show-if-canvas-colors',
        );

        $end        = array( 'type' => 'sectionend' );
        $settings   = array(
            $begin,
            $default_unit,
            $default_canvas_dimension,
            // $text_dimensions,
            $bg_type,
            $bg_colors,
            $default_hex_code,
            $end,
        );
        $raw_output = o_admin_fields( apply_filters("add-scoth-cj-config-canvas-metabox", $settings) );
        echo wp_kses( $raw_output, $allowed_tags );
        global $o_row_templates;
        ?>

        <script>
            var o_rows_tpl =<?php echo wp_json_encode( $o_row_templates ); ?>;
        </script>
        <?php
    }

    /**
     * Builds the text meta box
     */
    public function csd_get_config_text_metabox() {
        wp_enqueue_media();

        $args      = array( 'post_status' => 'publish' );
        $pages     = get_pages( $args );
        $pages_arr = array();

        foreach ( $pages as $page ) {
            $pages_arr[ $page->ID ] = $page->post_title;
        }
        $begin = array(
            'type' => 'sectionbegin',
            'id'   => 'csd-text-settings',
        );

        $default_font_size = array(
            'title' => esc_html__( 'Default font size', 'csd' ),
            'name'      => 'csd-metas[text-font-size]',
            'default'   => 30,
            'type'  => 'number',
            'desc'      => esc_html__( 'Default font size in the text section.', 'csd' ),
        );

        $bg_type = array(
            'title'     => esc_html__( 'Color Type', 'csd' ),
            'name'      => 'csd-metas[text-color-type]',
            'default'   => 'colors',
            'type'      => 'radio',
            'desc'      => esc_html__( 'How the customer will set the text color.', 'csd' ),
            'options'   => array(
                'colors'      => esc_html__( 'Colors Palette', 'csd' ),
                'colorpicker' => esc_html__( 'Colors Picker', 'csd' ),
            ),
            'row_class' => 'csd-text-color-type',
        );

        $price = array(
            'title' => esc_html__( 'Price', 'csd' ),
            'name'  => 'price',
            'type'  => 'number',
        );

        $hex_code = array(
            'title' => esc_html__( 'HEX color code', 'csd' ),
            'name'  => 'hex',
            'type'  => 'color',
        );

        $hex_name = array(
            'title' => esc_html__( 'HEX color name', 'csd' ),
            'name'  => 'name',
            'type'  => 'text',
        );

        $default_hex_code = array(
            'title'     => esc_html__( 'Default color', 'csd' ),
            'name'      => 'csd-metas[default-text-color]',
            'type'      => 'color',
            'default'   => '#ffffff',
            'row_class' => 'show-if-text-colorpicker',
        );

        $bg_colors = array(
            'title'         => esc_html__( 'Palette', 'csd' ),
            'name'          => 'csd-metas[text-colors-palette]',
            'type'          => 'repeatable-fields',
            'fields'        => array( $hex_code, $price, $hex_name ),
            'desc'          => esc_html__( 'Text color palette (if you are not sure about the color name, leave the Hex color name field empty. )', 'csd' ),
            'class'         => 'striped',
            'add_btn_label' => esc_html__( 'Add color', 'csd' ),
            'row_class'     => 'show-if-text-colors',
        );

        $end      = array( 'type' => 'sectionend' );
        $settings = array(
            $begin,
            $default_font_size,
            $bg_type,
            $bg_colors,
            $default_hex_code,
            $end,
        );

        // Escaping output.
        $allowed_tags = csd_allowed_tags();
        $raw_output   = o_admin_fields( $settings );
        echo wp_kses( $raw_output, $allowed_tags );
        global $o_row_templates;
        ?>
        <script>
            var o_rows_tpl =<?php echo wp_json_encode( $o_row_templates ); ?>;
        </script>
        <?php
    }

    /**
     * Builds the fonts meta box
     */
    public function csd_get_config_fonts_metabox() {
        wp_enqueue_media();

        $args      = array( 'post_status' => 'publish' );
        $pages     = get_pages( $args );
        $pages_arr = array();

        foreach ( $pages as $page ) {
            $pages_arr[ $page->ID ] = $page->post_title;
        }
        $begin = array(
            'type' => 'sectionbegin',
            'id'   => 'csd-fonts-settings',
        );

        $use_global_fonts = array(
            'title'     => esc_html__( 'Use global fonts', 'csd' ),
            'name'      => 'csd-metas[global-fonts]',
            'default'   => 'yes',
            'desc'      => esc_html__( 'Available fonts to use when designing.', 'csd' ),
            'type'      => 'radio',
            'options'   => array(
                'no'  => esc_html__( 'No', 'csd' ),
                'yes' => esc_html__( 'Yes', 'csd' ),
            ),
            'row_class' => 'csd-fonts-type',
        );

        $csd_fonts    = csd_refactor_fonts( get_option( 'csd-fonts' ) );
        $chosen_fonts = array(
            'title'             => esc_html__( 'Selected fonts', 'csd' ),
            'name'              => 'csd-metas[selected-fonts]',
            'desc'              => esc_html__( 'Please select the fonts that should be available for use.', 'csd' ),
            'type'              => 'select',
            'options'           => $csd_fonts,
            'row_class'         => 'show-if-fonts-global-no',
            'id'                => 'csd_fonts_selector',
            'custom_attributes' => array(
                'multiple' => 'multiple',
            ),
        );

        $end      = array( 'type' => 'sectionend' );
        $settings = array(
            $begin,
            $use_global_fonts,
            $chosen_fonts,
            $end,
        );

        // Escaping output.
        $allowed_tags = csd_allowed_tags();
        $raw_output   = o_admin_fields( $settings );
        echo wp_kses( $raw_output, $allowed_tags );
        global $o_row_templates;
        ?>
        <script>
            var o_rows_tpl =<?php echo wp_json_encode( $o_row_templates ); ?>;
        </script>
        <?php
    }

    /**
     * Get parts description page.
     */
    public function csd_get_config_cliparts() {
        $begin = array(
            'type' => 'sectionbegin',
            'id'   => 'csd-cliparts-settings',
        );
                
            $use_global_cliparts = array(
            'title'     => esc_html__( 'Use global cliparts', 'csd' ),
            'name'      => 'csd-metas[global-cliparts][use-global-cliparts]',
            'default'   => 'yes',
            'desc'      => esc_html__( 'Would you like to use all cliparts defined in CSD > Cliparts or only some of them?', 'csd' ),
            'type'      => 'radio',
            'options'   => array(
                'no'  => esc_html__( 'No', 'csd' ),
                'yes' => esc_html__( 'Yes', 'csd' ),
            ),
            'row_class' => 'csd-cliparts-group',
        );

            $colors_palette = get_posts(
            array(
                'post_status' => 'publish',
                'post_type'   => 'csd-cliparts',
                'nopaging'    => true,
            )
        );
                
        $options        = array();
        foreach ( $colors_palette as $colors_palette ) {
            $options[ $colors_palette->ID ] = $colors_palette->post_title;
        }
                
        $chosen_cliparts = array(
            'title'             => esc_html__( 'Selected cliparts', 'csd' ),
            'name'              => 'csd-metas[global-cliparts][selected-cliparts]',
            'desc'              => esc_html__( 'Please select the cliparts that should be available for use.', 'csd' ),
            'type'              => 'select',
            'options'           => $options,
            'id'                => 'csd_cliparts_selector',
            'custom_attributes' => array(
                'multiple' => 'multiple',
            ),
        );

        $end      = array( 'type' => 'sectionend' );
        $settings = array(
            $begin,
            $use_global_cliparts,
            $chosen_cliparts,
            $end,
        );
        echo o_admin_fields( $settings );
    }

    /**
     * Builds the pricing metabox
     */
    public function csd_get_config_pricing_metabox() {
        $post_id      = get_the_ID();
        $csd_metas    = get_post_meta( $post_id, 'csd-metas', true );
        $allowed_tags = csd_allowed_tags();

        $params     = array(
            'if_canvas_square_unit' => esc_html__( 'If Canvas square unit (WxH)', 'csd' ),
            'txt_nb_chars'          => esc_html__( 'NB chars in text', 'csd' ),
            'txt_nb_lines'          => esc_html__( 'NB lines in text', 'csd' ),
            'if_selected_font'      => esc_html__( 'If selected font', 'csd' ),
        );
        $first_rule = $this->csd_get_rule_tpl( $params, true );

        $rule_tpl = $this->csd_get_rule_tpl( $params, false );
        ?>
        <div class='csd-rules-table-container'>
            <textarea id='csd-rule-tpl' style='display: none;'>
                <?php echo esc_textarea( $rule_tpl ); ?>
            </textarea>
            <textarea id='csd-first-rule-tpl' style='display: none;'>
                <?php echo esc_textarea( $first_rule ); ?>
            </textarea>
            <?php
            $pricing_rules = array();
            if ( isset( $csd_metas['pricing-rules'] ) ) {
                $pricing_rules = $csd_metas['pricing-rules'];
            }
            if ( is_array( $pricing_rules ) && ! empty( $pricing_rules ) ) {
                $rule_group = 0;
                foreach ( $pricing_rules as $rules_group ) {
                    $rule_index = 0;
                    $rules      = $rules_group['rules'];
                    $a_price    = $rules_group['a_price'];
                    if ( isset( $rules_group['scope'] ) ) {
                        $scope = $rules_group['scope'];
                    } else {
                        $scope = 'item';
                    }
                    ?>
                    <table class="csd-rules-table widefat">
                        <tbody>
                            <?php
                            foreach ( $rules as $rule_arr ) {
                                if ( 0 === $rule_index ) {
                                    $rule_html = $this->csd_get_rule_tpl( $params, true, $rule_arr['param'], $rule_arr['operator'], $rule_arr['value'], $a_price, $scope, count( $rules ) );
                                } else {
                                    $rule_html = $this->csd_get_rule_tpl( $params, false, $rule_arr['param'], $rule_arr['operator'], $rule_arr['value'] );
                                }
                                $r1 = str_replace( '{rule-group}', $rule_group, $rule_html );
                                $r2 = str_replace( '{rule-index}', $rule_index, $r1 );
                                echo $r2;
                                $rule_index++;
                            }
                            $rule_group++;
                            ?>
                        </tbody>
                    </table>
                    <?php
                }
            } else {
                ?>
                <table class="csd-rules-table widefat">
                    <tbody>
                        <?php
                        $rule_group = 0;
                        $rule_index = 0;
                        $r1         = str_replace( '{rule-group}', $rule_group, $first_rule );
                        $r2         = str_replace( '{rule-index}', $rule_index, $r1 );
                        echo $r2;
                        ?>
                    </tbody>
                </table>
                <?php
            }
            ?>

        </div>
        <a class="button csd-add-group mg-bot-10i">Add rule group</a>
        <?php
    }

    public function csd_get_config_form_builder() {
        $begin = array(
            'type' => 'sectionbegin',
            'id'   => 'csd-form-builder-container',
        );

        $args = array(
            'post_type' => 'ofb',
            'nopaging'  => true,
        );

        $forms = array(
            'title'   => __( 'Form Builder', 'csd' ),
            'desc'    => __( 'The selected form will be displayed on the customization page to gather additional informations from the customer.', 'csd' ),
            'name'    => 'csd-metas[form-builder]',
            'type'    => 'post-type',
            'default' => 'default',
            'args'    => $args,
        );

        $end = array(
            'type' => 'sectionend',
        );

        $settings = array(
            $begin,
            $forms,
            $end,
        );

        // Escaping output.
        $allowed_tags = csd_allowed_tags();
        $raw_output   = o_admin_fields( $settings );
        echo wp_kses( $raw_output, $allowed_tags );
        global $o_row_templates;
        ?>
        <script>
            var o_rows_tpl =<?php echo wp_json_encode( $o_row_templates ); ?>;
        </script>
        <?php
    }

    /**
     * Builds the output metabox
     */
    public function csd_get_config_output_metabox() {
        global $csd_settings;
        $output_global_settings = get_proper_value( $csd_settings, 'csd-output-options', array() );

        $options           = array();
        $output_format     = array(
            'title'     => esc_html__( 'Output Format', 'csd' ),
            'name'      => 'csd-metas[output-settings][output-format]',
            'type'      => 'radio',
            'row_class' => 'config-output-format',
            'options'   => array(
                'png'     => esc_html__( 'PNG', 'csd' ),
                'jpg'     => esc_html__( 'JPG', 'csd' ),
                'pdf+png' => esc_html__( 'PDF + PNG', 'csd' ),
                'pdf+jpg' => esc_html__( 'PDF + JPG', 'csd' ),
            ),
            'default'   => 'png',
        );
        $output_loop_delay = array(
            'title'   => esc_html__( 'Output loop delay (milliseconds)', 'csd' ),
            'name'    => 'csd-metas[output-settings][csd-output-loop-delay]',
            'type'    => 'text',
            'default' => get_proper_value( $output_global_settings, 'csd-output-loop-delay' ),
        );

        $zip_output      = array(
            'title'     => esc_html__( 'Zip output file', 'csd' ),
            'name'      => 'csd-metas[output-settings][zip-output]',
            'type'      => 'radio',
            'row_class' => 'zip-output',
            'options'   => array(
                'yes' => esc_html__( 'Yes', 'csd' ),
                'no'  => esc_html__( 'No', 'csd' ),
            ),
            'default'   => 'no',
        );
        $zip_folder_name = array(
            'title'     => esc_html__( 'Zip output folder prefix', 'csd' ),
            'name'      => 'csd-metas[output-settings][zip-folder-name]',
            'type'      => 'text',
            'row_class' => 'show-if-zip',
        );
        $cmyk_conversion = array(
            'title'     => esc_html__( 'CMYK conversion (Requires ImageMagick)', 'csd' ),
            'name'      => 'csd-metas[output-settings][csd-cmyk-conversion]',
            'type'      => 'radio',
            'row_class' => 'show-if-jpg',
            'options'   => array(
                'no'  => esc_html__( 'No', 'csd' ),
                'yes' => esc_html__( 'Yes', 'csd' ),
            ),
            'default'   => get_proper_value( $output_global_settings, 'csd-cmyk-conversion' ),
        );

        $pdf_format = array(
            'title'     => esc_html__( 'PDF Format', 'csd' ),
            'name'      => 'csd-metas[output-settings][pdf-format]',
            'type'      => 'groupedselect',
            'desc'      => esc_html__( "Output PDF format and default dimensions ( used if the generated design's dimensions are smaller or equals to the default ones.)", 'csd' ),
            'data-id'   => '',
            'row_class' => 'show-if-pdf',
            'options'   => get_csd_pdf_formats(),
            'default'   => get_proper_value( $output_global_settings, 'pdf-format' ),
        );

        $pdf_w = array(
            'title'   => esc_html__( 'Width', 'csd' ),
            'name'    => 'csd-metas[output-settings][pdf-width]',
            'type'    => 'number',
            'default' => get_proper_value( $output_global_settings, 'pdf-width' ),
        );

        $pdf_h = array(
            'title'   => esc_html__( 'Height', 'csd' ),
            'name'    => 'csd-metas[output-settings][pdf-height]',
            'type'    => 'number',
            'default' => get_proper_value( $output_global_settings, 'pdf-height' ),
        );

        $pdf_custom_dimensions = array(
            'title'     => 'Custom PDF dimensions',
            'desc'      => esc_html__( 'These dimensions will only be used if the PDF format is set to Custom.', 'csd' ),
            'type'      => 'groupedfields',
            'row_class' => 'show-if-pdf',
            'fields'    => array( $pdf_w, $pdf_h /* $pdf_unit */ ),
        );
        $pdf_resolution        = array(
            'title'     => 'PDF resolution (dpi)',
            'desc'      => esc_html__( 'Output PDF resolution.', 'csd' ),
            'type'      => 'number',
            'name'      => 'csd-metas[output-settings][resolution]',
            'row_class' => 'hide-if-pixels show-if-pdf',
            'default'   => 300,
        );

        $pdf_margin_top_bottom = array(
            'title'   => esc_html__( 'PDF Margin Top & Bottom', 'csd' ),
            'name'    => 'csd-metas[output-settings][pdf-margin-tb]',
            'type'    => 'text',
            'default' => get_proper_value( $output_global_settings, 'pdf-margin-tb', 20 ),
        );

        $pdf_margin_left_right = array(
            'title'   => esc_html__( 'PDF Margin Left & Right', 'csd' ),
            'name'    => 'csd-metas[output-settings][pdf-margin-lr]',
            'type'    => 'text',
            'default' => get_proper_value( $output_global_settings, 'pdf-margin-lr', 20 ),
        );

        $margins = array(
            'title'     => esc_html__( 'Margins', 'csd' ),
            'type'      => 'groupedfields',
            'desc'      => esc_html__( 'Vertical and Horizontal margins in the PDF file.', 'csd' ),
            'row_class' => 'show-if-pdf',
            'fields'    => array( $pdf_margin_top_bottom, $pdf_margin_left_right ),
        );

        $crop_x = array(
            'title' => esc_html__( 'Center Abcissa', 'csd' ),
            'name'  => 'x',
            'type'  => 'number',
        );
        $crop_y = array(
            'title' => esc_html__( 'Center Ordinate', 'csd' ),
            'name'  => 'y',
            'type'  => 'number',
        );
        $crop_w = array(
            'title' => esc_html__( 'Width', 'csd' ),
            'name'  => 'w',
            'type'  => 'number',
        );
        $crop_h = array(
            'title' => esc_html__( 'Height', 'csd' ),
            'name'  => 'h',
            'type'  => 'number',
        );
        $type   = array(
            'title' => esc_html__( 'Type', 'csd' ),
            'name'  => 'type',
            'type'  => 'text',
        );

        $crop_marks = array(
            'title'           => esc_html__( 'Crop marks', 'csd' ),
            'name'            => 'csd-metas[output-settings][crop-marks]',
            'desc'            => __( 'Crop marks settings.<br> <strong>Type</strong>: One symbol per type separated by comma: <ul><li>T = TOP</li><li>F = BOTTOM</li> <li>L = LEFT</li> <li>R = RIGHT</li><li>TL = A = TOP-LEFT</li><li>TR = B = TOP-RIGHT</li><li>BL = C = BOTTOM-LEFT</li><li>BR = D = BOTTOM-RIGHT</li></ul>', 'csd' ),
            'type'            => 'repeatable-fields',
            'fields'          => array( $crop_x, $crop_y, $crop_w, $crop_h, $type ),
            'row_class'       => 'hide-if-pixels show-if-pdf',
            'ignore_desc_col' => false,
            'add_btn_label'   => esc_html__( 'Add Mark', 'csd' ),
        );

        $bleed_x = array(
            'title'             => esc_html__( 'H. spacing', 'csd' ),
            'name'              => 'h',
            'type'              => 'number',
            'custom_attributes' => array( 'step' => '0.01' ),
        );
        $bleed_y = array(
            'title'             => esc_html__( 'V. spacing', 'csd' ),
            'name'              => 'v',
            'type'              => 'number',
            'custom_attributes' => array( 'step' => '0.01' ),
        );

        $bleed_color = array(
            'title'             => esc_html__( 'Border color', 'csd' ),
            'name'              => 'color',
            'type'              => 'text',
            'custom_attributes' => array( 'placeholder' => '#FF0000 for ex.' ),
        );

        $bleed_dash_spacing = array(
            'title'             => esc_html__( 'Dash spacing', 'csd' ),
            'name'              => 'dash-spacing',
            'type'              => 'number',
            'custom_attributes' => array( 'step' => '0.01' ),
        );

        $bleed_border_width = array(
            'title'             => esc_html__( 'Border width', 'csd' ),
            'name'              => 'border-width',
            'type'              => 'number',
            'custom_attributes' => array( 'step' => '0.01' ),
        );

        $bleed = array(
            'title'           => __( 'Bleed, Trim & Safe zones', 'csd' ),
            'name'            => 'csd-metas[output-settings][zones]',
            'desc'            => __( '<ul><li><strong>H. spacing</strong>: Space between the left and right borders of the document and the area.</li><li><strong>V. spacing</strong>: Space between the top and bottom borders of the document and the area.</li></ul>', 'csd' ),
            'type'            => 'repeatable-fields',
            'fields'          => array( $bleed_x, $bleed_y, $bleed_color, $bleed_dash_spacing, $bleed_border_width ),
            'row_class'       => 'hide-if-pixels show-if-pdf',
            'ignore_desc_col' => false,
            'add_btn_label'   => __( 'Add zone', 'csd' ),
        );

        $pdf_orientation = array(
            'title'     => __( 'PDF Orientation', 'csd' ),
            'name'      => 'csd-metas[output-settings][pdf-orientation]',
            'default'   => 'P',
            'type'      => 'select',
            'row_class' => 'show-if-pdf',
            'options'   => array(
                'P' => esc_html__( 'Portrait', 'csd' ),
                'L' => esc_html__( 'Landscape', 'csd' ),
            ),
            'default'   => get_proper_value( $output_global_settings, 'pdf-orientation' ),
        );

        $output_options_begin = array(
            'type'  => 'sectionbegin',
            'title' => esc_html__( 'Output Settings', 'csd' ),
            'table' => 'metas',
            'id'    => 'csd_product_output',
        );

        $output_options_end = array(
            'type' => 'sectionend',
            'id'   => 'csd_product_output',
        );

        array_push( $options, $output_options_begin );
        array_push( $options, $output_format );
        array_push( $options, $pdf_format );
        array_push( $options, $pdf_custom_dimensions );
        array_push( $options, $margins );
        array_push( $options, $pdf_orientation );
        array_push( $options, $zip_output );
        array_push( $options, $zip_folder_name );
        array_push( $options, $output_options_end );

        // Escaping output.
        $raw_output = o_admin_fields( $options );
        echo $raw_output;
        global $o_row_templates;
        ?>
        <script>
            var o_rows_tpl = <?php echo wp_json_encode( $o_row_templates ); ?>;
        </script>
        <?php
    }

    /**
     * Builds the pricing rules table
     *
     * @param type $params Array of parameters.
     * @param type $with_price Use with price.
     * @param type $default_param Default parameter.
     * @param type $default_operator Default operator.
     * @param type $default_value Default value.
     * @param type $default_price Default price.
     * @param type $default_scope Default scope.
     * @param type $count Count.
     * @return type
     */
    private function csd_get_rule_tpl( $params, $with_price = false, $default_param = false, $default_operator = '<', $default_value = '', $default_price = '', $default_scope = 'per_item', $count = 1 ) {
        ob_start();
        $operators    = array(
            '<'  => esc_html__( 'is less than', 'csd' ),
            '<=' => esc_html__( 'is less or equal to', 'csd' ),
            '==' => esc_html__( 'equals', 'csd' ),
            '>'  => esc_html__( 'more than', 'csd' ),
            '>=' => esc_html__( 'more or equal to', 'csd' ),
            '!=' => esc_html__( 'not equals', 'csd' ),
        );
        $scopes       = array(
            'item'            => esc_html__( 'Per item', 'wpd' ),
            'additional_item' => esc_html__( 'Per additional item', 'wpd' ),
            'design'          => esc_html__( 'On whole design', 'wpd' ),
        );
        $allowed_tags = csd_allowed_tags();
        ?>
        <tr data-id="rule_{rule-group}">
            <td class="param">
                <select id="csd-group_{rule-group}_rule_{rule-index}_param" class="select csd-pricing-group-param" name="csd-metas[pricing-rules][group_{rule-group}][rules][rule_{rule-index}][param]">
                    <?php
                    foreach ( $params as $param_key => $param_val ) {
                        if ( $param_key === $default_param ) {
                            ?>
                            <option value='<?php echo esc_attr( $param_key ); ?>' selected="selected"><?php echo esc_attr( $param_val ); ?></option>
                            <?php
                        } else {
                            ?>
                            <option value='<?php echo esc_attr( $param_key ); ?>'><?php echo esc_attr( $param_val ); ?></option>
                            <?php
                        }
                    }
                    ?>
                </select>
            </td>
            <td class="operator">
                <select id="csd-pricing-group_{rule-group}_rule_{rule-index}_operator" class="select operator" name="csd-metas[pricing-rules][group_{rule-group}][rules][rule_{rule-index}][operator]">
                    <?php
                    $option = 0;
                    foreach ( $operators as $operator_key => $operator_val ) {
                        if ( $operator_key === $default_operator ) {
                            $op_html = "<option id='csd-pricing-group_{rule-group}_rule_{rule-index}_{option}' value='" . htmlentities( $operator_key ) . "' selected='selected'>" . $operator_val . '</option>';
                        } else {
                            $op_html = "<option id='csd-pricing-group_{rule-group}_rule_{rule-index}_{option}' value='" . htmlentities( $operator_key ) . "'>" . $operator_val . '</option>';
                        }
                        $op = str_replace( '{option}', $option, $op_html );
                        echo wp_kses( $op, $allowed_tags );
                        $option++;
                    }
                    ?>
                    <script type="text/javascript">
                        var operators =<?php echo wp_json_encode( $operators ); ?>;
                    </script>

                </select>
            </td>
            <td class="value">
                <input type="text" name="csd-metas[pricing-rules][group_{rule-group}][rules][rule_{rule-index}][value]" value="<?php echo esc_attr( $default_value ); ?>" placeholder="number">
            </td>
            <?php
            if ( $with_price ) {
                ?>
                <td class="a_price" rowspan="<?php echo esc_attr( $count ); ?>">
                    <input type="number" name="csd-metas[pricing-rules][group_{rule-group}][a_price]" value="<?php echo esc_attr( $default_price ); ?>" placeholder="price" step="any">
                    <select id="csd-pricing-group_{rule-group}_rule_{rule-index}_scope" class="select scope" name="csd-metas[pricing-rules][group_{rule-group}][scope]">
                        <?php
                        foreach ( $scopes as $scope_key => $scope_val ) {
                            if ( $scope_key == $default_scope ) {
                                ?>
                                <option value='<?php echo $scope_key; ?>' selected="selected" 
                                                          <?php
                                                            if ( $scope_key == 'additional_item' ) {
                                                                ?>
                                    disabled
                                                                <?php
                                                            }
                                                            ?>
                                 ><?php echo $scope_val; ?></option>
                                             <?php
                            } else {
                                ?>
                                        <option value='<?php echo $scope_key; ?>' 
                                                                  <?php
                                                                    if ( $scope_key == 'additional_item' ) {
                                                                        ?>
                                            disabled
                                                                        <?php
                                                                    }
                                                                    ?>
                                         ><?php echo $scope_val; ?></option>
                                                     <?php
                            }
                        }
                        ?>
                    </select>
                </td>
                <?php
            }
            ?>
            <td class="add">
                <a class="csd-add-rule button" data-group='{rule-group}'><?php echo esc_attr__( 'and', 'csd' ); ?></a>
            </td>
            <td class="remove">
                <a class="csd-remove-rule acf-button-remove"></a>
            </td>
        </tr>
        <?php
        $rule_tpl = ob_get_contents();
        ob_end_clean();
        return $rule_tpl;
    }

    /**
     * Save the config
     *
     * @param type $post_id Post id.
     */
    public function csd_save_config( $post_id ) {
        $meta_key   = 'csd-metas';
        $checkboxes = array( 'can-design-from-blank' );
        if ( isset( $_POST[ $meta_key ] ) && wp_verify_nonce( $_POST['csd_config_nonce'], basename( __FILE__ ) ) ) {
            $old_metas = get_post_meta( $post_id, $meta_key, true );
            if ( empty( $old_metas ) ) {
                $old_metas = array();
            }
            $new_metas = $_POST[ $meta_key ];
            foreach ( $checkboxes as $checkboxes_key ) {
                if ( ! isset( $_POST[ $meta_key ][ $checkboxes_key ] ) && isset( $new_metas[ $checkboxes_key ] ) ) {
                    unset( $new_metas[ $checkboxes_key ] );
                }
            }
            update_post_meta( $post_id, $meta_key, $new_metas );
        }
    }

    /**
     * Re order the metaboxes
     *
     * @param array $order Array of ordered elements.
     * @return string
     */
    public function csd_get_metabox_order( $order ) {
        $order['advanced'] = 'csd-metas-canvas,csd-metas-text,csd-metas-fonts,csd-metas-pricing-rules,csd-metas-output,csd-metas-form-builder,submitdiv';
        return $order;
    }

    /**
     * Duplicate the config link
     *
     * @param string $actions Actions.
     * @param type   $post Post object.
     * @return string
     */
    public function csd_duplicate_post_link( $actions, $post ) {
        if ( 'csd-config' === $post->post_type && current_user_can( 'edit_posts' ) ) {
            $actions['duplicate'] = '<a href="admin.php?action=csd_duplicate_config&amp;post=' . $post->ID . '&amp;duplicate_nonce=' . wp_create_nonce( basename( __FILE__ ) ) . '" title="Duplicate this item" rel="permalink">Duplicate</a>';
        }
        return $actions;
    }

    /**
     * Duplicate the configs
     */
    public function csd_duplicate_config() {
        global $wpdb;

        /*
         * Nonce verification
         */
        if ( ! isset( $_GET['duplicate_nonce'] ) || ! wp_verify_nonce( $_GET['duplicate_nonce'], basename( __FILE__ ) ) ) {
            return;
        } else {
            if ( ! ( isset( $_GET['post'] ) || isset( $_POST['post'] ) || ( isset( $_REQUEST['action'] ) && 'csd_duplicate_config' === sanitize_text_field( wp_unslash( $_REQUEST['action'] ) ) ) ) ) {
                wp_die( 'No post to duplicate has been supplied!' );
            }
        }

        /*
         * get the original post id
         */
        $post_id = ( isset( $_GET['post'] ) ? absint( $_GET['post'] ) : absint( $_POST['post'] ) );

        /*
         * and all the original post data then
         */
        $post = get_post( $post_id );

        /*
         * if you don't want current user to be the new post author,
         * then change next couple of lines to this: $new_post_author = $post->post_author;
         */
        $current_user    = wp_get_current_user();
        $new_post_author = $current_user->ID;

        /*
         * if post data exists, create the post duplicate
         */
        if ( isset( $post ) && null !== $post ) {
            /*
             * new post data array
             */
            $args = array(
                'comment_status' => $post->comment_status,
                'ping_status'    => $post->ping_status,
                'post_author'    => $new_post_author,
                'post_content'   => $post->post_content,
                'post_excerpt'   => $post->post_excerpt,
                'post_name'      => $post->post_name,
                'post_parent'    => $post->post_parent,
                'post_password'  => $post->post_password,
                'post_status'    => 'draft',
                'post_title'     => $post->post_title . esc_html__( ' - copy', 'csd' ),
                'post_type'      => $post->post_type,
                'to_ping'        => $post->to_ping,
                'menu_order'     => $post->menu_order,
            );

            // insert the post by wp_insert_post() function
            $new_post_id = wp_insert_post( $args );

            // Get all current post terms ad set them to the new post draft.
            // Returns array of taxonomy names for post type, ex array("category", "post_tag");.
            $taxonomies = get_object_taxonomies( $post->post_type );
            foreach ( $taxonomies as $taxonomy ) {
                $post_terms = wp_get_object_terms( $post_id, $taxonomy, array( 'fields' => 'slugs' ) );
                wp_set_object_terms( $new_post_id, $post_terms, $taxonomy, false );
            }

            // duplicate all post meta just in two SQL queries
            $post_meta_infos = $wpdb->get_results( "SELECT meta_key, meta_value FROM $wpdb->postmeta WHERE post_id=$post_id" );
            if ( count( $post_meta_infos ) !== 0 ) {
                $sql_query = "INSERT INTO $wpdb->postmeta (post_id, meta_key, meta_value) ";
                foreach ( $post_meta_infos as $meta_info ) {
                    $meta_key = $meta_info->meta_key;
                    if ( '_wp_old_slug' !== $meta_key ) {
                        continue;
                    }
                    $meta_value      = addslashes( $meta_info->meta_value );
                    $sql_query_sel[] = "SELECT $new_post_id, '$meta_key', '$meta_value'";
                }
                $sql_query .= implode( ' UNION ALL ', $sql_query_sel );
                $wpdb->query( $sql_query );
            }

            /*
             * finally, redirect to the edit post screen for the new draft
             */
            wp_redirect( admin_url( 'post.php?action=edit&post=' . $new_post_id ) );
            exit();
        } else {
            wp_die( 'Post creation failed, could not find original post: ' . esc_attr( $post_id ) );
        }
    }

    /**
     * Save the variation metas
     *
     * @param type $variation_id Variation id.
     */
    public function csd_save_variation_settings_fields( $variation_id ) {
        $meta_key = 'csd-metas';
        if ( isset( $_POST[ $meta_key ] ) && wp_verify_nonce( $_POST['csd_save_variation_config_nonce'], basename( __FILE__ ) ) ) {
            $variation = wc_get_product( $variation_id );
            $old_metas = get_post_meta( $variation->parent->id, $meta_key, true );
            if ( empty( $old_metas ) ) {
                $old_metas = array();
            }
            $new_metas = array_replace( $old_metas, sanitize_text_field( wp_unslash( $_POST[ $meta_key ] ) ) );
            update_post_meta( $variation->parent->id, $meta_key, $new_metas );
        }
    }

    /**
     * Get the config selector for each product
     */
    public function csd_get_product_config_selector() {
        $id = get_the_ID();

        $args        = array(
            'post_type' => 'csd-config',
            'nopaging'  => true,
        );
        $configs     = get_posts( $args );
        $configs_ids = array( '' => 'None' );
        foreach ( $configs as $config ) {
            $configs_ids[ $config->ID ] = $config->post_title;
        }
        ?>
        <div id="csd_config_data" class="show_if_simple">
            <?php
            $this->csd_get_product_tab_row( $id, $configs_ids, 'Configuration Custom Signs Designer for WooCommerce' );
            ?>
        </div>
        <?php
    }

    /**
     * Add a new tab for the product to choose a config
     *
     * @param type $pid Product id.
     * @param type $configs_ids Configuration ids.
     * @param type $title Title.
     */
    private function csd_get_product_tab_row( $pid, $configs_ids, $title ) {
        $begin = array(
            'type' => 'sectionbegin',
            'id'   => 'csd-metas-data',
        );

        $configurations = array(
            'title'   => $title,
            'name'    => "csd-metas[$pid][config-id]",
            'type'    => 'select',
            'options' => $configs_ids,
        );

        $end      = array( 'type' => 'sectionend' );
        $settings = apply_filters(
            'csd_product_tab_settings',
            array(
                $begin,
                $configurations,
                $end,
            )
        );

        // Escaping output.
        $allowed_tags = csd_allowed_tags();
        $raw_output   = o_admin_fields( $settings );
        wp_nonce_field( 'edit', 'csd_save_product_config_nonce' );
        echo "<div class='csd-product-config-row'>" . wp_kses( $raw_output, $allowed_tags ) . '</div>';
    }

    /**
     * Set Variables product configuration form*
     *
     * @param type $loop Loop.
     * @param type $variation_data Variation data.
     * @param type $variation Variation object.
     */
    public function csd_get_variation_product_config_selector( $loop, $variation_data, $variation ) {
        $id = $variation->ID;

        $args        = array(
            'post_type' => 'csd-config',
        );
        $configs     = get_posts( $args );
        $configs_ids = array( '' => 'None' );
        foreach ( $configs as $config ) {
            $configs_ids[ $config->ID ] = $config->post_title;
        }
        ?>
        <tr>
            <td>
                <?php
                $this->csd_get_product_tab_row( $id, $configs_ids, 'Configuration' );
                ?>
            </td>
        </tr>
        <?php
    }

    /**
     *  Force the layout in the config to be one column
     *
     * @param array $columns Array of columns number.
     * @return int
     */
    public function csd_get_layout_columns( $columns ) {
        $columns['csd-config'] = 1;
        return $columns;
    }

}
