<?php

/**
 * The plugin bootstrap file
 *
 * This file is read by WordPress to generate the plugin information in the plugin
 * admin area. This file also includes all of the dependencies used by the plugin,
 * registers the activation and deactivation functions, and defines a function
 * that starts the plugin.
 *
 * @link              https://designersuiteforwp.com
 * @since             1.1.0
 * @package           Csd
 *
 * @wordpress-plugin
 * Plugin Name:       Custom Signs Designer for WooCommerce
 * Plugin URI:        https://designersuiteforwp.com
 * Description:       A simple signs design plugin for WooCommerce
 * Version:           1.6
 * Author:            ORION
 * Author URI:        https://designersuiteforwp.com
 * License:           GPL-2.0+
 * License URI:       https://designersuiteforwp.com
 * Text Domain:       Csd
 * Domain Path:       /languages
 */

// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}

define( 'CSD_VERSION', '1.6' );
define( 'CSD_URL', plugins_url('/', __FILE__) );
define( 'CSD_DIR', dirname(__FILE__) );
define( 'CSD_MAIN_FILE', 'Csd/csd.php' );
define( 'CSD_PLUGIN_NAME', 'Custom Signs Designer for WooCommerce' );
define( 'CSD_PLUGIN_ID', 001 );


/**
 * The code that runs during plugin activation.
 * This action is documented in includes/class-csd-activator.php
 */
function activate_csd() {
	require_once plugin_dir_path( __FILE__ ) . 'includes/class-csd-activator.php';
	csd_Activator::activate();
}

/**
 * The code that runs during plugin deactivation.
 * This action is documented in includes/class-csd-deactivator.php
 */
function deactivate_csd() {
	require_once plugin_dir_path( __FILE__ ) . 'includes/class-csd-deactivator.php';
	Csd_Deactivator::deactivate();
}

register_activation_hook( __FILE__, 'activate_csd' );
register_deactivation_hook( __FILE__, 'deactivate_csd' );

/**
 * The core plugin class that is used to define internationalization,
 * admin-specific hooks, and public-facing site hooks.
 */
require plugin_dir_path( __FILE__ ) . 'includes/class-csd.php';
require_once( CSD_DIR . DIRECTORY_SEPARATOR . 'admin' . DIRECTORY_SEPARATOR . 'class-csd-clipart.php' );
require_once( CSD_DIR . DIRECTORY_SEPARATOR . 'admin' . DIRECTORY_SEPARATOR . 'class-csd-config.php' );

if(!function_exists("o_admin_fields"))
{
    require_once plugin_dir_path(__FILE__) . 'includes/utils.php';
}

require_once plugin_dir_path(__FILE__) . 'includes/functions.php';

require_once CSD_DIR . DIRECTORY_SEPARATOR . 'includes' . DIRECTORY_SEPARATOR . 'class-csd-retarded-actions.php';

//include default skin
require_once( CSD_DIR . DIRECTORY_SEPARATOR . 'includes' . DIRECTORY_SEPARATOR . 'skins' . DIRECTORY_SEPARATOR . 'default' . DIRECTORY_SEPARATOR . 'class-csd-default-skin.php' );

require_once( CSD_DIR . DIRECTORY_SEPARATOR . 'includes' . DIRECTORY_SEPARATOR . 'class-csd-editor.php' );

if(!class_exists("TCPDF")){
    require_once( CSD_DIR . DIRECTORY_SEPARATOR . 'includes' . DIRECTORY_SEPARATOR . 'tcpdf' . DIRECTORY_SEPARATOR . 'tcpdf.php' );
}


/**
 * Begins execution of the plugin.
 *
 * Since everything within the plugin is registered via hooks,
 * then kicking off the plugin from this point in the file does
 * not affect the page life cycle.
 *
 * @since    1.0.0
 */
function run_csd() {

	$plugin = new Csd();
	$plugin->run();

}
run_csd();
