var csd_EDITOR = (function ($, csd_editor) {
    'use strict';
    $(document).ready(function () {
        var text_property = {};
        //Text
        $(document).on('click', '#csd-add-text', function (event) {
            event.preventDefault();
            var new_text = $("#new-text").val();
            var is_curved = $("#cb-curved").is(":checked");

            var height = csd_editor.getTextHeight();
            if (new_text.length == 0)
                alert(csd.translated_strings.empty_txt_area_msg);
            else if (!is_curved) {
                csd_editor.add_text(new_text, height);
            } else
            {
                add_curved_text(new_text);
            }
        });

        csd_editor.add_text = function (txt, height) {

            var text = create_text_elmt(txt);

            csd_editor.canvas.add(text);

            text.center();

            var csd_canvas = csd_editor.canvas;
            wp.hooks.doAction('csd_EDITOR.after_adding_text_on_canvas', text, csd_canvas);
            csd_editor.canvas.renderAll();
            text.setCoords();

            $("#new-text").val("");
            csd_editor.save_canvas();
        };

        $("#font-family-selector").trigger('change');

        $("#font-family-selector").css('font-family', $("#font-family-selector").val());
        $("#font-family-selector").change(function ()
        {
            var selected_object = csd_editor.canvas.getActiveObject();
            var font_size = parseFloat($("#csd-font-size").val());
            if(font_size) 
            {
                var csd_unit = $('#csd-unit-select').val();
                font_size = csd_editor.convert_to_px(font_size, csd_unit) * csd_editor.multiplier;
            }

            var font_family = $(this).attr("value");
            $(this).css('font-family', font_family);
            if ((selected_object != null) && (selected_object.type == "i-text"))
            {
                selected_object.set('fontFamily', font_family);
                if (font_size)
                    selected_object.set("fontSize", parseFloat(font_size));
                csd_editor.canvas.renderAll();
                csd_editor.save_canvas();
            } else if ((selected_object != null) && (selected_object.type == "group"))
            {
                selected_object.forEachObject(function (a) {
                    a.set('fontFamily', font_family);
                    if (font_size)
                        a.set("fontSize", parseFloat(font_size));
                    csd_editor.canvas.renderAll();
                    csd_editor.save_canvas();
                });
            }
        });


        $("#font-size-selector").change(function ()
        {
            var selected_object = csd_editor.canvas.getActiveObject();
            var font_size = parseInt($("#font-size-selector").val());
            if ((selected_object != null) && (selected_object.type == "i-text"))
            {
                selected_object.setFontSize(parseInt(font_size));
                csd_editor.canvas.renderAll();
            } else if ((selected_object != null) && (selected_object.type == "group"))
            {
                recreate_group(selected_object);
            }
        });

        $(".txt-align").change(function ()
        {
            var align = $(this).val();
            var selected_object = csd_editor.canvas.getActiveObject();
            if ((selected_object != null) && (selected_object.type == "i-text"))
            {
                selected_object.set("textAlign", align);
                csd_editor.canvas.renderAll();
            } else if ((selected_object != null) && (selected_object.type == "group"))
            {
                selected_object.forEachObject(function (a) {
                    a.set("textAlign", align);
                    csd_editor.canvas.renderAll();
                });
            }
            csd_editor.save_canvas();

        });

        //Bold styles
        $("#bold-cb").change(function ()
        {
            var selected_object = csd_editor.canvas.getActiveObject();
            var is_bold = $("#bold-cb").is(":checked");
            if ((selected_object != null) && (selected_object.type == "i-text"))
            {
                if (is_bold)
                    selected_object.set("fontWeight", "bold");
                else
                    selected_object.set("fontWeight", "normal");
                csd_editor.canvas.renderAll();
            } else if ((selected_object != null) && (selected_object.type == "group"))
            {
                selected_object.forEachObject(function (a) {
                    if (is_bold)
                        a.set("fontWeight", "bold");
                    else
                        a.set("fontWeight", "normal");
                    csd_editor.canvas.renderAll();
                });
            }
            csd_editor.save_canvas();
        });

        $("#italic-cb").change(function ()
        {
            var selected_object = csd_editor.canvas.getActiveObject();
            var is_italic = $("#italic-cb").is(":checked");
            if ((selected_object != null) && (selected_object.type == "i-text"))
            {
                if (is_italic)
                    selected_object.set("fontStyle", "italic");
                else
                    selected_object.set("fontStyle", "normal");
                csd_editor.canvas.renderAll();
            } else if ((selected_object != null) && (selected_object.type == "group"))
            {
                selected_object.forEachObject(function (a) {
                    if (is_italic)
                        a.set("fontStyle", "italic");
                    else
                        a.set("fontStyle", "normal");
                    csd_editor.canvas.renderAll();
                });
            }
            csd_editor.save_canvas();
        });

        $("#o-thickness-slider").change(function ()
        {
            var selected_object = csd_editor.canvas.getActiveObject();
            if ((selected_object != null) && (selected_object.type == "i-text"))
            {
                if ($(this).val() > 0)
                {
                    var stroke = $("#txt-outline-color-selector").css('background-color');
                    selected_object.set("strokeWidth", parseInt($(this).val()));
                    selected_object.set("stroke", stroke);
                } else
                    selected_object.set("stroke", false);
                csd_editor.canvas.renderAll();
            } else if ((selected_object != null) && (selected_object.type == "group"))
            {
                recreate_group(selected_object);
            }
            csd_editor.save_canvas();
        });


        //Is curved checkbox
        function csd_get_curved_state() {
            if ($('.csd-text-curved-wrap #cb-curved').is(':checked')) {

                $('.csd-text-curved-wrap #cb-curved').parents('.csd-text-curved-wrap').find('.csd-text-curved-content').addClass('is-active');
            } else {

                $('.csd-text-curved-wrap #cb-curved').parents('.csd-text-curved-wrap').find('.csd-text-curved-content').removeClass('is-active');
            }
        }

        $("#cb-curved").change(function ()
        {
            csd_get_curved_state();
            var is_curved = $("#cb-curved").is(":checked");
            var selected_object = csd_editor.canvas.getActiveObject();

            if (is_curved)
            {
                if (selected_object != null)
                {

                    var left = selected_object.get("left");
                    var top = selected_object.get("top");
                    if (selected_object.type == "i-text")
                    {
                        text_property.color = selected_object.fill;
                        selected_object.bg_color = selected_object.backgroundColor;
                        var text = selected_object.get("text");

                        csd_editor.canvas.remove(selected_object);
                        add_curved_text(text, top, left);
                        csd_editor.save_canvas();
                        csd_editor.canvas.renderAll();
                        $("#cb-curved").attr('checked', 'checked');
                        $('.csd-font-view').html(text);
                    }
                }
            } else
            {
                if (selected_object != null)
                {
                    var left = selected_object.get("left");
                    var top = selected_object.get("top");
                    if (selected_object.type == "group")
                    {
                        var text = selected_object.get("originalText");
                        csd_editor.canvas.remove(selected_object);
                        csd_editor.add_text(text, top, left, false, false, false);
                        csd_editor.save_canvas();
                        csd_editor.canvas.renderAll();
                    }
                }
            }


        });


        $("#curved-txt-radius-slider, #curved-txt-spacing-slider").change(function () {
            var selected_object = csd_editor.canvas.getActiveObject();
            //activeSelection
            if ((selected_object != null) && (selected_object.type == "group")) {
                var obj = selected_object.getObjects();
                var text_color = "#000", bg_color = "#000";
                obj.forEach(element => {
                    if(element.type != null && element.type=="i-text"){
                        text_color = element.fill;
                        bg_color = element.fill;
                    }
                });

                text_property.color = text_color;
                text_property.bgcolor = bg_color;
                recreate_group(selected_object);
            }

        });

        $("[id$='opacity-slider']").change(function ()
        {
            var selected_object = csd_editor.canvas.getActiveObject();
            if (selected_object != null)
            {
                selected_object.set("opacity", $(this).val());
                csd_editor.canvas.renderAll();
                csd_editor.save_canvas();
            }
        });

        $(".txt-decoration").change(function ()
        {

            var selected_object = csd_editor.canvas.getActiveObject();
            var decoration = $(this).val();

            var decorations = ["underline", "linethrough", "overline"];
            if ((selected_object != null) && (selected_object.type == "i-text"))
            {
                if (decoration == "none") {
                    $.each(decorations, function (index, value) {
                        selected_object.set(value, false);
                    });
                }
                selected_object.set(decoration, true);
                csd_editor.canvas.renderAll();
            }
            csd_editor.save_canvas();
        });

        wp.hooks.applyFilters('scotch.special-characters', csd_editor);

        function add_curved_text(str, custom_top, custom_left)
        {
            var len = str.length;
            var s;
            var radius = $("#curved-txt-radius-slider").val();
            var spacing = $("#curved-txt-spacing-slider").val();
            var font_color = $('#txt-color-selector').css('background-color');
            if(!font_color && typeof text_property.color !== "undefined") font_color = text_property.color;
            if (!radius)
                radius = 150;
            if (!spacing)
                spacing = 10;
            var curAngle = 0;
            var angleRadians = 0;
            var align = 0;
            var centerX = csd_editor.canvas.width / 2;
            var centerY = csd_editor.canvas.height - 30;
            align = (spacing / 2) * (len - 1);
            var reverse = false;
            var coef = 1;
            if (reverse)
                coef = -1;
            var items = [];
            for (var n = 0; n < len; n++) {
                s = str[n];

                var text = create_text_elmt(s);
                curAngle = (n * parseInt(spacing, 10)) - align;
                angleRadians = curAngle * (Math.PI / 180);
                if (reverse)
                {
                    curAngle = (-n * parseInt(spacing, 10)) + align;
                    angleRadians = curAngle * (Math.PI / 180);
                }

                var top = (centerX + (-Math.cos(angleRadians) * radius)) * coef;
                var left = (centerY + (Math.sin(angleRadians) * radius)) * coef;
                text.set('top', top);
                text.set('left', left);
                text.set("angle", curAngle);
                text.set("spacing", spacing);
                text.set("radius", radius);
                items.push(text);
            }
            var group = new fabric.Group(items, {
                left: 150,
                top: 100,
                fill: font_color
            });

            if (custom_top != null)
                csd_editor.canvas.setActiveObject(group);
            csd_editor.setCustomProperties(group);
            group["originalText"] = str;
            group["radius"] = radius;
            group["spacing"] = spacing;
            group.set("csdcurved", str);
            csd_editor.canvas.add(group);

            if (custom_top == null)
                group.center();
            else
            {
                group.set("left", custom_left);
                group.set("top", custom_top);
            }

            csd_editor.save_canvas();
            csd_editor.canvas.renderAll();
            group.setCoords();
        }

   

        function create_text_elmt(txt)
        {
            var strokeWidth = $("#o-thickness-slider").val();
            var fontWeight = "normal";
            var textDecoration = "";
            var fontStyle = "";
            var font_color = $('#txt-color-selector').css('background-color');
            if(!font_color && typeof text_property.color !== "undefined") font_color = text_property.color;
            var fontFamily = $('#font-family-selector').val();
            var font_size = parseFloat($("#csd-font-size").val());
            var opacity = $("#opacity-slider").val();
            var strokeColor = $("#txt-outline-color-selector").css('background-color');
            var is_bold = $("#bold-cb").is(":checked");
            var is_underlined = $("#underline-cb").is(":checked");
            var is_italic = $("#italic-cb").is(":checked");

            if(font_size) 
            {
                var csd_unit = $('#csd-unit-select').val();
                font_size = csd_editor.convert_to_px(font_size, csd_unit) * csd_editor.multiplier;
            }

            if (is_bold)
                fontWeight = "bold";
            if (is_underlined)
                textDecoration = "underline";
            if (is_italic)
                fontStyle = "italic";
            if (!fontFamily)
                fontFamily = "Arial";
            if (!font_size)
                font_size = 30;
            if (!fontWeight)
                fontWeight = "normal";
            if (!font_color)
                font_color = "rgb(198, 196, 196)";
            if (!opacity)
                opacity = 1;

            var text = new fabric.IText(txt, {
                padding: 0,
                left: 30,
                top: 70,
                fontFamily: fontFamily,
                fontSize: font_size,
                fontWeight: fontWeight,
                fontStyle: fontStyle,
                textDecoration: textDecoration,
                selectable: true,
                fill: font_color,
                opacity: opacity,
                strokeWidth: 0
            });
            if (csd.clip_w) {
                var optimal_dimensions = csd_editor.get_optimal_canvas_dimensions();
                var scaleFactor = optimal_dimensions[0] / csd.canvas_w;
                text.set("fixedWidth", csd.clip_w * scaleFactor);
            }
            text.padding = 0;
            text.set("id", "toto");
            text.set("originX", "center");
            text.set("originY", "center");

            if (strokeWidth > 0)
            {
                text.set("stroke", strokeColor);
                text.set("strokeWidth", parseInt(strokeWidth));
            }
            csd_editor.setCustomProperties(text);

            return text;
        }

        function recreate_group(group)
        {
            var left = group.get("left");
            var top = group.get("top");
            csd_editor.canvas.remove(group);
            add_curved_text(group.originalText, top, left);
        }

        $('#new-text').keyup(function ()
        {
            var selected_object = csd_editor.canvas.getActiveObject();
            var new_text = $('#new-text').val();
            if ((selected_object != null) && (selected_object.type == "i-text"))
            {
                selected_object.set("text", new_text);
                csd_editor.save_canvas();
                csd_editor.canvas.renderAll();
            } else if ((selected_object != null) && (selected_object.type == "group"))
            {
                var left = selected_object.get("left");
                var top = selected_object.get("top");
                csd_editor.canvas.remove(selected_object);
                add_curved_text(new_text, top, left);
            }
        });

        if (csd.text_palette_type === "colors") {
            $('#txt-color-selector').css('background-color', csd.text_palette_tpl[0]);
            var default_text_color = csd.text_palette_tpl[0];
            var showTextPaletteOnly = true;
            var showTextPalette = true;
        } else if (csd.text_palette_type === "colorpicker") {
            if (csd.text_default_color !== '' && csd.text_default_color !== null) {
                $('#txt-color-selector').css('background-color', csd.text_default_color);
                var default_text_color = csd.text_default_color;
            } else {
                $('#txt-color-selector').css('background-color', "#0000ff");
                var default_text_color = "#0000ff";
            }
            var showTextPaletteOnly = false;
            var showTextPalette = false;
        }

        if (csd.bg_palette_type === "colors") {
            $('#txt-bg-color-selector').css('background-color', csd.bg_palette_tpl[0]);
            var default_bg_color = csd.bg_palette_tpl[0];
            var showBgPaletteOnly = true;
            var showBgPalette = true;
        } else if (csd.bg_palette_type === "colorpicker") {
            if (csd.bg_default_color !== '' && csd.bg_default_color !== null) {
                $('#txt-bg-color-selector').css('background-color', csd.bg_default_color);
                var default_bg_color = csd.bg_default_color;
            } else {
                $('#txt-bg-color-selector').css('background-color', "#0000ff");
                var default_bg_color = "#0000ff";
            }
            var showBgPaletteOnly = false;
            var showBgPalette = false;
        }


        $('[id$="color-selector"]').each(function ()
        {
            var id = $(this).attr("id");
            if (id === 'txt-color-selector')
            {

                $('#' + id).spectrum({
                    color: default_text_color,
                    flat: true,
                    showAlpha: false,
                    allowAlpha: true,
                    showPalette: showTextPalette,
                    showPaletteOnly: showTextPaletteOnly,
                    showButtons: false,
                    clickoutFiresChange: true,
                    preferredFormat: "hex",
                    palette: csd.text_palette_tpl,
                    move: function (color) {
                        $(this).parents(".csd-tool-row").find(".csd-color-value").val(color);
                        $("#txt-color-selector").val(color);
                        csd_editor.change_item_color(id, color);

                    },
                });
            } else {
                $('#' + id).spectrum({
                    color: default_text_color,
                    flat: false,
                    showAlpha: false,
                    allowAlpha: true,
                    showPalette: showBgPalette,
                    showPaletteOnly: showBgPaletteOnly,
                    showButtons: false,
                    clickoutFiresChange: true,
                    preferredFormat: "hex",
                    palette: csd.bg_palette_tpl,
                    move: function (color) {
                        // $(this).parents(".csd-tool-row").find(".csd-color-value").val(color);
                        $("#txt-bg-color-selector").val(color);
                        csd_editor.change_item_color(id, color);

                    },

                });
            }

        });
    });
    return csd_editor;
}(jQuery, (csd_EDITOR || {})));