var csd_EDITOR = (function ($, csd_editor) {
    'use strict';
    var csd_editor = {};
    csd_editor.canvas = {};
    csd_editor.serialized_parts = {};
    csd_editor.final_canvas_parts = {};
    csd_editor.selected_part = -1;
    csd_editor.canvasManipulationsPosition = [];
    csd_editor.box_center_x = false;
    csd_editor.box_center_y = false;
    csd_editor.scale_factor = false;
    csd_editor.output_loop_delay = 3000;
    csd_editor.multiplier = 1;
    csd_editor.color_values = ['#FDC689', '#2C4FAA', '#2CAA31', '#AA2C2C', '#2C7DAA', '#9DD500', '#AA872C', '#AA2C8F'];
    csd_editor.arr_filters = ['grayscale', 'invert', 'remove-white', 'sepia', 'sepia2',
        'brightness', 'noise', 'gradient-transparency', 'pixelate',
        'blur', 'convolute'];

    $(document).ready(function () {

        setTimeout(function () {
            csd_update_price();
        }, 5);

        csd_editor.convert_to_px = function (dimensions, unit) {
            var dimensions = dimensions;
            var unit = unit;
            if (typeof unit != 'undefined' && typeof dimensions != 'undefined') {
                switch (unit) {
                    case 'Cm':
                        dimensions = dimensions * 120;
                        break;
                    case 'Inch':
                        dimensions = dimensions * 300;
                        break;
                    case 'Feet':
                        dimensions = dimensions * 3600;
                        break;
                    case 'Mm':
                        dimensions = dimensions * 118;
                        break;
                    case 'Px':
                        dimensions = dimensions;
                    default:
                        break;
                }
                return parseFloat(dimensions).toFixed(2);
            }
        }

        csd_editor.get_true_dimension = function (dimension, units = " ") {
            var dimension = dimension;
            var unit;
            if (units == " ") {
                unit = $('#csd-unit-select').val();
            } else {
                unit = units;
            }
            if (typeof unit != 'undefined' && typeof dimension != 'undefined') {
                switch (unit) {
                    case 'Inch':
                        dimension = dimension / 300;
                        break;
                    case 'Cm':
                        dimension = dimension / 120;
                        break;
                    case 'Feet':
                        dimension = dimension / 3600;
                        break;
                    case 'Mm':
                        dimension = dimension / 118;
                        break;
                    case 'Px':
                        dimension = dimension;
                    default :
                        break;
                }
                return parseFloat(dimension).toFixed(2);
            }
        }

        if (typeof csd != 'undefined')
        {
            accounting.settings = {
                currency: {
                    symbol: csd.currency, // default currency symbol is '$'
                    format: csd.price_format, // controls output: %s = symbol, %v = value/number (can be object: see below)
                    decimal: csd.decimal_sep, // decimal point separator
                    thousand: csd.thousand_sep, // thousands separator
                    precision: csd.nb_decimals   // decimal places
                },
                number: {
                    precision: csd.nb_decimals, // default precision on numbers is 0
                    thousand: csd.thousand_sep,
                    decimal: csd.decimal_sep
                }
            }
        }

        if (typeof to_load !== "undefined") {
            if (typeof to_load.output !== "undefined") {
                if (typeof to_load.output.form_fields !== "undefined") {
                    var fields = to_load.output.form_fields;
                    if (fields) {
                        set_form_builder_preload_data(fields);
                    }
                }
            }
        }

        function set_form_builder_preload_data(fields) {
            $('form.formbuilt').find('[name]').each(function (index, value) {
                var that = $(this),
                        name = that.attr('name'),
                        type = that.prop('type');
                if (type == 'select-one') {
                    $(that).find('[value]').each(function (index, value) {
                        var option = $(this);
                        var value = option.attr('value');
                        for (var i in fields) {
                            if (name == i && value == fields[i]) {
                                option.attr('selected', 'selected');
                            }
                        }
                    });
                } else if (type == 'radio' || type == 'checkbox') {
                    var price = that.attr('data-price');
                    for (var i in fields) {
                        if (name == i) {
                            if (typeof (fields[i]) == 'object') {
                                var options = fields[i];
                                for (var j in options) {
                                    if (value.value == options[j]) {
                                        that.attr('checked', 'checked');
                                    }
                                }
                            } else {
                                if (value.value == fields[i]) {
                                    that.attr('checked', 'checked');
                                }
                            }
                        }
                    }
                } else if (type == 'file') {
                    for (var i in fields) {
                        if (name == i) {
                        }
                    }
                } else {
                    for (var i in fields) {
                        if (name == i) {
                            that.attr('value', fields[i]);
                        }
                    }
                }
            });
            csd_update_price();
        }

        function isEmpty(obj) {
            for (var prop in obj) {
                if (obj.hasOwnProperty(prop))
                    return false;
            }
            return JSON.stringify(obj) === JSON.stringify({});
        }

        $("form.formbuilt input").change(function () {
            csd_update_price();
        });

        $("form.formbuilt textarea").change(function () {
            csd_update_price();
        });

        $("form.formbuilt select").change(function () {
            csd_update_price();
        });

        $(".csd-responsive-mode .csd-editor-menu-right").toggle(
                function () {
                    $(".csd-responsive-mode .csd-editor-col.right").css("display", "inline-block");
                },
                function () {
                    $(".csd-responsive-mode .csd-editor-col.right").css("display", "none");
                }
        );

        var resizeId;
        var tools_accordion = new Spry.Widget.Accordion("csd-tools-box-container", {useFixedPanelHeights: false, defaultPanel: -1});
        new Spry.Widget.Accordion("my-designs-accordion", {useFixedPanelHeights: false, defaultPanel: -1});
        $("[data-tooltip-title]").otooltip();

        csd_editor.get_optimal_canvas_dimensions = function ()
        {
            var available_width = $(".csd-editor-content").width();
            var available_height = $(".csd-editor-content").height();

            var signs_width;
            var signs_height;
            var default_unit = csd.default_unit;

            var csd_optimal_dimension;

            if (default_unit !== "Px") {
                signs_width = csd_editor.convert_to_px(csd.canvas_w, default_unit);
                signs_height = csd_editor.convert_to_px(csd.canvas_h, default_unit);
            } else {
                signs_width = csd.canvas_w;
                signs_height = csd.canvas_h;
            }

            csd_editor.signs_width = signs_width;
            csd_editor.signs_height = signs_height;

            if (signs_width > available_width) {
                csd_optimal_dimension = update_dimension_with_width(signs_width, signs_height, available_width);
                return [csd_optimal_dimension[0], csd_optimal_dimension[1]];
            } else {
                return [signs_width, signs_height];
            }
        };

        init_canvas();
        init_empty_canvas_data_array();

        $(document).on("click", "#csd-parts-bar > li", function (e) {

            var part_index = $(this).index();

            load_background_overlay_if_needed(part_index);
            $("#csd-parts-bar > li").removeClass("active");
            $(this).addClass("active");
            if (csd_editor.selected_part >= 0)
            {
                csd_editor.save_canvas();
//                csd_editor.canvas.clear();
            }
            $("#csd-editor-container .canvas-container canvas").css({border: "1px solid black", top: csd.canvas_top + "px", left: 0, right: 0, margin: "0 auto"});
            csd_editor.selected_part = $(this).index();

            csd_editor.refresh_undo_redo_status();
        });

        wp.hooks.applyFilters('scotch.background-image', csd_editor);

        csd_editor.applyImageFilters = function () {
            csd_editor.canvas.forEachObject(function (obj) {
                if (obj.type === 'image' && obj.filters.length) {
                    obj.applyFilters();
                }
            });
        }

        function reformat_csd_parameters_arr()
        {
            var keys = ["canvas_h", "canvas_w", "clip_h", "clip_r", "clip_rr", "clip_w", "clip_x", "clip_y", "output_loop_delay", "output_w"];
            var i = 0;
            for (i = 0; i < keys.length; i++) {
                csd[keys[i]] = parseFloat(csd[keys[i]]);
            }
        }

        function init_canvas()
        {
            var canvas_w = csd.canvas_w;
            var canvas_h = csd.canvas_h;
            var default_unit = csd.default_unit;
            // console.log(canvas_h+" "+canvas_w+" "+default_unit);
            //Converts csd elements strings into float to avoid formatting issues
            reformat_csd_parameters_arr();
            //We determine the best dimensions to use
            var optimal_dimensions = csd_editor.get_optimal_canvas_dimensions();

            csd_editor.canvas = new fabric.Canvas('csd-editor', {width: optimal_dimensions[0], height: optimal_dimensions[1]});
            csd_editor.canvas.backgroundImageStretch = false;
            set_clipping_area();
            csd_editor.canvas.renderAll();
            load_canvas_listeners();
            $('#sumamary-width').text(canvas_w);
            $('#sumamary-height').text(canvas_h);
            $("#csd-canvas-width").val(canvas_w);
            $("#csd-canvas-height").val(canvas_h);
            $("#csd-unit-select").val(default_unit);

            if(default_unit == "Feet")
                var csd_step = 0.01;
            else var csd_step = 0.5;

            $(".csd-text-size").attr("step", csd_step);
            $(".csd-canvas-size").attr("step", csd_step);
            $(".csd-object-size").attr("step", csd_step);
            
            $.each(csd_editor.color_values, function (key, value) {
                $('.csd-previous-color').append("<span id=csd_color_" + key + " style=background-color:" + value + "></span>");
            });

        }

        function set_clipping_area()
        {
            //We determine the best dimensions to use
            var optimal_dimensions = csd_editor.get_optimal_canvas_dimensions();
            csd_editor.canvas.clipTo = null;
            var scaleFactor = optimal_dimensions[0] / csd.canvas_w;
            if (scaleFactor != 1)
                csd_editor.scale_factor = scaleFactor;
            if (csd.clip_w && csd.clip_h && csd.clip_w > 0 && csd.clip_h > 0 && csd.clip_type == "rect")
            {
                var clip_x = (optimal_dimensions[0] - csd.clip_w * scaleFactor) / 2;
                if (csd.clip_x || csd.clip_x == "0")
                {
                    clip_x = csd.clip_x * scaleFactor;
                    csd_editor.box_center_x = parseFloat(clip_x) + parseFloat(csd.clip_w * scaleFactor) / 2;
                } else if (scaleFactor != 1)
                    csd_editor.box_center_x = csd.canvas_w / 2;
                else
                    csd_editor.box_center_x = optimal_dimensions[0] / 2;
                var clip_y = (optimal_dimensions[1] - csd.clip_h * scaleFactor) / 2;
                if (csd.clip_y || csd.clip_y == "0")
                {
                    clip_y = csd.clip_y * scaleFactor;
                    csd_editor.box_center_y = parseFloat(clip_y) + parseFloat(csd.clip_h * scaleFactor) / 2;
                } else if (scaleFactor != 1)
                    csd_editor.box_center_y = csd.canvas_h / 2;
                else
                    csd_editor.box_center_y = optimal_dimensions[1] / 2;
                csd_editor.canvas.clipTo = function (ctx) {
                    if (csd.clip_type == "rect" || csd.clip_type == "")
                    {
                        if (csd.clip_rr > 0)
                            roundRect(ctx, (clip_x), (clip_y), parseFloat(csd.clip_w * scaleFactor), parseFloat(csd.clip_h * scaleFactor), parseFloat(csd.clip_rr * scaleFactor), "", csd.clip_border);
                        else
                        {
                            ctx.rect(clip_x, clip_y, csd.clip_w * scaleFactor, csd.clip_h * scaleFactor);
                            if (csd.clip_border)
                            {
                                ctx.strokeStyle = csd.clip_border;
                                ctx.stroke();
                            }
                        }

                    }
                };
            } else if (csd.clip_r && csd.clip_r > 0 && csd.clip_type == "arc")
            {
                var clip_x = csd.canvas_w / 2;
                if (csd.clip_x)
                    clip_x = csd.clip_x * scaleFactor;
                var clip_y = optimal_dimensions[1] - csd.clip_h * scaleFactor / 2;
                if (csd.clip_y)
                    clip_y = csd.clip_y;
                csd_editor.canvas.clipTo = function (ctx) {
                    ctx.arc(clip_x, clip_y, csd.clip_r * scaleFactor, 0, 2 * Math.PI);
                    if (csd.clip_border)
                    {
                        ctx.strokeStyle = csd.clip_border;
                        ctx.stroke();
                    }
                };
            }
        }

        function resize_object() {
            csd_editor.canvas.forEachObject(function (csd_object) {
                if (csd_editor.multiplier !== 1) {

                    if (csd_object.type == "i-text") {
                        var fontSize = csd_object.getFontSize();
                        var trueFont = fontSize * csd_editor.multiplier;
                        csd_object.setFontSize(trueFont);
                    } else if (csd_object.type == "image") {
                        var width = csd_object.width;
                        var height = csd_object.height;
                        var trueWidth = width * csd_editor.multiplier;
                        var trueHeight = height * csd_editor.multiplier;
                        csd_object.set("width", trueWidth);
                        csd_object.set("height", trueHeight);
                    }
                }
            });
            csd_editor.canvas.renderAll();
        }

        function scale_object_text(options) {
            if(typeof options.target === "undefined" && typeof options.type!== "undefined")
                var obj = options;
            else var obj = options.target;

            var width = obj.width;
            var height = obj.height;

            if (obj.type == "i-text") {
                var fontSize = obj.fontSize;
                fontSize = (fontSize * obj.scaleX);
                fontSize = csd_editor.get_true_dimension(fontSize);
                $("#csd-font-size").val(fontSize / csd_editor.multiplier);
            } 

            width = csd_editor.get_true_dimension(width * obj.scaleX);
            height = csd_editor.get_true_dimension(height * obj.scaleY);
            $('#csd-object-width').val(width / csd_editor.multiplier);
            $('#csd-object-height').val(height / csd_editor.multiplier);
        }

        csd_editor.getTextHeight = function () {
            var fontSize = $('#csd-font-size').val();
            var unit = $("#csd-unit-select").val();
            if ("undefined" !== typeof fontSize && "" !== fontSize) {
                fontSize = csd_editor.convert_to_px(fontSize, unit) / csd_editor.multiplier;
            } else {
                var default_size = csd.default_font_size;
                var fontSize = csd_editor.convert_to_px(default_size, unit) * csd_editor.multiplier;
                $("#csd-font-size").val(default_size);
            }
            return fontSize;
        }


        function give_dimension_or_fontsize(options) {
            var csd_type_object = options.target.type;
            if (csd_type_object == "i-text") {
                options.target.setControlsVisibility({
                    mt: false,
                    mb: false,
                    mr: false,
                    ml: false,
                });
                options.target.set({
                    padding: 0
                });
            }
            scale_object_text(options);
            csd_editor.save_canvas();

        }
        function handler(options) {
            wp.hooks.doAction('csd_EDITOR.object_selected', options);
            give_dimension_or_fontsize(options);
            if (options.target) {
                var objectType = options.target.type;
                var arr_shapes = ["rect", "circle", "triangle", "polygon", "path"];
                if (objectType == "i-text")
                {


                    tools_accordion.openPanel("text-panel");
                    wp.hooks.applyFilters('csd.add_text_selected', options);
                    $('#font-family-selector').val(options.target.get("fontFamily"));
                    $('#font-size-selector').val(options.target.get("fontSize"));
                    $('#txt-color-selector .csd-color-view').css("background-color", options.target.get("fill"));
                    $('#txt-bg-color-selector .csd-color-view').css("background-color", options.target.get("backgroundColor"));
                    $('#new-text').val(options.target.get("text"));
                    $(".txt-align[value='" + options.target.get("textAlign") + "']").attr('checked', 'checked');
                    $(".txt-decoration[value='" + options.target.get("textDecoration") + "']").attr('checked', 'checked');
                    var fontWeight = options.target.get("fontWeight");
                    if (fontWeight == "bold")
                        $("#bold-cb").attr('checked', 'checked');
                    else
                        $("#bold-cb").removeAttr('checked');
                    var fontStyle = options.target.get("fontStyle");
                    if (fontStyle == "italic")
                        $("#italic-cb").attr('checked', 'checked');
                    else
                        $("#italic-cb").removeAttr('checked');
                    if (options.target.get("stroke") != false && options.target.get("stroke") != null)
                    {
                        $('#txt-outline-color-selector').css("background-color", options.target.get("stroke"));
                        $('#o-thickness-slider').val(options.target.get("strokeWidth"));
                    } else
                    {
                        $('#o-thickness-slider').val(0);
                    }

                    var txt_opacity = options.target.opacity;
                    $("#opacity-slider").val(txt_opacity);
                } else if (objectType == "group")
                {
                    var obj = options.target.getObjects();
                    var original_text = "", text_spacing = 0, text_radius = 0, text_color = "#000";
                    obj.forEach(element => {
                        if(element.type != null && element.type=="i-text"){
                            text_spacing = element.spacing;
                            text_radius = element.radius;
                            original_text += element.text;
                            text_color = element.fill;
                        }
                    });

                    options.target.originalText = original_text;
                    options.target.spacing = text_spacing;
                    options.target.radius = text_radius;
                    //If it's a curved text, we load the first item properties (which should be the same than all other items
                    if (options.target.get("originalText"))
                    {
                        $("#cb-curved").attr('checked', 'checked');
                        tools_accordion.openPanel("text-panel");
                        $('.csd-font-view').html(options.target.get("originalText"));
                        $('#font-family-selector').val(options.target.item(0).get("fontFamily")).trigger('change');
                        $('#font-size-selector').val(options.target.item(0).get("fontSize"));
                        $('#txt-color-selector .csd-color-view').css("background-color", text_color);
                        $('#txt-bg-color-selector .csd-color-view').css("background-color", options.target.item(0).get("backgroundColor"));
                        $('#new-text').val(options.target.get("originalText"));
                        $("#curved-txt-radius-slider").val(options.target.get("radius"));
                        $("#curved-txt-spacing-slider").val(options.target.get("spacing"));
                        $(".txt-align[value='" + options.target.item(0).get("textAlign") + "']").attr('checked', 'checked');
                        $(".txt-decoration[value='" + options.target.item(0).get("textDecoration") + "']").attr('checked', 'checked');
                        var fontWeight = options.target.item(0).get("fontWeight");
                        if (fontWeight == "bold")
                            $("#bold-cb").attr('checked', 'checked');
                        else
                            $("#bold-cb").removeAttr('checked');
                        var fontStyle = options.target.item(0).get("fontStyle");
                        if (fontStyle == "italic")
                            $("#italic-cb").attr('checked', 'checked');
                        else
                            $("#italic-cb").removeAttr('checked');
                        if (options.target.item(0).get("stroke") != false)
                        {
                            $('#txt-outline-color-selector').css("background-color", options.target.item(0).get("stroke"));
                            $('#o-thickness-slider').val(options.target.item(0).get("strokeWidth"));
                        } else
                        {
                            $('#o-thickness-slider').val(0);
                        }

                        var txt_opacity = options.target.item(0).opacity;
                        $("#opacity-slider").val(txt_opacity);
                    }

                } else if (jQuery.inArray(objectType, arr_shapes) >= 0)
                {
                    var shape_opacity = options.target.opacity;
                    $("#shape-opacity-slider").val(shape_opacity);
                    $('#shape-color-selector').css("background-color", options.target.get("fill"));
                    $('#shape-outline-color-selector').css("background-color", options.target.get("stroke"));
                    $("#shape-thickness-slider").val(options.target.get("strokeWidth"));
                    wp.hooks.applyFilters('csd.add_shape_selected');
                    tools_accordion.openPanel("shapes-panel");
                } else if (objectType == "image")
                {
                    var img_src = options.target.getSrc();
                    open_src_panel(options);
                    var filters = options.target.filters;
                    $("#img-effects input:checkbox").removeAttr('checked');
                    $.each(filters, function (index, value) {
                        if (value)
                        {
                            var filter = value.type;
                            var matrix = value.matrix;
                            var blur_matrix = [1 / 9, 1 / 9, 1 / 9, 1 / 9, 1 / 9, 1 / 9, 1 / 9, 1 / 9, 1 / 9];
                            var sharpen_maxtrix = [0, -1, 0, -1, 5, -1, 0, -1, 0];
                            var emboss_matrix = [1, 1, 1, 1, 0.7, -1, -1, -1, -1];
                            if (filter == "Grayscale")
                                $(".acd-grayscale").attr('checked', 'checked');
                            else if (filter == "Invert")
                                $(".acd-invert").attr('checked', 'checked');
                            else if (filter == "Sepia")
                                $(".acd-sepia").attr('checked', 'checked');
                            else if (filter == "Sepia2")
                                $(".acd-sepia2").attr('checked', 'checked');
                            else if (filter == "Convolute")
                            {
                                if (($(matrix).not(blur_matrix).length == 0 && $(blur_matrix).not(matrix).length == 0))
                                    $(".acd-blur").attr('checked', 'checked');
                                else if (($(matrix).not(sharpen_maxtrix).length == 0 && $(sharpen_maxtrix).not(matrix).length == 0))
                                    $(".acd-sharpen").attr('checked', 'checked');
                                else if (($(matrix).not(emboss_matrix).length == 0 && $(emboss_matrix).not(matrix).length == 0))
                                    $(".acd-emboss").attr('checked', 'checked');
                            }
                        }
                    });
                    var target = options.target;
                    var image_type = open_src_panel(options);
                    wp.hooks.doAction('csd_EDITOR.image_selected', target, image_type);
                } else if ((objectType == "path" || objectType == "path-group") || (objectType == "group") && csd.svg_colorization != "none")
                {
                    var target = options.target;
                    if ((objectType == "group")) {
                        wp.hooks.applyFilters('csd.add_text_selected');
                        //csd_get_curved_state();
                    } else {
                        // var target = options.target;
                        var image_type = open_src_panel(options);
                        console.log(objectType);
                        wp.hooks.doAction('csd_EDITOR.image_selected', target, image_type);
                    }
                    var active_tab = open_src_panel(options, "cliparts-panel");

                    $("#" + active_tab + " .clipart-bg-color-container").html("");
                    if (options.target.isSameColor && options.target.isSameColor() || !options.target.paths) {
                        var color_picker_id = 'clipart-bg-' + 1 + '-color-selector';
                        var colorpicker_tpl = '<span id="' + color_picker_id + '" class="svg-color-selector" data-placement="top" data-tooltip-title="' + csd.translated_strings.svg_background_tooltip + '" style="background-color:' + options.target.get("fill") + '"></span>';
                        $("#" + active_tab + " .clipart-bg-color-container").append(colorpicker_tpl);
                        $("[data-tooltip-title]").otooltip();
                        load_svg_color_picker(color_picker_id);
                    } else if (options.target.paths) {
                        var used_colors = [];
                        var picker_index = 0;
                        for (var i = 0; i < options.target.paths.length; i++) {
                            var color_picker_id = 'clipart-bg-' + picker_index + '-color-selector';
                            var current_color = options.target.paths[i].fill;
                            var colorpicker_tpl = '<span id="' + color_picker_id + '" class="svg-color-selector" data-placement="top" data-tooltip-title="' + csd.translated_strings.svg_background_tooltip + '" style="background-color:' + current_color + '" data-index="' + i + '"></span>';
                            if (csd.svg_colorization == "by-colors")
                            {
                                var color_pos = jQuery.inArray(current_color, used_colors);
                                if (color_pos == -1)
                                {
                                    $("#" + active_tab + " .clipart-bg-color-container").append(colorpicker_tpl);
                                    $("[data-tooltip-title]").otooltip();
                                    load_svg_color_picker(color_picker_id);
                                    used_colors.push(current_color);
                                    picker_index++;
                                } else
                                {
                                    var original_picker_id = '#clipart-bg-' + color_pos + '-color-selector';
                                    var old_indexes = $(original_picker_id).attr("data-index");
                                    $(original_picker_id).attr("data-index", old_indexes + "," + i);
                                }
                            } else
                            {
                                $("#" + active_tab + " .clipart-bg-color-container").append(colorpicker_tpl);
                                $("[data-tooltip-title]").otooltip();
                                load_svg_color_picker(color_picker_id);
                                picker_index++;
                            }
                        }
                    }

                }

                if (options.target.get("lockMovementX"))
                    $("#lock-mvt-x").attr('checked', 'checked');
                else
                    $("#lock-mvt-x").removeAttr('checked');
                if (options.target.get("lockMovementY"))
                    $("#lock-mvt-y").attr('checked', 'checked');
                else
                    $("#lock-mvt-y").removeAttr('checked');
                if (options.target.get("lockScalingX"))
                    $("#lock-scl-x").attr('checked', 'checked');
                else
                    $("#lock-scl-x").removeAttr('checked');
                if (options.target.get("lockScalingY"))
                    $("#lock-scl-y").attr('checked', 'checked');
                else
                    $("#lock-scl-y").removeAttr('checked');
                if (options.target.get("lockDeletion"))
                    $("#lock-Deletion").attr('checked', 'checked');
                else
                    $("#lock-Deletion").removeAttr('checked');
            }
        }

        function load_canvas_listeners(selected_part)
        {
            wp.hooks.applyFilters('scotch.load-canvas-listeners', csd_editor);
            csd_editor.canvas.on('before:selection:cleared', function (options) {
                var image_type = open_src_panel(options);
                wp.hooks.doAction('csd_EDITOR.object_deselected', options, image_type);
            });
            csd_editor.canvas.on('selection:created', function (options) {
                handler(options);
            });

            csd_editor.canvas.on('selection:updated', function (options) {
                handler(options);
            });

            //            function csd_get_curved_state() {
            //            if ($('.csd-text-curved-wrap #cb-curved').is(':checked')) {
            //
            //                $('.csd-text-curved-wrap #cb-curved').parents('.csd-text-curved-wrap').find('.csd-text-curved-content').addClass('is-active');
            //            } else {
            //
            //                $('.csd-text-curved-wrap #cb-curved').parents('.csd-text-curved-wrap').find('.csd-text-curved-content').removeClass('is-active');
            //            }
            //        }

            csd_editor.canvas.on('object:added', function (options) {
                if (options.target) {
                    csd_editor.canvas.calcOffset();
                    csd_editor.canvas.renderAll();
                    options.target.setCoords();
                    var objectType = options.target.type;
                    if (objectType == "i-text")
                    {
                        reset_text_palette();
                    }
                    csd_editor.canvas.calcOffset();
                }
                csd_update_price();
            });

            csd_editor.canvas.on('object:modified', function (options) {
                csd_editor.canvas.calcOffset();
                csd_editor.canvas.renderAll();
                options.target.setCoords();
                give_dimension_or_fontsize(options);
                csd_editor.save_canvas();
                csd_update_price();

            });

            csd_editor.canvas.on('object:scaling', function (options) {
                if (options.target) {
                    csd_editor.canvas.calcOffset();
                    csd_editor.canvas.renderAll();
                    options.target.setCoords();
                    give_dimension_or_fontsize(options);
                    csd_update_price();
                }
                csd_editor.save_canvas();

            });
        }

        function _setCustomProperties(object)
        {
            object.toObject = (function (toObject) {
                return function () {
                    return fabric.util.object.extend(toObject.call(this), {
                        lockMovementX: this.lockMovementX,
                        lockMovementY: this.lockMovementY,
                        lockScalingX: this.lockScalingX,
                        lockScalingY: this.lockScalingY,
                        lockDeletion: this.lockDeletion,
                        price: this.price,
                        originalText: this.originalText,
                        boundingBox: this.boundingBox,
                        radius: this.radius,
                        spacing: this.spacing
                    });
                };
            })(object.toObject);
        }

        csd_editor.setCustomProperties = function (object)
        {
            _setCustomProperties(object);
        };

        csd_editor.is_json = function (data)
        {
            if (/^[\],:{}\s]*$/.test(data.replace(/\\["\\\/bfnrtu]/g, '@').
                    replace(/"[^"\\\n\r]*"|true|false|null|-?\d+(?:\.\d*)?(?:[eE][+\-]?\d+)?/g, ']').
                    replace(/(?:^|:|,)(?:\s*\[)+/g, '')))
                return true;
            else
                return false;
        };

        csd_editor.save_canvas = function ()
        {
            var data_id = $("#csd-parts-bar > li:eq(" + csd_editor.selected_part + ")").attr("data-id");
            if (typeof csd_editor.serialized_parts[data_id] == "undefined")
                csd_editor.serialized_parts[data_id] = ["{}"];
            var i;
            for (i = csd_editor.canvasManipulationsPosition[data_id]; i <= csd_editor.serialized_parts[data_id].length - 2; i++)
            {
                csd_editor.serialized_parts[data_id].pop();
            }

            csd_editor.canvasManipulationsPosition[data_id]++;
            var json = JSON.stringify(csd_editor.canvas.toJSON(['lockMovementX', 'lockMovementY', 'lockRotation', 'lockScalingX', 'lockScalingY', 'price', 'lockDeletion', 'originalText', 'radius', 'spacing']));
            csd_editor.serialized_parts[data_id].push(json);
            csd_editor.refresh_undo_redo_status();
        };

        csd_editor.reload_background_overlay_if_needed = function (index, callback)
        {
            load_background_overlay_if_needed(index, callback);
        };

        csd_editor.change_item_color = function (id, hex)
        {
            id = id + " .csd-color-view";
            $('#' + id).css('background-color', hex);
            var selected_object = csd_editor.canvas.getActiveObject();
            if ((selected_object != null) && (selected_object.type != "group"))
            {
                csd_set_color(id, selected_object, hex);
            } else if ((selected_object != null) && (selected_object.type == "group"))
            {
                var i = 0;
                selected_object.forEachObject(function (a) {
                    i+=1;
                    if(i == 1) csd_set_color(id, a, hex);
                    else csd_set_color(id, a, hex, false);
                });
            }
        };

        function open_src_panel(options, default_panel)
        {
            var obj = options.target.toObject();
            var img_src = obj.src;
            var in_cliparts = $("#csd-all-cliparts img[src='" + img_src + "']").length;
            var in_facebook = $("#facebook-panel img[src='" + img_src + "']").length;
            var in_instagram = $("#instagram-panel img[src='" + img_src + "']").length;
            var in_uploads = $("#uploads-panel img[src='" + img_src + "']").length;
            if (in_cliparts)
                tools_accordion.openPanel("cliparts-panel");
            else if (in_facebook)
                tools_accordion.openPanel("facebook-panel");
            else if (in_instagram)
                tools_accordion.openPanel("instagram-panel");
            else if (!in_uploads && typeof default_panel != "undefined")
                tools_accordion.openPanel(default_panel);
            else
                tools_accordion.openPanel("uploads-panel");
            if (in_cliparts)
                return "cliparts-panel";
            else if (in_facebook)
                return "facebook-panel";
            else if (in_instagram)
                return "instagram-panel";
            else if (!in_uploads && typeof default_panel != "undefined")
                return default_panel;
            else
                return "uploads-panel";
        }


        function csd_set_color(id, selected_object, hex, save_color = true)
        {
            if ((id == "txt-color-selector .csd-color-view") || (id == "shape-color-selector") || (id == "clipart-bg-color-selector")) {
                if (jQuery.type(hex) === "object") {
                    hex = hex.toRgbString();
                }

                selected_object.set("fill", hex);

                if(save_color){
                    var i = 7;
                    $.each(csd_editor.color_values, function (key, value) {
                        if (value !== hex) {
                            csd_editor.color_values[i] = csd_editor.color_values[i - 1];
                            if (i == 0) {
                                csd_editor.color_values[0] = hex;
                            }
                            i--;
                        }
                    });
                    $.each(csd_editor.color_values, function (key, value) {
                        $('#csd_color_' + key).css('background-color', value);
                    });
                }
                
            } else if (id == "txt-bg-color-selector .csd-color-view")
                selected_object.set("textBackgroundColor", hex);
            else if (id == "txt-outline-color-selector")
                selected_object.set("stroke", hex);
            wp.hooks.doAction('csd_EDITOR.element_color_changed', id, selected_object, hex);
            csd_editor.canvas.renderAll();
            csd_update_price();
            csd_editor.save_canvas();
        }

        csd_editor.refresh_undo_redo_status = function ()
        {
            var data_id = $("#csd-parts-bar > li:eq(" + csd_editor.selected_part + ")").attr("data-id");
            if ((csd_editor.serialized_parts[data_id].length == 1) || (csd_editor.canvasManipulationsPosition[data_id] == 0))
                $("#undo-btn").addClass("disabled");
            else
                $("#undo-btn").removeClass("disabled");
            if ((csd_editor.serialized_parts[data_id].length > 0) && (csd_editor.canvasManipulationsPosition[data_id] < csd_editor.serialized_parts[data_id].length - 1))
                $("#redo-btn").removeClass("disabled");
            else
                $("#redo-btn").addClass("disabled");
        };

        function init_empty_canvas_data_array()
        {
            if (typeof to_load == 'undefined')
            {
                $("#csd-parts-bar > li").each(function (key) {
                    var data_id = $(this).attr("data-id");
                    csd_editor.serialized_parts[data_id] = [];
                    csd_editor.canvasManipulationsPosition[data_id] = -1;
                    var nb_parts = $("#csd-parts-bar > li").length;
                    if (key == nb_parts - 1)
                    {
                        loop_through_parts(csd.output_loop_delay, click_on_part,
                                function () {
                                    $("#csd-parts-bar > li").first().click();
                                    csd_editor.canvas.renderAll();
                                    rescale_canvas_if_needed();
                                    $.unblockUI();
                                });
                    }
                });
            }
        }

        function click_on_part(part_index)
        {
            $("#csd-parts-bar > li:eq(" + part_index + ")").click();
        }

        function loop_through_parts(delay, loop_callback, end_callback)
        {
            $.blockUI({message: csd.translated_strings.loading_msg});
            var nb_parts = $("#csd-parts-bar > li").length;
            var current_part = 0;
            var loopKey = setInterval(function () {
                if ($.isFunction(loop_callback))
                    loop_callback(current_part);

                if (current_part == nb_parts - 1)
                {
                    window.clearInterval(loopKey);
                    if ($.isFunction(end_callback))
                    {
                        
                        setTimeout(function () {
                            end_callback();
                        }, delay);
                    } else
                        $.unblockUI();
                } else
                    current_part++;
            }, delay);
        }

        var setBackgroundOrOverfayFromDataUrl = (dataUrl, maxWidth, maxHeight, overlay = false) => {
            if (!dataUrl) {
                return true;
            }

            var img = new Image();
            img.setAttribute('crossOrigin', 'anonymous');
            img.onload = () => {

                // maxWidth // Max width of the canvas
                // maxHeight //Max height of canvas

                var imgWidth = img.width; // Current image width
                var imgHeight = img.height; // Current image height

                var widthAspectRatio = maxWidth / imgWidth;
                var heightAspectRatio = maxHeight / imgHeight;

                var finalAspectRatio = Math.min(widthAspectRatio, heightAspectRatio)

                var finalHeight = imgHeight * finalAspectRatio;
                var finalWidth = imgWidth * finalAspectRatio;

                var imgTop = 0;
                if (maxHeight > finalHeight) {
                    imgTop = (Math.round(maxHeight) - Math.round(finalHeight)) / 2;
                }

                var imgLeft = 0;
                if (maxWidth > finalWidth) {
                    imgLeft = (Math.round(maxWidth) - Math.round(finalWidth)) / 2;
                }

                var nsgImage = new fabric.Image(img).scale(finalAspectRatio);
                nsgImage.set({left: imgLeft, top: imgTop});
                if (!overlay)
                {
                    csd_editor.canvas.setBackgroundImage(nsgImage, () => csd_editor.canvas.renderAll());
                } else
                {
                    csd_editor.canvas.setOverlayImage(nsgImage, () => csd_editor.canvas.renderAll());
                }

            };

            img.src = dataUrl;

        };

        function load_background_overlay_if_needed(index, callback, generating_output)
        {
            var selector = $("#csd-parts-bar > li:eq(" + index + ")");
            var overlay_not_included = selector.attr("data-ovni");
            if (typeof generating_output == 'undefined')
                generating_output = false;
            var canvas_bg = selector.data("bg");
            if (canvas_bg == "")
                canvas_bg = null;
            var canvas_ov = selector.data("ov");
            if (canvas_ov == "")
                canvas_ov = null;

            if (canvas_bg != null) {
                setBackgroundOrOverfayFromDataUrl(canvas_bg, csd_editor.canvas.width, csd_editor.canvas.height);
            } else
                csd_editor.canvas.backgroundImage = null;

            if (overlay_not_included == "-1" && generating_output)
            {
                //White bg if CMYK mode
                if (csd.output_format == "jpg")
                {
                    csd_editor.canvas.setBackgroundColor("rgba(255, 255, 255, 1)", csd_editor.canvas.renderAll.bind(csd_editor.canvas));
                }
                csd_editor.canvas.overlayImage = null;
                csd_editor.canvas.renderAll.bind(csd_editor.canvas);
            } else
            {
                //White bg if CMYK mode
                if (csd.output_format == "jpg")
                {
                    csd_editor.canvas.setBackgroundColor("", csd_editor.canvas.renderAll.bind(csd_editor.canvas));
                }
                var ov_img = new Image();
                ov_img.onload = function () {
                    var dimensions = csd_editor.get_img_best_fit_dimensions(ov_img, csd.canvas_w, csd.canvas_h);
                    csd_editor.canvas.setOverlayImage(ov_img.src, csd_editor.canvas.renderAll.bind(csd_editor.canvas), {
                        left: csd.canvas_w / 2,
                        top: csd.canvas_h / 2,
                        originX: 'center',
                        originY: 'center',
                        width: dimensions[0],
                        height: dimensions[1]
                    });
                };
                if (canvas_ov != null)
                    ov_img.src = canvas_ov;
                else
                    csd_editor.canvas.overlayImage = null;
            }

            if ($.isFunction(callback))
                setTimeout(function () {
                    callback(index);
                }, 200);
        }

        csd_editor.get_img_best_fit_dimensions = function (img, max_width, max_height)
        {
            var w = img.width;
            var h = img.height;
            if (w < max_width && h < max_height)
                return [w, h];
            var ratio = w / h;
            w = max_width;
            h = max_width / ratio;
            if (h > max_height)
            {
                h = max_height;
                w = max_height * ratio;
            }
            return wp.hooks.applyFilters('csd_EDITOR.csd_img_dimensions', [w, h], img);
        };

        function load_svg_color_picker(id)
        {
            var selector = $('#' + id);
            var index = selector.data("index");
            var initial_color = selector.css("background-color");
            if (!initial_color)
                initial_color = "#0000ff";
            if (csd.palette_type == "custom")
            {
                selector.qtip({
                    content: "<div class='csd-custom-svg-colors-container' data-id='" + id + "' data-index='" + index + "'>" + csd.palette_tpl + "</div>",
                    position: {
                        corner: {
                            target: 'middleRight',
                            tooltip: 'leftTop'
                        }
                    },
                    style: {
                        width: 200,
                        padding: 5,
                        background: 'white',
                        color: 'black',
                        border: {
                            width: 1,
                            radius: 1,
                            color: '#08AED6'
                        }
                    },
                    tip: 'bottomLeft',
                    show: 'click',
                    hide: {when: {event: 'unfocus'}}
                });
            } else
            {
                selector.ColorPicker({
                    color: initial_color,
                    onShow: function (colpkr) {
                        $(colpkr).fadeIn(500);
                        return false;
                    },
                    onHide: function (colpkr) {
                        $(colpkr).fadeOut(500);
                        var selected_object = csd_editor.canvas.getActiveObject();
                        if ((selected_object != null))
                        {
                            csd_editor.save_canvas();
                        }
                        return false;
                    },
                    onChange: function (hsb, hex, rgb) {
                        change_svg_color(id, hex, index);
                    }
                });
            }
        }

        function change_svg_color(id, hex, index)
        {
            $('#' + id).css('background-color', '#' + hex);
            var selected_object = csd_editor.canvas.getActiveObject();
            if ((selected_object != null) && (selected_object.type == "path" || selected_object.type == "path-group" || selected_object.type == "group"))
            {
                {
                    if (selected_object.isSameColor && selected_object.isSameColor() || !selected_object.paths) {
                        selected_object.set("fill", '#' + hex);
                    } else if (selected_object.paths) {
                        if (csd.svg_colorization == "by-colors")
                        {
                            index = $("#" + id).attr("data-index");
                            var indexes = index.split(',');
                            $.each(indexes, function (key, value)
                            {
                                selected_object.paths[value].setFill('#' + hex);
                            });
                        } else
                            selected_object.paths[index].setFill('#' + hex);
                    }
                }
                csd_editor.canvas.renderAll();
            }
        }

        $(document).on("click", ".csd-custom-colors-container span", function (e) {
            var id = $(this).parent().data("id");
            var hex = $(this).data("color");
            csd_editor.change_item_color(id, hex);
        });

        $(document).on("click", ".csd-previous-color span", function (e) {
            var id = 'txt-color-selector';
            var hex = $(this).css("background-color");
            csd_editor.change_item_color(id, hex);
        });

        $(document).on("click", ".csd-custom-svg-colors-container span", function (e) {
            var id = $(this).parent().data("id");
            var index = $(this).parent().data("index");
            var hex = $(this).data("color");
            change_svg_color(id, hex, index);
        });

        function reset_text_palette()
        {
            $("#new-text").val("");
            $(".txt-align").removeAttr('checked');
            $(".txt-decoration").removeAttr('checked');
            $("#bold-cb").removeAttr('checked');
            $("#italic-cb").removeAttr('checked');
            $("#font-family-selector").val($("#font-family-selector option:first").val()).trigger('change');
            ;
            $("#o-thickness-slider").val($("#o-thickness-slider option:first").val());
            $("#opacity-slider").val(1);
        }

        function not_yet_implemented(e) {
            e.preventDefault();
            alert("Not yet implemented");
            return;
        }

        function reload_first_part_data()
        {
            var data_id = $("#csd-parts-bar > li:eq(0)").attr("data-id");
            csd_editor.canvas.clear();
            csd_editor.canvas.loadFromJSON(csd_editor.serialized_parts[data_id][csd_editor.canvasManipulationsPosition[data_id]], function () {
                csd_editor.canvas.renderAll.bind(csd_editor.canvas);
                rescale_canvas_if_needed();
            });
        }


        //Download design
        $(document).on("click", "#download-btn", function (e)
        {
            $("#debug").html("");
            if (csd.clip_include_in_output == "no")
                csd_editor.canvas.clipTo = null;
            var product_id = $('#csd_add_to_cart').attr("data-id");
            var canvas_w = $('#csd-canvas-width').val();
            var canvas_h = $('#csd-canvas-height').val();
            loop_through_parts(csd.output_loop_delay,
                    generate_final_canvas_part,
                    function () {
                        if (jQuery.isEmptyObject(csd_editor.final_canvas_parts))
                        {
                            $("#debug").html("<div class='csd-failure'>" + csd.translated_strings.empty_object_msg + "</div>");
                            set_clipping_area();
                            $.unblockUI();
                        } else {

                            var frm_data = new FormData();
                            if (typeof tpl === 'undefined') {
                                var tpl = "";
                            }
                            frm_data.append("tpl", tpl);
                            frm_data.append('product_id', product_id);
                            if (typeof csd_editor.signs_width !== 'undefined' || typeof csd_editor.signs_height !== 'undefined') {
                                frm_data.append("canvas_w", csd_editor.signs_width);
                                frm_data.append("canvas_h", csd_editor.signs_height);
                            } else {
                                frm_data.append("canvas_w", canvas_w);
                                frm_data.append("canvas_h", canvas_h);
                            }
                            frm_data.append("multiplier", csd_editor.multiplier);
                            frm_data.append("final_canvas_parts", csd_editor.final_canvas_parts);
                            frm_data = convert_final_canvas_parts_to_blob(frm_data);
                            frm_data.append('action', "csd_generate_downloadable_file");
                            if (typeof window.csd_key !== 'undefined') {
                                frm_data.append("csd_key", window.csd_key);
                            }
                            $.ajax({
                                type: 'POST',
                                url: csd_ajax_object.csd_ajax_url,
                                data: frm_data,
                                processData: false,
                                contentType: false
                            }).done(function (data) {
                                set_clipping_area();
                                $.unblockUI();
                                if (csd_editor.is_json(data))
                                {
                                    var response = JSON.parse(data);
                                    if ($("#csd-parts-bar > li").length > 1)
                                    {
                                        $("#csd-parts-bar > li").first().click();
                                    } else
                                        reload_first_part_data();

                                    $("#debug").html(response.message);
                                } else
                                {

                                    $("#debug").html(data);

                                }

                            });

                        }
                    }
            );
        });

        //Save design for later
        $(document).on("touchstart click", "#save-btn", function (e)
        {
            not_yet_implemented(e);
            return;
        });

        $('.csd-rp-attribute.cart-item-edit').click(function () {
            return confirm(csd.translated_strings.cart_item_edition_switch);
        });

        //Generate design for output
        function generate_final_canvas_part(part_index)
        {
            generate_canvas_part(part_index, false);
        }

        function generate_canvas_part(part_index, preview)
        {
            csd_editor.selected_part = part_index;
            preview = typeof preview !== 'undefined' ? preview : true;

            var data_id = $("#csd-parts-bar > li:eq(" + part_index + ")").attr("data-id");
            var data_part_img = $("#csd-parts-bar > li:eq(" + part_index + ")").attr("data-url");
            if (typeof csd_editor.serialized_parts[data_id] == "undefined")
            {
                csd_editor.serialized_parts[data_id] = ["{}"];
            }
            csd_editor.save_canvas();
            csd_editor.canvas.loadFromJSON(csd_editor.serialized_parts[data_id][csd_editor.canvasManipulationsPosition[data_id]],
                    function () {
                        csd_editor.applyImageFilters();
                        load_background_overlay_if_needed(csd_editor.selected_part, function () {

                            var multiplier = csd.output_w / csd_editor.canvas.width;
                            if (preview)
                                multiplier = 1;
                            //We split the multiplier per 2 if we're in retina mode
                            if (window.devicePixelRatio !== 1 && csd.responsive != 1 && csd.enable_retina === "yes") {
                                multiplier = multiplier / 2;
                            }
                            multiplier = 1 / csd_editor.multiplier;
                            var image = csd_editor.canvas.toDataURL({format: csd.output_format, multiplier: multiplier, quality: 1});
                            var svg = "";
                            if (csd.generate_svg)
                                svg = csd_editor.canvas.toSVG();
                            var blob_image = dataURItoBlob(image);
                            if (preview)
                            {

                                var modal_content = "";
                                if (csd.watermark)
                                {
                                    var frm_data = new FormData();
                                    frm_data.append("action", "get_watermarked_preview");
                                    frm_data.append("format", csd.output_format);
                                    frm_data.append("image", blob_image);

                                    $.ajax({
                                        type: 'POST',
                                        url: ajax_object.ajax_url,
                                        data: frm_data,
                                        processData: false,
                                        contentType: false
                                    }).done(function (data) {
                                        if (csd_editor.is_json(data))
                                        {
                                            var response = JSON.parse(data);
                                            if (data_part_img)
                                                modal_content = "<div style='background-image:url(" + data_part_img + ");'><img src='" + response.url + "'></div>";
                                            else
                                                modal_content = "<div><img src='" + response.url + "'></div>";
                                            $("#csd-modal .omodal-body").append(modal_content);
                                        } else
                                        {
                                            $("#debug").html(data);
                                        }

                                    });
                                } else
                                {
                                    if (data_part_img)
                                        modal_content = "<div style='background-image:url(" + data_part_img + ");'><img src='" + image + "'></div>";
                                    else
                                        modal_content = "<div><img src='" + image + "'></div>";
                                    $("#csd-modal .omodal-body").append(modal_content);
                                }
                            } else
                            {

                                var canvas_obj = $.parseJSON(csd_editor.serialized_parts[data_id][csd_editor.canvasManipulationsPosition[data_id]]);
                                var layers = [];
                                if (csd.print_layers)
                                {
                                    var objects = canvas_obj.objects;
                                    $.each(objects, function (key, curr_object) {
                                        var tmp_canvas_obj = canvas_obj;
                                        tmp_canvas_obj.objects = [curr_object];
                                        var tmp_canvas_json = JSON.stringify(tmp_canvas_obj);
                                        csd_editor.canvas.loadFromJSON(tmp_canvas_json, function () {
                                            csd_editor.applyImageFilters();
                                            csd_editor.canvas.renderAll.bind(csd_editor.canvas);
                                            //Removes overlay not included from layers
                                            load_background_overlay_if_needed(csd_editor.selected_part, "", true);
                                            var multiplier = csd.output_w / csd_editor.canvas.width;
                                            var layer = csd_editor.canvas.toDataURL({format: csd.output_format, multiplier: multiplier, quality: 1});
                                            var blob_layer = dataURItoBlob(layer);
                                            layers.push(blob_layer);
                                            //Loads the complete canvas before the save later otherwise, we end up with the last layer loaded as part data
                                            if (key == objects.length - 1)
                                            {
                                                csd_editor.save_canvas();
                                                csd_editor.canvas.loadFromJSON(csd_editor.serialized_parts[data_id][csd_editor.canvasManipulationsPosition[data_id]]);
                                                csd_editor.applyImageFilters();
                                            }
                                        });
                                    });
                                }
                                csd_editor.save_canvas();
                                csd_editor.final_canvas_parts[data_id] = {json: csd_editor.serialized_parts[data_id][csd_editor.canvasManipulationsPosition[data_id]], image: blob_image, original_part_img: data_part_img, layers: layers, svg: svg};
                            }
                            load_background_overlay_if_needed(csd_editor.selected_part);
                        }, true);
                    });
        }

        function dataURItoBlob(dataURI) {
            // convert base64/URLEncoded data component to raw binary data held in a string
            var byteString;
            if (dataURI.split(',')[0].indexOf('base64') >= 0)
                byteString = atob(dataURI.split(',')[1]);
            else
                byteString = unescape(dataURI.split(',')[1]);
            // separate out the mime component
            var mimeString = dataURI.split(',')[0].split(':')[1].split(';')[0];
            // write the bytes of the string to a typed array
            var ia = new Uint8Array(byteString.length);
            for (var i = 0; i < byteString.length; i++) {
                ia[i] = byteString.charCodeAt(i);
            }

            var blob = new Blob([ia], {type: mimeString});
            return blob;
        }

        function convert_final_canvas_parts_to_blob(frm_data)
        {
            $.each(csd_editor.final_canvas_parts, function (part_key, part_data) {
                $.each(part_data, function (data_key, data_value) {
                    if (data_key == "image")
                        frm_data.append(part_key + "[" + data_key + "]", data_value);
                    else if (data_key == "layers")
                    {
                        $.each(data_value, function (layer_index, layer_data) {
                            frm_data.append("layers[" + part_key + "][]", layer_data);
                        });
                    } else
                        frm_data.append("final_canvas_parts[" + part_key + "][" + data_key + "]", data_value);
                });
            });
            return frm_data;
        }

        function rescale_canvas_if_needed()
        {
            if (csd.responsive != 1)
                return false;
            var optimal_dimensions = csd_editor.get_optimal_canvas_dimensions();
            var scaleFactor = optimal_dimensions[0] / csd.canvas_w;
            if (scaleFactor != 1) {
                csd_editor.scale_factor = scaleFactor;
                csd_editor.canvas.set("width", optimal_dimensions[0]);
                csd_editor.canvas.set("height", optimal_dimensions[1]);
                csd_editor.canvas.setZoom(scaleFactor);
                csd_editor.canvas.calcOffset();
                csd_editor.canvas.renderAll();
            }

            csd_editor.applyImageFilters();
        }

        $(window).resize(function () {
            clearTimeout(resizeId);
            resizeId = setTimeout(handle_resize, 500);
        });
        function handle_resize()
        {
            $(".canvas-container").hide();
            rescale_canvas_if_needed();
            set_clipping_area();
            $(".canvas-container").show();
        }

        //Shortcuts
        if (parseInt(csd.disable_shortcuts) != 1)
        {
            $(document).keydown(function (e) {
                var selected_object = csd_editor.canvas.getActiveObject();
                var selected_group;
                if (typeof selected_object !== "undefined" && selected_object !== null) {
                    if (selected_object.type == "activeSelection")
                        selected_group = selected_object;
                    if (e.which == 46) //Delete button
                        $("#delete_btn").click();
                    else if (e.which == 37) //Left button
                    {
                        if (typeof selected_group !== "undefined" && !selected_group.get("lockMovementX"))
                        {
                            selected_group.set("left", selected_group.left - 1);
                            csd_editor.canvas.renderAll();
                            csd_editor.save_canvas();
                        } else if ((typeof selected_object !== null) && !selected_object.get("lockMovementX"))
                        {
                            selected_object.set("left", selected_object.left - 1);
                            csd_editor.canvas.renderAll();
                            csd_editor.save_canvas();
                        }

                    } else if (e.which == 39) //Right button
                    {
                        if (typeof selected_group !== "undefined" && !selected_group.get("lockMovementX"))
                        {
                            selected_group.set("left", selected_group.left + 1);
                            csd_editor.canvas.renderAll();
                            csd_editor.save_canvas();
                        } else if ((selected_object != null) && !selected_object.get("lockMovementX"))
                        {
                            selected_object.set("left", selected_object.left + 1);
                            csd_editor.canvas.renderAll();
                            csd_editor.save_canvas();
                        }
                    } else if (e.which == 38) //Top button
                    {

                        if (typeof selected_group !== "undefined" && !selected_group.get("lockMovementY"))
                        {
                            e.preventDefault();
                            selected_group.set("top", selected_group.top - 1);
                            csd_editor.canvas.renderAll();
                            csd_editor.save_canvas();
                        } else if ((selected_object != null) && !selected_object.get("lockMovementY"))
                        {
                            e.preventDefault();
                            selected_object.set("top", selected_object.top - 1);
                            csd_editor.canvas.renderAll();
                            csd_editor.save_canvas();
                        }
                    } else if (e.which == 40) //Bottom button
                    {
                        if (typeof selected_group !== "undefined" && !selected_group.get("lockMovementY"))
                        {
                            e.preventDefault();
                            selected_group.set("top", selected_group.top + 1);
                            csd_editor.canvas.renderAll();
                            csd_editor.save_canvas();
                        } else if ((selected_object != null) && !selected_object.get("lockMovementY"))
                        {
                            e.preventDefault();
                            selected_object.set("top", selected_object.top + 1);
                            csd_editor.canvas.renderAll();
                            csd_editor.save_canvas();
                        }

                    } else if (e.keyCode == 67 && e.ctrlKey)//ctrl+c
                    {
                        $("#copy_paste_btn").click();
                    } else if (e.keyCode == 90 && e.ctrlKey)//ctrl+z
                    {
                        $("#undo-btn").click();
                    } else if (e.keyCode == 89 && e.ctrlKey)//ctrl+y
                    {
                        $("#redo-btn").click();
                    }
                }
            });
        }

        $('canvas').bind('contextmenu', function (e) {
            return false;
        });

        $(document).on('click', '[id$="color-selector"] .o-colorpicker_clear', function (event) {
            var parent_id = $(this).parents(".csd-colorpicker").attr("id");
            var selected_object = csd_editor.canvas.getActiveObject();
            if (parent_id.indexOf("bg-color-selector") >= 0 && selected_object != null)
            {
                selected_object.set("backgroundColor", "transparent");
                csd_editor.canvas.renderAll();
                csd_editor.save_canvas();
            } else if (parent_id.indexOf("outline-color-selector") >= 0 && selected_object != null)
            {
                selected_object.set("stroke", "transparent");
                csd_editor.canvas.renderAll();
                csd_editor.save_canvas();
            } else if (parent_id.indexOf("color-selector") >= 0 && selected_object != null)
            {
                selected_object.set("fill", "transparent");
                csd_editor.canvas.renderAll();
                csd_editor.save_canvas();
            }
            csd_update_price();
        });

        /**
         * Draws a rounded rectangle using the current state of the canvas.
         */
        function roundRect(ctx, x, y, width, height, radius, fill, stroke) {
            if (typeof stroke == "undefined") {
                stroke = false;
            }
            if (typeof radius === "undefined") {
                radius = 5;
            }
            ctx.beginPath();
            ctx.moveTo(x + radius, y);
            ctx.lineTo(x + width - radius, y);
            ctx.quadraticCurveTo(x + width, y, x + width, y + radius);
            ctx.lineTo(x + width, y + height - radius);
            ctx.quadraticCurveTo(x + width, y + height, x + width - radius, y + height);
            ctx.lineTo(x + radius, y + height);
            ctx.quadraticCurveTo(x, y + height, x, y + height - radius);
            ctx.lineTo(x, y + radius);
            ctx.quadraticCurveTo(x, y, x + radius, y);
            ctx.closePath();
            if (stroke) {
                ctx.strokeStyle = stroke;
                ctx.stroke();
            }
            if (fill) {
                ctx.fill();
            }
        }

        $(document).on("click", "#txt-bg-color-selector-none", function (e)
        {
            if (null !== csd_editor.canvas.getActiveObject()) {
                csd_editor.canvas.getActiveObject().set("backgroundColor", "transparent");
                csd_editor.canvas.renderAll();
            }
        });


        $('.csd-custom-select-wrap ul li').on('click', function () {
            var previousUnit = $('#csd-unit-select').val();
            var newUnit = $(this).text();
            var canvas_w = parseFloat($('#csd-canvas-width').val());
            var canvas_h = parseFloat($('#csd-canvas-height').val());
            var object_w = parseFloat($("#csd-object-width").val());
            var object_h = parseFloat($("#csd-object-height").val());
            var font_size = parseFloat($("#csd-font-size").val());
            if (previousUnit !== newUnit) {

                canvas_w = csd_editor.convert_to_px(canvas_w, previousUnit);
                canvas_h = csd_editor.convert_to_px(canvas_h, previousUnit);

                csd_editor.signs_width = canvas_w;
                csd_editor.signs_height = canvas_h;

                object_w = csd_editor.convert_to_px(object_w, previousUnit);
                object_h = csd_editor.convert_to_px(object_h, previousUnit);

                font_size = csd_editor.convert_to_px(font_size, previousUnit);

                canvas_w = parseFloat(csd_editor.get_true_dimension(canvas_w, newUnit));
                canvas_h = parseFloat(csd_editor.get_true_dimension(canvas_h, newUnit));
                object_w = parseFloat(csd_editor.get_true_dimension(object_w, newUnit));
                object_h = parseFloat(csd_editor.get_true_dimension(object_h, newUnit));
                font_size = parseFloat(csd_editor.get_true_dimension(font_size, newUnit));

            }
            $('#csd-canvas-width').val(canvas_w);
            $('#csd-canvas-height').val(canvas_h);
            $("#csd-object-width").val(object_w);
            $("#csd-object-height").val(object_h);
            $("#csd-font-size").val(font_size);
            $("#csd-unit-select").val(newUnit);

            if(newUnit == "Feet")
                var csd_step = 0.01;
            else var csd_step = 0.5;

            $(".csd-text-size").attr("step", csd_step);
            $(".csd-canvas-size").attr("step", csd_step);
            $(".csd-object-size").attr("step", csd_step);
        });

        function update_dimension_with_width(width, height, outerWidth) {
            if (typeof width != 'undefined' && typeof height != 'undefined' && typeof outerWidth != 'undefined') {
                var c_width;
                var c_height;
                var multiplier = parseFloat(outerWidth / width).toFixed(2);
                c_width = outerWidth;
                c_height = height * multiplier;
                csd_editor.multiplier = multiplier;
                return [c_width, c_height];

            }
            return false;
        }

        $(".csd-canvas-size, .csd-object-size, .csd-text-size").bind("change", function () {
            if ($(this).data('previousValue') !== $(this).val()) {
                var is_active;
                var canvas_w;
                var canvas_h;
                var update_width;
                var update_height;
                var ratio;
                var unit = $("#csd-unit-select").val();
                var this_class=$(this).attr('class');
                var tmp=this_class.split(" ");
                this_class=tmp[0];
                if (this_class === 'csd-object-size') {
                    is_active = $('.csd-option-ratio-object').hasClass('active');
                    var options = csd_editor.canvas.getActiveObject();

                    if (options != null) {
                        canvas_w = options.width;
                        canvas_h = options.height;
                        if ($(this).attr('id') === 'csd-object-width') {
                            update_width = $('#csd-object-width').val();
                            if (is_active) {
                                ratio = canvas_h / canvas_w;
                                update_height = update_width * ratio;
                            } else {
                                update_height = $('#csd-object-height').val();
                            }

                        } else if ($(this).attr('id') === 'csd-object-height') {
                            update_height = $('#csd-object-height').val();
                            if (is_active) {
                                ratio = canvas_w / canvas_h;
                                update_width = update_height * ratio;
                            } else {
                                update_width = $('#csd-object-width').val();
                            }

                        }

                        $('#csd-object-width').val(update_width);
                        $('#csd-object-height').val(update_height);

                        update_width = csd_editor.convert_to_px(update_width, unit);
                        update_height = csd_editor.convert_to_px(update_height, unit);

                        var trueWidth = update_width * csd_editor.multiplier;
                        var trueHeight = update_height * csd_editor.multiplier;

                        //fontSize = csd_editor.get_true_dimension(fontSize);

                        if(options.type == "image"){
                            options.set({
                                scaleX:trueWidth / options.width,
                                scaleY:trueHeight / options.height
                            })
                        }

                        options.setCoords();
                        csd_editor.canvas.renderAll();
                        csd_update_price();
                        csd_editor.save_canvas();
                    }

                } else if (this_class === 'csd-canvas-size') {
                    is_active = $('.csd-option-ratio-canvas').hasClass('active');
                    canvas_w = csd_editor.canvas.width;
                    canvas_h = csd_editor.canvas.height;

                    if ($(this).attr('id') === 'csd-canvas-width') {
                        update_width = $('#csd-canvas-width').val();
                        if (is_active) {
                            ratio = canvas_h / canvas_w;
                            update_height = update_width * ratio;
                        } else {
                            update_height = $('#csd-canvas-height').val();
                        }

                    } else if ($(this).attr('id') === 'csd-canvas-height') {
                        update_height = $('#csd-canvas-height').val();
                        if (is_active) {
                            ratio = canvas_w / canvas_h;
                            update_width = update_height * ratio;
                        } else {
                            update_width = $('#csd-canvas-width').val();
                        }
                    }

                    $('#csd-canvas-width').val(update_width);
                    $('#csd-canvas-height').val(update_height);
                    $('#sumamary-width').text(update_width);
                    $('#sumamary-height').text(update_height);

                    var signs_width;
                    var signs_height;

                    csd_editor.signs_width = signs_width = csd_editor.convert_to_px(update_width, unit);
                    csd_editor.signs_height = signs_height = csd_editor.convert_to_px(update_height, unit);

                    if (signs_width > 15000 || signs_height > 15000) {
                        if (unit == "Inch") {
                            alert('The size of the canvas should not exceed 50in X 50in');
                            $('#csd-canvas-width').val(50);
                            $('#csd-canvas-height').val(50);
                        }
                        if (unit == "Cm") {
                            alert('The size of the canvas should not exceed 125cm X 125cm');
                            $('#csd-canvas-width').val(125);
                            $('#csd-canvas-height').val(125);
                        }
                        if (unit == "Feet") {
                            alert('The size of the canvas should not exceed 4ft X 4ft');
                            $('#csd-canvas-width').val(4);
                            $('#csd-canvas-height').val(4);
                        }
                        if (unit == "Px") {
                            alert('The size of the canvas should not exceed 15000px X 15000px');
                            $('#csd-canvas-width').val(15000);
                            $('#csd-canvas-height').val(15000);
                        }
                        $(".csd-canvas-size").bind("change");
                    }

                    var max_width = $(".csd-editor-content").width();
                    var csd_optimal_dimension;

                    if (signs_width > max_width) {
                        csd_optimal_dimension = update_dimension_with_width(signs_width, signs_height, max_width);
                        csd_editor.canvas.setDimensions({width: csd_optimal_dimension[0], height: csd_optimal_dimension[1]});
                    } else {
                        csd_editor.canvas.setDimensions({width: signs_width, height: signs_height});
                    }
                    csd_editor.canvas.renderAll();
                    csd_update_price();
                    csd_editor.save_canvas();

                } else if (this_class === 'csd-text-size') {
                    var options = csd_editor.canvas.getActiveObject();
                    if (options != null) {
                        var csd_fontSize = $('#csd-font-size').val();
                        csd_fontSize = csd_editor.convert_to_px(csd_fontSize, unit);
                        if (csd_editor.multiplier !== 1) {
                            var trueFont = csd_fontSize * csd_editor.multiplier;
                            options.set("fontSize", trueFont);
                        } else {
                            options.set("fontSize", csd_fontSize);
                        }

                        csd_editor.canvas.renderAll();
                        csd_update_price();
                        csd_editor.save_canvas();
                    }

                }
            }

        });

        /*
         * Count the number of occurrences in a string of characters
         */
        function occurrences(string, substring) {
            var n = 0;
            var pos = 0;

            while (true) {
                pos = string.indexOf(substring, pos);
                if (pos !== -1) {
                    n++;
                    pos += substring.length;
                } else {
                    break;
                }
            }
            return(n + 1);
        }

        var hexDigits = new Array("0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "a", "b", "c", "d", "e", "f");

        //Function to convert rgb color to hex format
        function rgb2hex(rgb) {
            rgb = rgb.match(/^rgb\((\d+),\s*(\d+),\s*(\d+)\)$/);
            if (rgb != null) {
                return "#" + hex(rgb[1]) + hex(rgb[2]) + hex(rgb[3]);
            } else {
                return false;
            }
        }

        function hex(x) {
            return isNaN(x) ? "00" : hexDigits[(x - x % 16) / 16] + hexDigits[x % 16];
        }

        /*
         * function to update the price (input and span)
         */

        function csd_update_price() {
            var additionnal_rules = csd.pricing_rules;
            var basing_price = parseFloat(csd.product_price);
            var text_palette_tpl_price = csd.text_palette_tpl_pricing;
            var bg_palette_tpl_price = csd.bg_palette_tpl_pricing;
            var text_color_price = 0;
            var bg_color_price = 0;
            var additionnal_price_on_design = 0;
            var additionnal_price_per_additionnal_item = 0;

            if(typeof csd_pricing_rules !== "undefined" && csd_pricing_rules !='')
            {
                var csd_editor_pricing_rules = JSON.parse(csd_pricing_rules);
                var csd_price_key = Object.keys(csd_editor_pricing_rules);
            }
            
            if ($('#txt-color-selector').size() > 0 ) {
                var text_color = $('#txt-color-selector').val();
                if (typeof text_palette_tpl_price !== 'undefined') {
                    $.each(text_palette_tpl_price, function (index, text) {
                        if (text.hex === text_color || text.name === text_color) {
                            if ($.isNumeric(parseFloat(text.price))) {
                                text_color_price = parseFloat(text.price);
                            }
                        }
                    });
                }
            }

            var text_bg_color = $('.csd-color-view').css('backgroundColor');
            var bg_hex = rgb2hex(text_bg_color);
            if (bg_hex != false) {
                if (bg_palette_tpl_price !== 'undefined') {
                    $.each(bg_palette_tpl_price, function (index, bg) {
                        if (bg.hex === bg_hex || bg.name === bg_hex) {
                            if ($.isNumeric(parseFloat(bg.price))) {
                                bg_color_price = parseFloat(bg.price);
                            }
                        }
                    });
                }
            }

            if (typeof additionnal_rules !== 'undefined') {
                $.each(additionnal_rules, function (key, rule) {
                    var verify_result = csd_verify_condition(rule);
                    if (verify_result) {
                        if (verify_result.verify) {
                            additionnal_price_per_additionnal_item = additionnal_price_per_additionnal_item + parseFloat(verify_result.additionnal_price);
                            if ($.isNumeric(parseFloat(rule.a_price))) {
                                if (rule.scope === 'design') {
                                    additionnal_price_on_design = additionnal_price_on_design + parseFloat(rule.a_price);
                                }
                            }
                        }
                    }
                });
            }
            var parts_json = {};
            var product_id = $('#csd_add_to_cart').attr("data-id");
            var tthat = $('form.formbuilt');
            if (tthat.length !== 0) {
                var form_fields = tthat.serializeJSON();
                delete form_fields.id_ofb;
                parts_json["form_fields"] = form_fields;
            }
            parts_json["product_id"] = product_id;
            $.post(
                    csd_ajax_object.csd_ajax_url,
                    {
                        action: "csd_get_design_price",
                        serialized_parts: JSON.stringify(parts_json),

                    },
                    function (data) {
                        if (csd_editor.is_json(data))
                        {
                            var response = JSON.parse(data);
                            var price = 0;
                            price = response.prices;
                            var qty = parseInt($(".csd-qty").val());
                            if(typeof csd_editor_pricing_rules !== "undefined" && typeof csd_editor_pricing_rules[qty] !== "undefined" && csd_price_key.length > 0)
                            {
                                basing_price = csd_editor_pricing_rules[qty];
                                var total = parseFloat(basing_price) + (parseFloat(text_color_price) + parseFloat(bg_color_price) + +parseFloat(additionnal_price_per_additionnal_item) + parseFloat(additionnal_price_on_design) + price) * parseInt(qty);
                            }
                            else
                                var total = (parseFloat(basing_price) + parseFloat(text_color_price) + parseFloat(bg_color_price) + +parseFloat(additionnal_price_per_additionnal_item) + parseFloat(additionnal_price_on_design) + price) * parseInt(qty);
                            $(".total_order").html(accounting.formatMoney(total));
                        } else
                            $("#debug").html(data);
                    }

            );
        }

        function csd_verify_condition(rule) {

            if (typeof rule != 'undefined') {
                var comparaison_result = [];
                $.each(rule.rules, function (index, this_rule) {
                    if (typeof this_rule != 'undefined' && this_rule.param != 'undefined') {
                        switch (this_rule.param) {
                            case "txt_nb_chars":
                                csd_editor.canvas.forEachObject(function (obj) {
                                    if (obj.type == "i-text") {
                                        var formattedText = obj.text;
                                    }
                                    if (typeof formattedText != 'undefined') {
                                        var text_size = formattedText.length;
                                        comparaison_result.push(csd_operator(this_rule.operator, text_size, this_rule.value, rule.scope));
                                    }
                                });
                                break;
                            case "txt_nb_lines":
                                csd_editor.canvas.forEachObject(function (obj) {
                                    if (obj.type == "i-text") {
                                        var formattedText = obj.text;
                                    }
                                    if (typeof formattedText != 'undefined') {
                                        var text_nb_line = occurrences(formattedText, '\n');
                                        comparaison_result.push(csd_operator(this_rule.operator, text_nb_line, this_rule.value, rule.scope));
                                    }
                                });
                                break;
                            case "if_canvas_square_unit":
                                var canvas_w = parseInt($('#csd-canvas-width').val());
                                var canvas_h = parseInt($('#csd-canvas-height').val());
                                if (typeof canvas_w != 'undefined' && typeof canvas_h != 'undefined') {
                                    var canvas_square = canvas_h * canvas_w;
                                    comparaison_result.push(csd_operator(this_rule.operator, canvas_square, this_rule.value, rule.scope));
                                }
                                break;
                            case "if_selected_font":
                                //Retrieve the selected font.
                                csd_editor.canvas.forEachObject(function (obj) {
                                    if (obj.type == "i-text") {
                                        var font = obj.fontFamily;
                                    }
                                    rule.scope = 'design';
                                    if (typeof font != 'undefined') {
                                        comparaison_result.push(csd_operator(this_rule.operator, font, this_rule.value, rule.scope));
                                    }
                                });
                                break;
                            default:
                                comparaison_result = comparaison_result;
                                break;
                        }
                    }
                });
                var verify = 0;
                $.each(comparaison_result, function (key, value) {
                    if (value.result) {
                        verify++;
                    }
                });
                var additionnal_price_per_item = 0;
                var result = [];
                result['verify'] = false;

                if (verify === comparaison_result.length) {
                    result['verify'] = true;
                    $.each(comparaison_result, function (key, value) {
                        if (rule.scope === 'additional_item' || rule.scope === "item" && typeof (value.remaining_number) != 'undefined') {
                            additionnal_price_per_item = additionnal_price_per_item + (value.remaining_number * rule.a_price);
                        } else {
                            additionnal_price_per_item = additionnal_price_per_item + 0;
                        }

                    });
                    result['additionnal_price'] = additionnal_price_per_item;
                }
                return result;
            }
        }

        function csd_operator(operator, giving_value, rule_value, rule_scope) {
            var data = [];
            if (operator === ">") {
                data['result'] = giving_value > rule_value;
                if (giving_value > rule_value) {
                    if (rule_scope === "additional_item") {
                        data['remaining_number'] = giving_value - rule_value;
                    } else if (rule_scope === "item") {
                        data['remaining_number'] = giving_value;
                    }

                }
                return data;
            } else if (operator === ">=") {
                data['result'] = giving_value >= rule_value;
                if (giving_value >= rule_value) {
                    if (rule_scope === "additional_item") {
                        data['remaining_number'] = (giving_value - rule_value) + 1;
                    } else if (rule_scope === "item") {
                        data['remaining_number'] = giving_value;
                    }
                }
                return data;
            } else if (operator === "<" && rule_scope !== "additional_item") {
                data['result'] = giving_value < rule_value;
                if (giving_value < rule_value) {
                    if (rule_scope === "item") {
                        data['remaining_number'] = giving_value;
                    }
                }
                return data;
            } else if (operator === "<=" && rule_scope !== "additional_item") {
                data['result'] = giving_value <= rule_value;
                if (giving_value < rule_value) {
                    if (rule_scope === "item") {
                        data['remaining_number'] = giving_value;
                    }
                }
                return data;
            } else if (operator === "==" && rule_scope !== "additional_item") {
                data['result'] = giving_value == rule_value;
                if ($.isNumeric(parseInt(giving_value)) && $.isNumeric(parseInt(rule_value)) && giving_value == rule_value) {
                    if (rule_scope === "item") {
                        data['remaining_number'] = giving_value;
                    }
                }
                return data;
            } else {
                data['result'] = giving_value != rule_value;
                if ($.isNumeric(parseInt(giving_value)) && $.isNumeric(parseInt(rule_value)) && giving_value !== rule_value) {
                    if (rule_scope === "item") {
                        data['remaining_number'] = giving_value;
                    }

                }
                return data;
            }
        }

        $("form.formbuilt input").change(function () {
            csd_update_price();
        });

        $("form.formbuilt textarea").change(function () {
            csd_update_price();
        });

        $("form.formbuilt select").change(function () {
            csd_update_price();
        });

        $(document).on('click', '.csd-qty-container input', function (e) {
            e.preventDefault();
            // Get values
            var $qty = $(".csd-qty");
            var currentVal = parseFloat($qty.val());
            var max = parseFloat($qty.attr('max'));
            var min = parseFloat($qty.attr('min'));
            var step = $qty.attr('step');
            // Format values
            if (!currentVal || currentVal === '' || currentVal === 'NaN')
                currentVal = 0;
            if (max === '' || max === 'NaN')
                max = '';
            if (min === '' || min === 'NaN')
                min = 0;
            if (step === 'any' || step === '' || step === undefined || parseFloat(step) === 'NaN')
                step = 1;
            // Change the value
            if ($(this).is('.csd-qty-plus')) {

                if (max && (max === currentVal || currentVal > max)) {
                    $qty.val(max);
                } else {
                    $qty.val(currentVal + parseFloat(step));
                }

            } else if ($(this).is('.csd-qty-minus')) {

                if (min && (min === currentVal || currentVal < min)) {
                    $qty.val(min);
                } else if (currentVal > 0) {
                    $qty.val(currentVal - parseFloat(step));
                }

            }
            $("#summary-qty-values").html($qty.val());
            csd_update_price();
        });

        $(".csd-qty").bind("keyup mouseup change click", function () {
            if ($(this).data("previousValue") != $(this).val())
            {
                var max = parseFloat($(this).attr('max'));
                var min = parseFloat($(this).attr('min'));
                var currentVal = parseFloat($(this).val());
                if (max === '' || max === 'NaN')
                    max = '';
                if (min === '' || min === 'NaN')
                    min = 0;

                if (max && (max === currentVal || currentVal > max)) {
                    $(this).val(max);
                }
                if (min && (min === currentVal || currentVal < min)) {
                    $(this).val(min);
                }
                $("#summary-qty-values").text($(this).val());
                csd_update_price();
                // $(this).data("previousValue", $(this).val());
            }
        });


        //add design to cart
        $("#csd_add_to_cart").on("click", function () {
            $.blockUI({message: csd.translated_strings.loading_msg});
            var quantity = $(".csd-qty").val();
            var product_id = $(this).attr("data-id");
            var canvas_w = parseInt($('#csd-canvas-width').val());
            var canvas_h = parseInt($('#csd-canvas-height').val());

            //Make sure the last modification is handled
            csd_editor.save_canvas();

            loop_through_parts(csd_editor.output_loop_delay,
                    generate_final_canvas_part,
                    function () {
                        var frm_data = new FormData();
                        if (typeof tpl === 'undefined') {
                            var tpl = "";
                        }
                        frm_data.append("tpl", tpl);
                        frm_data.append('product_id', product_id);
                        frm_data.append("quantity", quantity);
                        frm_data.append('canvas_w', canvas_w);
                        frm_data.append('canvas_h', canvas_h);
                        frm_data.append('action', "csd_add_custom_design_to_cart");
                        frm_data.append("final_canvas_parts", csd_editor.final_canvas_parts);
                        frm_data = convert_final_canvas_parts_to_blob(frm_data);
                        var tthat = $('form.formbuilt');
                        if (tthat.length !== 0) {
                            tthat.validationEngine('validate', {showArrow: false});
                            var form_fields = tthat.serializeJSON();
                            delete form_fields.id_ofb;
                            var jsonform_fields = JSON.stringify(form_fields);
                            frm_data.append("form_fields", jsonform_fields);
                        }

                        if (typeof window.csd_key !== 'undefined') {
                            frm_data.append("csd_key", window.csd_key);
                        }

                        $.ajax({
                            type: 'POST',
                            url: csd_ajax_object.csd_ajax_url,
                            data: frm_data,
                            processData: false,
                            contentType: false
                        }).done(function (data) {
                            if (csd_editor.is_json(data))
                            {
                                var response = JSON.parse(data);
                                if (csd.redirect_after == 1 && response.success)
                                {
                                    $(location).attr('href', response.url);
                                } else
                                {
                                    $("#debug-cart").html(response.message);
                                    $.ajax($fragment_refresh);
                                    $.unblockUI();
                                }
                            } else
                            {
                                $("#debug-cart").html(data);
                                set_clipping_area();
                                $.ajax($fragment_refresh);
                                $.unblockUI();
                            }

                        });
                    });
        });

        if (typeof wc_cart_fragments_params != 'undefined')
        {
            var $fragment_refresh = {
                url: wc_cart_fragments_params.wc_ajax_url.toString().replace('%%endpoint%%', 'get_refreshed_fragments'),
                type: 'POST',
                success: function (data) {
                    if (data && data.fragments) {
                        $.each(data.fragments, function (key, value) {
                            $(key).replaceWith(value);
                        });

                        $(document.body).trigger('wc_fragments_refreshed');
                    }
                }
            };
        }


        //Preview
        $(document).on("touchstart click", "#preview-btn", function (e)
        {
            e.preventDefault();
            $("#csd-modal .omodal-body").html("");
            //Make sure the last modification is handled
            csd_editor.save_canvas();
            // if (wpd.clip_include_in_output == "no")
            //     wpd_editor.canvas.clipTo = null;
            loop_through_parts(csd.output_loop_delay,
                    generate_canvas_part,
                    function () {
                        $("#csd-parts-bar > li").first().click();
                        $('#csd-modal').omodal("show");
                        set_clipping_area();
                        $.unblockUI();
                    }
            );
        });

    });

    return csd_editor;
}(jQuery, csd_EDITOR));