<?php

class CSD_Updater {

    protected $version_url = 'https://static.designersuiteforwp.com/csd-updater.xml';
    public $title = 'Custom Signs Designer for WooCommerce for WooCommerce';
    protected $auto_updater = false;
    protected $upgrade_manager = false;
    protected $iframe = false;

    public function init() {
        add_filter('upgrader_pre_download', array($this, 'upgradeFilter'), 10, 4);
        add_action('upgrader_process_complete', array($this, 'removeTemporaryDir'));
    }

    /**
     * Setter for manager updater.
     *
     * @param CSD_Updating_Manager $updater
     */
    public function setUpdateManager(CSD_Updating_Manager $updater) {
        $this->auto_updater = $updater;
    }

    /**
     * Getter for manager updater.
     *
     * @return CSD_Updating_Manager
     */
    public function updateManager() {
        return $this->auto_updater;
    }

    /**
     * Get url for version validation
     * @return string
     */
    public function versionUrl() {
        return $this->version_url;
    }

    /**
     * Downloads new VC from Envato marketplace and unzips into temporary directory.
     *
     * @param $reply
     * @param $package
     * @param $updater
     * @return mixed|string|WP_Error
     */
    public function upgradeFilter($reply, $package, $updater) {
        global $wp_filesystem;
        if ( (isset($updater->skin->plugin) && $updater->skin->plugin === CSD_MAIN_FILE) ||
                (isset($updater->skin->plugin_info) && htmlspecialchars_decode($updater->skin->plugin_info['Name']) === $this->title)
        ) {
            $updater->strings['download_from_servers'] = __('Downloading package from ORION servers...', 'csd');
            $updater->skin->feedback('download_from_servers');
            $package_filename = 'lettering-signs-designer.zip';
            $res = $updater->fs_connect(array(WP_CONTENT_DIR));
            if (!$res) {
                return new WP_Error('no_credentials', __("Error! Can't connect to filesystem", 'csd'));
            }
            $options = get_option('csd-license-options');
            if (isset($options['license-key']) && $options['license-key'] !== "") {
                $license_key = $options['license-key'];
            } else {
                return new WP_Error('no_credentials', __('To receive automatic updates, license activation is required. Please visit <a href="' . admin_url('admin.php?page=csd-manage-settings') . '' . '" target="_blank">Settings</a> to activate your Custom Signs Designer for WooCommerce.', 'csd'));
            }

            $args = array('timeout' => 600);
            $site_url = get_site_url();
            $plugin_name = CSD_PLUGIN_NAME;
            $url = "https://designersuiteforwp.com/service/olicenses/v1/checking/?license-key=" . urlencode($license_key) . "&siteurl=" . urlencode($site_url) . "&name=" . urlencode($plugin_name);
            $response = wp_remote_get($url, $args);
            if (!is_wp_error($response) ) {
                if (isset($response["body"]) && intval($response["body"]) == 200) {
                    $json = wp_remote_get($this->downloadUrl($this->title), $args);
                    if (is_wp_error($json)) {
                        return $json->get_error_message();
                    }
                    if (isset($json["body"])) {
                        $answer = $json["body"];
                    }
                    
                    $result = array();
                    
                    if (is_array(json_decode($answer, true))) {
                        $result = json_decode($answer, true);
                    } else {
                        return new WP_Error('no_file', __('Error! No file found. Please contact the plugin owners.', 'csd'));;
                    }
                    
                    if (!isset($result['download_url'])) {
                        return new WP_Error('no_file', __('Error! No file found. Please contact the plugin owners.', 'csd'));
                    }
                    
                    $download_file = download_url($result['download_url']);
                    if (is_wp_error($download_file)) {
                        return $download_file;
                    }
                    $uploads_dir_obj = wp_upload_dir();
                    $upgrade_folder = $uploads_dir_obj["basedir"] . '/csd_package';
                    if (!is_dir($upgrade_folder)) {
                        mkdir($upgrade_folder);
                    }
                    //We rename the tmp file to a zip file
                    $new_zipname = str_replace(".tmp", ".zip", $download_file);
                    rename($download_file, $new_zipname);
                    //The upgrade is in the unique directory inside the upgrade folder
                    $new_version = "$upgrade_folder/$package_filename";
                    $result = copy($new_zipname, $new_version);
                    if ($result && is_file($new_version)) {
                        return $new_version;
                    }
                    return new WP_Error('no_credentials', __('Error on unzipping package', 'csd'));
                } else {
                    return new WP_Error('network_error', __('Wrong license key provided.', 'csd'));
                }
            } else {
                return $response->get_error_message();
            }
        }
        return $reply;
    }

    public function removeTemporaryDir() {
        global $wp_filesystem;
        if (is_dir($wp_filesystem->wp_content_dir() . 'uploads/csd_package')) {
            $wp_filesystem->delete($wp_filesystem->wp_content_dir() . 'uploads/csd_package', true);
        }
    }

    protected function downloadUrl($name) {
        //return "https://designersuiteforwp.com/service/oupdater/v1/update/?name=". rawurlencode($name);
        return "https://demos.designersuiteforwp.com/lettering-signs-designer/service/oupdater/v1/update/?name=". rawurlencode($name);
    }

}
