<?php
/**
 * The public-facing functionality of the plugin.
 *
 * @link       orionorigin@orionorigin.com
 * @since      1.0.0
 *
 * @package    Csd
 * @subpackage Csd/public
 */

/**
 * The public-facing functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the public-facing stylesheet and JavaScript.
 *
 * @package    Csd
 * @subpackage Csd/public
 * @author     orionorigin <orionorigin@orionorigin.com>
 */
class Csd_Public {

    /**
     * The ID of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string    $plugin_name    The ID of this plugin.
     */
    private $plugin_name;

    /**
     * The version of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string    $version    The current version of this plugin.
     */
    private $version;

    /**
     * Initialize the class and set its properties.
     *
     * @since    1.0.0
     * @param      string $plugin_name       The name of the plugin.
     * @param      string $version    The version of this plugin.
     */
    public function __construct($plugin_name, $version) {

        $this->plugin_name = $plugin_name;
        $this->version = $version;
    }

    /**
     * Register the stylesheets for the public-facing side of the site.
     *
     * @since    1.0.0
     */
    public function enqueue_styles() {

        /**
         * This function is provided for demonstration purposes only.
         *
         * An instance of this class should be passed to the run() function
         * defined in Pdb_Loader as all of the hooks are defined
         * in that particular class.
         *
         * The Pdb_Loader will then create the relationship
         * between the defined hooks and the functions defined in this
         * class.
         */
        wp_enqueue_style('csd-public-css', CSD_URL . 'public/css/csd-public.css', array(), $this->version, 'all');

        wp_enqueue_style('csd-bs-modal-public-css', CSD_URL . 'public/js/modal/modal.min.css', array(), $this->version, 'all');

    }

    /**
     * Register the JavaScript for the public-facing side of the site.
     *
     * @since    1.0.0
     */
    public function enqueue_scripts() {
        
        /**
         * This function is provided for demonstration purposes only.
         *
         * An instance of this class should be passed to the run() function
         * defined in Pdb_Loader as all of the hooks are defined
         * in that particular class.
         *
         * The Pdb_Loader will then create the relationship
         * between the defined hooks and the functions defined in this
         * class.
         */   
        wp_enqueue_script('jquery');   
        
        wp_enqueue_script($this->plugin_name, CSD_URL . 'public/js/csd-public.js', array('jquery'), $this->version, false);
        
        wp_dequeue_script('bs-modal');
        
        wp_enqueue_script('bs-modal', CSD_URL . 'public/js/modal/csd-modal.min.js', array('jquery'), $this->version, false);
        


        wp_enqueue_script('csd-jquery-mCustomScrollbar-js', CSD_URL . 'includes/skins/default/assets/js/jquery.mCustomScrollbar.min.js', array(), CSD_VERSION, false);




        wp_localize_script($this->plugin_name, 'csd_ajax_object', array('csd_ajax_url' => admin_url('admin-ajax.php')));

    }

    /**
     * Register the plugin shortcodes
     */
    public function csd_register_shortcodes() {
        add_shortcode('csd-editor', array($this, 'csd_get_editor_shortcode_handler'));
        add_shortcode('csd_editor', array($this, 'csd_get_editor_shortcode_handler'));
        add_shortcode('csd_builder', array($this, 'csd_get_editor_shortcode_handler'));
        add_shortcode('csd', array($this, 'csd_get_editor_shortcode_handler'));
    }

    /**
     * Callback for the shortcode csd-editor
     */
    public function csd_get_editor_shortcode_handler($atts) {
        global $wp_query;
        $product_id = get_query_var("product_id", false);

        //Maybe the product ID is included in the shortcode
        if (isset($atts["product"])) {
            $product = $atts["product"];
        }
        if (!$product_id) {
            $product_id = $product;
        }

        if (!$product_id) {
            return esc_html__("You're trying to access the customization page whitout a product to customize. This page should only be accessed using one of the customization buttons.", 'csd');
        }

        $editor_obj = new CSD_Editor($product_id);
        $raw_output = $editor_obj->get_editor();
        $output = apply_filters("csd_output_editor", $raw_output, $product_id);
        return $output;
    }

    /**
     * Function to set variables filters.
     * @global type $csd_settings
     */
    public function csd_set_variable_action_filters() {
        global $csd_settings;
        $options = $csd_settings['csd-general-options'];
        $woo_version = $this->csd_get_woo_version_number();
        if ('name' === $options['csd-parts-position-cart']) {
            if ($woo_version < 2.1) {
//Old WC versions
                add_filter('woocommerce_in_cart_product_title', array($this, 'csd_get_data'), 10, 3);
            } else {
//New WC versions
                add_filter('woocommerce_cart_item_name', array($this, 'csd_get_data'), 10, 3);
            }
        } else {
            if ($woo_version < 2.1) {
//Old WC versions
                add_filter('woocommerce_in_cart_product_thumbnail', array($this, 'csd_get_data'), 10, 3);
            } else {
//New WC versions
                add_filter('woocommerce_cart_item_thumbnail', array($this, 'csd_get_data'), 10, 3);
            }
        }
        $append_content_filter = $options['csd-content-filter'];

        if ('0' !== $append_content_filter && !is_admin()) {

            add_filter('the_content', array($this, 'csd_filter_content'), 99);
        }
    }

    /**
     * Display the customizer in the proper page
     */
    public function csd_filter_content($content) {
        global $csd_settings;
        global $wp_query;
        $options = $csd_settings['csd-general-options'];
        $csd_page_id = $options['csd_page_id'];
        if (function_exists('icl_object_id')) {
            $csd_page_id = icl_object_id($csd_page_id, 'page', false, ICL_LANGUAGE_CODE);
        }
        $current_page_id = get_the_ID();
        if (intval($csd_page_id) === intval($current_page_id)) {
            $item_id = $wp_query->query_vars['product_id'];
            $editor_obj = new CSD_Editor($item_id);
            $raw_output = $editor_obj->get_editor();
            $output = apply_filters("csd_output_editor", $raw_output, $item_id);
            $content .= $output;
        }

        return $content;
    }

    /**
     * Retrieve csd data in cart
     *
     * @param type $thumbnail_code Thumbnail code.
     * @param type $values Values to be added in the cart.
     * @param type $cart_item_key Cart item key.
     * @return string
     */
    public function csd_get_cart_data($thumbnail_code, $values, $cart_item_key) {
        $variation_id = $values['product_id'];
        if (isset($values['csd_generated_data']) && isset($values['csd_generated_data']['output'])) {
            $thumbnail_code .= '<br>';
            $image = $values['csd_generated_data']['output']['files'];
            $thumbnail_code .= '<img src="' . $image . '">';
        }
        return $thumbnail_code;
    }

    /**
     * Add csd query vars
     *
     * @param type $a_vars Query variables.
     * @return string
     */
    public function csd_add_query_vars($a_vars) {
        $a_vars[] = 'product_id';
        $a_vars[] = 'tpl';
        $a_vars[] = 'edit';
        $a_vars[] = 'design_index';
        $a_vars[] = 'oid';
        return $a_vars;
    }

    /**
     * Add csd query vars
     *
     * @global type $csd_settings
     * @global type $wp_rewrite
     * @param type $param Params.
     */
    public function csd_add_rewrite_rules($param) {
        global $csd_settings;
        global $wp_rewrite;
        $options = $csd_settings['csd-general-options'];
        $csd_page_id = $options['csd_page_id'];
        if (function_exists('icl_object_id')) {
            $csd_page_id = icl_object_id($csd_page_id, 'page', false, ICL_LANGUAGE_CODE);
        }
        $csd_page = get_post($csd_page_id);
        if (is_object($csd_page)) {
//$slug = $csd_page->post_name;
            $raw_slug = get_permalink($csd_page->ID);
            $home_url = home_url('/');
            $slug = str_replace($home_url, '', $raw_slug);
//If the slug does not have the trailing slash, we get 404 (ex postname = /%postname%)
            $sep = '';
            if (substr($slug, -1) !== '/') {
                $sep = '/';
            }
//            var_dump(substr($slug, -1));
            add_rewrite_rule(
// The regex to match the incoming URL
                    $slug . $sep . 'design/([^/]+)/?$',
                    // The resulting internal URL: `index.php` because we still use WordPress
// `pagename` because we use this WordPress page
// `designer_slug` because we assign the first captured regex part to this variable
                    'index.php?pagename=' . $slug . '&product_id=$matches[1]',
                    // This is a rather specific URL, so we add it to the top of the list
// Otherwise, the "catch-all" rules at the bottom (for pages and attachments) will "win"
                    'top'
            );
            add_rewrite_rule(
// The regex to match the incoming URL
                    $slug . $sep . 'design/([^/]+)/([^/]+)/?$',
                    // The resulting internal URL: `index.php` because we still use WordPress
// `pagename` because we use this WordPress page
// `designer_slug` because we assign the first captured regex part to this variable
                    'index.php?pagename=' . $slug . '&product_id=$matches[1]&tpl=$matches[2]',
                    // This is a rather specific URL, so we add it to the top of the list
// Otherwise, the "catch-all" rules at the bottom (for pages and attachments) will "win"
                    'top'
            );
            add_rewrite_rule(
// The regex to match the incoming URL
                    $slug . $sep . 'edit/([^/]+)/([^/]+)/?$',
                    // The resulting internal URL: `index.php` because we still use WordPress
// `pagename` because we use this WordPress page
// `designer_slug` because we assign the first captured regex part to this variable
                    'index.php?pagename=' . $slug . '&product_id=$matches[1]&edit=$matches[2]',
                    // This is a rather specific URL, so we add it to the top of the list
// Otherwise, the "catch-all" rules at the bottom (for pages and attachments) will "win"
                    'top'
            );
            add_rewrite_rule(
// The regex to match the incoming URL
                    $slug . $sep . 'ordered-design/([^/]+)/([^/]+)/?$',
                    // The resulting internal URL: `index.php` because we still use WordPress
// `pagename` because we use this WordPress page
// `designer_slug` because we assign the first captured regex part to this variable
                    'index.php?pagename=' . $slug . '&product_id=$matches[1]&oid=$matches[2]',
                    // This is a rather specific URL, so we add it to the top of the list
// Otherwise, the "catch-all" rules at the bottom (for pages and attachments) will "win"
                    'top'
            );

            add_rewrite_rule(
// The regex to match the incoming URL
                    $slug . $sep . 'saved-design/([^/]+)/([^/]+)/?$',
                    // The resulting internal URL: `index.php` because we still use WordPress
// `pagename` because we use this WordPress page
// `designer_slug` because we assign the first captured regex part to this variable
                    'index.php?pagename=' . $slug . '&product_id=$matches[1]&design_index=$matches[2]',
                    // This is a rather specific URL, so we add it to the top of the list
// Otherwise, the "catch-all" rules at the bottom (for pages and attachments) will "win"
                    'top'
            );

            $wp_rewrite->flush_rules(false);
        }
    }

    /**
     * Get design url
     *
     * @param type $post_id Product id.
     * @param type $product Product object.
     */
    public function get_design_from_blank_urls($post_id, $product) {
        $design_from_blank_urls = array();
        if ('variable' === $product->get_type()) {
            $variations = $product->get_available_variations();
            foreach ($variations as $variation) {
                $variation_id = $variation['variation_id'];
                $csd_product = new CSD_Product($variation_id);
                $design_from_blank_urls[$variation_id] = $csd_product->csd_get_design_url();
            }
        } else {
            $csd_product = new CSD_Product($post_id);
            $design_from_blank_urls[$post_id] = $csd_product->csd_get_design_url();
        }
        ?>
        <script>
            var design_from_blank_urls =<?php echo wp_json_encode($design_from_blank_urls); ?>;
        </script>
        <?php
    }

    /**
     * Get custom button on product page
     */
    public function csd_get_customize_btn() {
        $product_id = get_the_ID();
        $csd_product = new CSD_Product($product_id);
        $allowed_tags = csd_allowed_tags();
        $raw_output = $csd_product->csd_get_buttons();
        echo wp_kses($raw_output, $allowed_tags);
    }

    /**
     * Get custom button for each element of shop page
     *
     * @global type $csd_settings Array of settings.
     * @param type $html Html output.
     * @param type $product Product object.
     * @return type
     */
    public function csd_get_customize_btn_loop($html, $product) {
        global $csd_settings;
        $general_options = $csd_settings['csd-general-options'];
        $hide_buttons_shop_page = get_proper_value($general_options, 'csd-hide-btn-shop-pages', 0);
        $hide_cart_button_shop_page = get_proper_value($general_options, 'csd-hide-cart-shop-pages', 0);
        $product_id = $product->get_id();
        $csd_product = new CSD_Product($product_id);
        $product_class = get_class($product);
        if ('WC_Product_Simple' === $product_class && $csd_product->is_customizable()) {
            if (!$hide_buttons_shop_page) {
                $html .= $csd_product->csd_get_buttons();
            }

            if ($hide_cart_button_shop_page) {
                $html .= '<script>
                    jQuery(".add_to_cart_button[data-product_id=' . $product_id . ']").hide();
                </script>';
            } else {
                
            }
        }
        return $html;
    }

    /**
     * Add classes to body
     */
    public function csd_add_class_to_body($classes) {
        global $wp_query;
        if (isset($wp_query->query_vars['product_id'])) {
            array_push($classes, 'csd-customization-page csd-product-' . $wp_query->query_vars['product_id']);
        }
        return $classes;
    }

    /**
     * Add class to item
     */
    public function csd_get_item_class($classes, $class, $post_id) {
        global $csd_settings;
        $general_options = $csd_settings['csd-general-options'];
        $hide_cart_button = get_proper_value($general_options, 'csd-hide-cart-button', true);
        $hide_cart_button_shop_page = get_proper_value($general_options, 'csd-hide-cart-shop-pages', 0);

        if (in_array('product', $classes, true)) {
            $csd_product = new CSD_Product($post_id);
            if ($csd_product->is_customizable()) {
                array_push($classes, 'csd-is-customizable');
            }

            if ($hide_cart_button) {
                array_push($classes, 'csd-hide-cart-button');
            }

            if ($hide_cart_button_shop_page) {
                array_push($classes, 'csd-hide-cart-button-on-shop');
            }
        }
        return $classes;
    }

    private function get_output_zip_folder_name($prod_id) {
        global $csd_settings;

        $csd_product = new CSD_Product($prod_id);
        $product_metas = $csd_product->settings;
        $product_output_settings = get_proper_value($product_metas, 'output-settings');
        $zip_name = get_proper_value($product_output_settings, 'zip-folder-name');
        if (empty($zip_name)) {
            $zip_name = uniqid('csd_');
        }

        return $zip_name . '.zip';
    }

    /*
     * Ajax callback that controls if design is already added or not
     */

    public function add_custom_design_to_cart_ajax() {
        global $woocommerce;
        $message = '';
        if (csd_woocommerce_version_check()) {
            $cart_url = wc_get_cart_url();
        } else {
            $cart_url = $woocommerce->cart->get_cart_url();
        }

        if (isset($_POST['final_canvas_parts']) && isset($_POST['tpl']) && isset($_POST['product_id']) && isset($_POST['quantity'])) {

            $tpl = $_POST['tpl'];
            $quantity = $_POST['quantity'];
            $cart_item_key = false;
            $tmp_dir = uniqid();
            $upload_dir = wp_upload_dir();

            $generation_path = $upload_dir['basedir'] . "/CSD/$tmp_dir";

            $generation_url = $upload_dir['baseurl'] . "/CSD/$tmp_dir";

            $product_id = $_POST['product_id'];
            $config = csd_get_product_config($product_id);
            $product_metas = $config->settings;
            if (class_exists('Ofb')) {
                if (isset($product_metas['form-builder'])) {
                    if ($product_metas['form-builder'] != '') {
                        $form_fields = stripslashes_deep(filter_input(INPUT_POST, 'form_fields'));
                        if (isset($form_fields) && !empty($form_fields)) {
                            if (isset($form_fields)) {
                                $form_fields_decode = json_decode($form_fields);
                                foreach ($form_fields_decode as $key => $value) {
                                    $data[$key] = $value;
                                }
                            }
                        }
                        $total_price_form = get_form_data($product_metas['form-builder'], $data);
                    }
                }
            }

            $zip_name = $this->get_output_zip_folder_name($product_id);

            $final_data = $_POST['final_canvas_parts'];

            $result = $this->export_data_to_files($generation_path, $final_data, $product_id, $zip_name);
           
            $final_data["output"]=array(
                "files" => $result,
                "working_dir" => $tmp_dir,
                "tpl"   => $tpl
            );

            $final_data["quantity"] = $_POST['quantity'];
            $final_data["product_id"] = $product_id;
            $final_data['canvas_h'] = $_POST['canvas_h'];
            $final_data['canvas_w'] = $_POST['canvas_w'];
           
            if (class_exists('Ofb')) {

                if (isset($product_metas['form-builder'])) {

                    if ($product_metas['form-builder'] != '') {
                        if (isset($data) && !empty($data)) {
                            $final_data['output']['form_fields'] = $data;
                        }

                        $final_data['output']['total_price_form'] = $total_price_form;
                    }
                }
            }
            $newly_added_cart_item_key = true;
            if ($cart_item_key) {
                $woocommerce->cart->cart_contents[$cart_item_key]['csd_generated_data'] = $final_data;
                $woocommerce->cart->calculate_totals();
                $message = "<div class='csd_notification success f-right'>" . esc_html__('Item successfully updated.', 'csd') . " <a href='$cart_url'>" . esc_html__('View Cart', 'csd') . '</a></div>';
            } else {
                $newly_added_cart_item_key = $this->add_design_to_cart($final_data);
                if ($newly_added_cart_item_key) {
                    do_action('csd_add_custom_data_to_cart', $newly_added_cart_item_key);
                    $message = "<div class='csd_notification success f-right'>" . esc_html__('Product successfully added to basket.', 'csd') . " <a href='$cart_url'>View Cart</a></div>";
                } else {
                    $message = "<div class='csd_notification failure f-right'>" . esc_html__('A problem occured while adding the product to the cart. Please try again.', 'csd') . '</div>';
                }
            }
            echo wp_json_encode(
                    array(
                        'success' => $newly_added_cart_item_key,
                        'message' => $message,
                        'url' => $cart_url,
                        'd' => $final_data
//                        'form_fields' => $data,
                    )
            );
        }
        die();
    }

    public function generate_downloadable_file() {
        $final_canvas_parts = $_POST['final_canvas_parts'];
        $tmp_dir = uniqid();
        $upload_dir = wp_upload_dir();
        $generation_path = $upload_dir['basedir'] . "/CSD/$tmp_dir";
        $generation_url = $upload_dir['baseurl'] . "/CSD/$tmp_dir";
        $product_id = $_POST['product_id'];
        $config = csd_get_product_config($product_id);
        $product_metas = $config->settings;
        $output_settings = get_proper_value($product_metas, 'output-settings', array());
        if (wp_mkdir_p($generation_path)) {

            $zip_name = $this->get_output_zip_folder_name($product_id);

            $canvas_w = $_POST['canvas_w'];
            $canvas_h = $_POST['canvas_h'];
            $multiplier = $_POST['multiplier'];

            // $result = $this->createFunctionWithImagik($final_canvas_parts, $canvas_w, $canvas_h, $generation_path, $multiplier, $product_id, $zip_name);

            $result = $this->export_data_to_files($generation_path, $final_canvas_parts, $product_id, $zip_name, true);

            if (!empty($result) && is_array($result)) {
                $output_msg = '';
                if ($output_settings['zip-output'] == 'yes') {
                    $output_msg = '<div>' . __('The generation has been successfully completed. Please click ', 'wpd') . "<a href='$generation_url/" . $zip_name . "' download='" . $zip_name . "'>" . __('here', 'wpd') . '</a> ' . __('to download your design', 'wpd') . '.</div>';
                } else {
                    foreach ($result as $part_key => $part_file_arr) {
                        $part_file = apply_filters('csd_downloadable_file_url', $part_file_arr['file'], $part_file_arr);
                        $output_msg .= '<div>' . ucfirst($part_key) . __(': please click ', 'wpd') . "<a href='$generation_url/$part_key/$part_file' download='$part_file'>" . __('here', 'csd') . '</a> ' . __('to download', 'csd') . '.</div>';
                    }
                }
                echo json_encode(
                        array(
                            'success' => 1,
                            'message' => "<div class='csd-success'>" . $output_msg . '</div>',
                        )
                );
            } else {
                echo json_encode(
                        array(
                            'success' => 0,
                            'message' => "<div class='csd-failure'>" . __('An error occured in the generation process. Please try again later.', 'csd') . '</div>',
                        )
                );
            }
        } else {
            echo json_encode(
                    array(
                        'success' => 0,
                        'message' => "<div class='csd-failure'>" . __("Can't create a generation directory...", 'csd') . '</div>',
                    )
            );
        }
        do_action('after_generated_file');
        die();
    }

//actual function to add the design to cart
    private function add_design_to_cart($final_data) {
        session_start();
        global $woocommerce;
        $variation_id = intval($_POST["product_id"]);
        $quantity = intval($final_data['quantity']);
        $product = wc_get_product($variation_id);
        $variation = array();
        if (isset($_SESSION['csd_key'])) {
            $variations = get_transient($_SESSION['csd_key']);
        }
        if ('simple' === $product->get_type()) {
            $product_id = $variation_id;
            $newly_added_cart_item_key = $woocommerce->cart->add_to_cart($product_id, $quantity, '', '', array('csd_generated_data' => $final_data));

        } else {
            if (isset($_SESSION["csd-save-user-choice-".$variation_id])) {
                $variation = $_SESSION["csd-save-user-choice-".$variation_id];
            }
            else $variation = $product->get_variation_attributes();
            
            $product_id = $product->get_parent_id();
            $newly_added_cart_item_key = $woocommerce->cart->add_to_cart($product_id, $quantity, $variation_id, $variation, array('csd_generated_data' => $final_data));
        }
        if (method_exists($woocommerce->cart, 'maybe_set_cart_cookies')) {
            $woocommerce->cart->maybe_set_cart_cookies();
        }
        session_write_close();
        return $newly_added_cart_item_key;
    }
    

    /*
     * csd_handle_picture_upload
     *
     */

    function csd_handle_picture_upload() {
        $nonce = $_POST['nonce'];
        if (!check_ajax_referer('csd-picture-upload-nonce', 'nonce', false)) {
            $busted = __('Cheating huh?', 'csd');
            die($busted);
        }

        $upload_dir = wp_upload_dir();
        $generation_path = $upload_dir['basedir'] . '/CSD';
        if (!is_dir($generation_path)) {
            wp_mkdir_p($generation_path);
        }
        $generation_url = $upload_dir['baseurl'] . '/CSD';
        $file_name = uniqid();
        $options = get_option('csd-upload-options');
        $valid_formats = $options['csd-upl-extensions'];
        if (!$valid_formats)
            $valid_formats = array("jpg", "jpeg", "png", "gif", "bmp", "svg", "psd", "eps", "pdf", "webp"); //wpc-upl-extensions

        $name = $_FILES['userfile']['name'];
        $size = $_FILES['userfile']['size'];

        if (isset($_POST) and $_SERVER['REQUEST_METHOD'] == 'POST') {
            if (strlen($name)) {
                $success = 0;
                $message = '';
                $img_url = '';
                $img_id = uniqid();
                $path_parts = pathinfo($name);
                $ext = $path_parts['extension'];
                $ext = strtolower($ext);
                if (in_array($ext, $valid_formats)) {
                    $tmp = $_FILES['userfile']['tmp_name'];
                    if (move_uploaded_file($tmp, $generation_path . '/' . $file_name . ".$ext")) {
                        $min_width = $options['csd-min-upload-width'];
                        $min_height = $options['csd-min-upload-height'];
                        $custom_attributes = $name;

                        // $custom_attributes = csd_build_attributes_from_array($custom_attributes);
                        $valid_formats_for_thumb = array('psd', 'eps', 'pdf');
                        if (in_array($ext, $valid_formats_for_thumb)) {
                            $thumb_generation_success = csd_generate_adobe_thumb($generation_path, $file_name . ".$ext", $file_name . '.png');
                            // If the thumb generation is a success, we force the extension to be png so the rest of the code can use it
                            if ($thumb_generation_success) {
                                $ext = 'png';
                            }
                        }
                        if ($min_width > 0 || $min_height > 0) {
                            list($width, $height, $type, $attr) = getimagesize($generation_path . '/' . $file_name . ".$ext");
                            if (( $min_width > $width || $min_height > $height ) && $ext != 'svg') {
                                $success = 0;
                                $message = sprintf(__('Uploaded file dimensions: %1$spx x %2$spx, minimum required ', 'csd'), $width, $height);
                                if ($min_width > 0 && $min_height > 0) {
                                    $message .= __('dimensions:', 'csd') . " $min_height" . 'px' . " x $min_height" . 'px';
                                } elseif ($min_width > 0) {
                                    $message .= "width: $min_width" . 'px';
                                } elseif ($min_height > 0) {
                                    $message .= "height: $min_height" . 'px';
                                }
                            } else {
                                $success = 1;
                                $message = "<span class='clipart-img'><img id='$img_id' src='$generation_url/$file_name.$ext'  " . implode(' ', $custom_attributes) . '></span>';
                                $img_url = "$generation_url/$file_name.$ext";
                            }
                        } else {
                            $success = 1;
                            $img_url = apply_filters('wpd_image_uploaded', "$generation_url/$file_name.$ext", "$generation_path/$file_name.$ext");
                            $message = "<span class='clipart-img'><img id='$img_id' src='$img_url'  " . implode(' ', $custom_attributes) . '></span>';
                        }
                        if ($success == 1) {
                            $custom_signs_designer_upload_300_dpi = apply_filters("woocommerce_product_designer_upload_300_dpi", $generation_path . "/" . $file_name . ".$ext");
                            if (0 === $custom_signs_designer_upload_300_dpi) {
                                $success = 0;
                                $message = __('The resolution of your image does not reach 300dpi', 'wpd');
                            } elseif (1 === $custom_signs_designer_upload_300_dpi) {
                                $success = 1;
                                $message = "<span class='clipart-img'><img id='$img_id' src='$generation_url/$file_name.$ext'  " . implode(' ', $custom_attributes) . "></span>";
                                $img_url = "$generation_url/$file_name.$ext";
                            }
                        }
                        if ($success == 0) {
                            unlink($generation_path . '/' . $file_name . ".$ext");
                        }
                    } else {
                        $success = 0;
                        $message = __('An error occured during the upload. Please try again later', 'csd');
                    }
                } else {
                    $success = 0;
                    $message = __('Incorrect file extension: ' . $ext . '. Allowed extensions: ', 'csd') . implode(', ', $valid_formats);
                }

                if ($success === 1) {
                    $new_csd_upload = array(
                        'img_url' => $img_url,
                        'img_id' => $img_id,
                        'img_name' => $name = $_FILES['userfile']['name'],
                    );
                    if (!is_user_logged_in()) {
                        if (isset($_COOKIE['csd-upload']) && !empty($_COOKIE['csd-upload'])) {
                            $csd_upload_json = json_decode(base64_decode($_COOKIE['csd-upload']));
                            if (null === $csd_upload_json) {
                                $csd_upload = array();
                            } else {
                                $csd_upload = $csd_upload_json;
                            }
                        } else {
                            $csd_upload = array();
                        }
                        array_push($csd_upload, $new_csd_upload);
                        $csd_upload_json = base64_encode(json_encode($csd_upload));
                        setcookie('csd-upload', $csd_upload_json, time() + 365 * 24 * 3600, '/');
                    } else {
                        global $current_user;
                        $current_user_id = $current_user->ID;
                        $user_uploads = get_user_meta($current_user_id, 'csd_saved_uploads');
                        if (empty($user_uploads)) {
                            $user_uploads = array();
                        } else {
                            $user_uploads = $user_uploads[0];
                        }
                        $user_uploads[] = $new_csd_upload;
                        update_user_meta($current_user_id, 'csd_saved_uploads', $user_uploads);
                    }
                }

                echo json_encode(
                        array(
                            'success' => $success,
                            'message' => $message,
                            'img_url' => $img_url,
                            'img_id' => $img_id,
                        )
                );
            }
        }
        die();
    }

    /**
     * Handle delete picture upload.
     * 
     * @global type $current_user
     */
    public function csd_handle_delete_picture_upload() {
        $success = 0;
        $element_key = '';
        $img_id = filter_input(INPUT_POST, 'img_id');
        $img_url = filter_input(INPUT_POST, 'img_url');
        $img_name = filter_input(INPUT_POST, 'img_name');
        if (!is_user_logged_in()) {
            if (isset($_COOKIE['csd-upload']) && !empty($_COOKIE['csd-upload'])) {
                $csd_upload_json = json_decode(base64_decode($_COOKIE['csd-upload']));
                if (null === $csd_upload_json) {
                    $csd_upload = array();
                } else {
                    $csd_upload = $csd_upload_json;
                }
            } else {
                $csd_upload = array();
            }
            if (is_array($csd_upload) && !empty($csd_upload)) {
                foreach ($csd_upload as $key => $value) {
                    if ($value->img_id == $img_id && $value->img_name == $img_name && $value->img_url == $img_url) {
                        $element_key = $key;
                        break;
                    }
                }
            }
            if (!empty($element_key) || 0 === $element_key) {
                unset($csd_upload[$element_key]);
                //array_splice( $csd_upload, $element_key );
                $csd_upload_json = base64_encode(json_encode($csd_upload));
                setcookie('csd-upload', $csd_upload_json, time() + 365 * 24 * 3600, '/');
                $success = 1;
            } else {
                $success = 0;
            }
        } else {
            global $current_user;
            $current_user_id = $current_user->ID;
            $user_uploads = get_user_meta($current_user_id, 'csd_saved_uploads');
            if (empty($user_uploads)) {
                $user_uploads = array();
            } else {
                $user_uploads = $user_uploads[0];
            }
            if (is_array($user_uploads) && !empty($user_uploads)) {
                foreach ($user_uploads as $key => $value) {
                    if ($value['img_id'] == $img_id && $value['img_name'] == $img_name && $value['img_url'] == $img_url) {
                        $element_key = $key;
                        break;
                    }
                }
            }
            if (!empty($element_key) || 0 === $element_key) {
                unset($user_uploads[$element_key]);
                //array_splice( $user_uploads, $element_key );
                update_user_meta($current_user_id, 'csd_saved_uploads', $user_uploads);
                $success = 1;
            } else {
                $success = 0;
            }
        }
        echo json_encode($success);
        die();
    }

    /**
     * Move user picture uploads.
     *
     * @param type $user_id The user id.
     */
    public function move_user_picture_uplaods($user_id) {
        $user_uploads = array();
        if (isset($_COOKIE['csd-upload']) && !empty($_COOKIE['csd-upload'])) {
            $csd_upload_json = json_decode(base64_decode($_COOKIE['csd-upload']));
            if (null === $csd_upload_json) {
                $csd_upload = array();
            } else {
                $csd_upload = $csd_upload_json;
            }
        } else {
            $csd_upload = array();
        }
        foreach ($csd_upload as $image) {
            $user_upload = array();
            if (isset($image->img_id) && !empty($image->img_id)) {
                $user_upload['img_id'] = $image->img_id;
            }
            if (isset($image->img_url) && !empty($image->img_url)) {
                $user_upload['img_url'] = $image->img_url;
            }
            if (isset($image->img_name) && !empty($image->img_name)) {
                $user_upload['img_name'] = $image->img_name;
            }

            array_push($user_uploads, $user_upload);
        }
        add_user_meta($user_id, 'csd_saved_uploads', $user_uploads);
    }

    /*
     * Retrieve woocommerce version
     *
     */

    private function csd_get_woo_version_number() {
// If get_plugins() isn't available, require it
        if (!function_exists('get_plugins')) {
            require_once ABSPATH . 'wp-admin/includes/plugin.php';
        }

// Create the plugins folder and file variables
        $plugin_folder = get_plugins('/woocommerce');
        $plugin_file = 'woocommerce.php';

// If the plugin version number is set, return it
        if (isset($plugin_folder[$plugin_file]['Version'])) {
            return $plugin_folder[$plugin_file]['Version'];
        } else {
// Otherwise return null
            return null;
        }
    }

    /**
     * Display user selection in cart
     *
     * @param type $thumbnail_code Thumbnail code.
     * @param type $cart_item Cart item.
     * @param type $cart_item_key Cart item key.
     * @return string
     */
    public function csd_get_data($thumbnail_code, $cart_item, $cart_item_key) {
        if (isset($cart_item['csd_generated_data']) && isset($cart_item['csd_generated_data']['output'])) {
            $variation_id = $cart_item['variation_id'];
            $thumbnail_code .= '<br>';
            $customization_list = $cart_item['csd_generated_data'];
            $upload_dir = wp_upload_dir();
            $modals = '';
            if (class_exists('Ofb')) {
                if (isset($customization_list['output']['form_fields'])) {
                    $form_fields = $customization_list['output']['form_fields'];
                    foreach ($form_fields as $key => $value) {
                        if (!is_array($value)) {
                            $thumbnail_code .= '<p>' . $key . ' : ' . $value . '</p>';
                        } else {
                            $thumbnail_code .= '<p>' . $key . ' : ';
                            foreach ($value as $item => $data) {
                                $thumbnail_code .= $data . ' ';
                            }
                            $thumbnail_code .= ' </p>';
                        }
                    }
                }
            }
            $i = 0;

            foreach ($customization_list['output']['files'] as $customisation_key => $customization) {

                $tmp_dir = $customization_list['output']['working_dir'];
                $generation_url = $upload_dir['baseurl'] . "/CSD/$tmp_dir/$customisation_key/";
                if (isset($customization['preview'])) {
                    $image = $generation_url . $customization['preview'];
                } else {
                    $image = $generation_url . $customization['image'];
                }
                $original_part_img_url = $customization_list[$customisation_key]['original_part_img'];
                $modal_id = $variation_id . '_' . $cart_item_key . "$customisation_key-$i";

                $thumbnail_code .= '<span><a class="button" data-toggle="o-modal" data-target="#' . $modal_id . '">' . ucfirst($customisation_key) . '</a></span>';
                $modals .= '<div class="omodal fade wpc-modal wpc_part" id="' . $modal_id . '" tabindex="-1" role="dialog" aria-labelledby="myModalLabel" aria-hidden="true">
                                    <div class="omodal-dialog">
                                      <div class="omodal-content">
                                        <div class="omodal-header">
                                          <button type="button" class="close" data-dismiss="omodal" aria-hidden="true">&times;</button>
                                          <h4 class="omodal-title">' . __('Preview', 'wpd') . '</h4>
                                        </div>
                                        <div class="omodal-body txt-center">
                                            <div style="background-image:url(' . $original_part_img_url . ')"><img src="' . $image . '"></div>
                                        </div>
                                      </div>
                                    </div>
                                  </div>';
                $i++;
            }
            array_push(csd_retarded_actions::$code, $modals);
            add_action('wp_footer', array('csd_retarded_actions', 'display_code'), 10, 1);
        }
        
        echo $thumbnail_code;
    }

    /**
     *
     * @global type $csd_settings
     * @param type $item_id Item id.
     * @param type $order_item Oder item object.
     * @param type $order_id Order id.
     */
    public function save_customized_item_meta($item_id, $order_item, $order_id) {
        global $csd_settings;
        $output_options = get_proper_value($csd_settings, 'csd-output-options', array());
        $use_order_id = get_proper_value($output_options, 'use-order-id-as-zip-name', 'no');
        if ($use_order_id == 'yes') {
            $upload_dir = wp_upload_dir();
            $old_tmp_dir = $order_item->legacy_values['csd_generated_data']['output']['working_dir'];
            $old_generation_path = $upload_dir['basedir'] . "/CSD/$old_tmp_dir";
            $new_generation_path = $upload_dir['basedir'] . "/CSD/$order_id-$item_id";
            // Suppression ancien zip
            unlink($old_generation_path . '/' . $order_item->legacy_values['csd_generated_data']['output']['zip']);
            // Renommage du répertoire
            rename($old_generation_path, $new_generation_path);
            // Regénération du zip
            $new_zip_name = "$order_id-$item_id.zip";
            $this->generate_design_archive($new_generation_path, "$new_generation_path/$new_zip_name");
            // MAJ des metas
            $order_item->legacy_values['csd_generated_data']['output']['working_dir'] = "$order_id-$item_id";
            $order_item->legacy_values['csd_generated_data']['output']['zip'] = $new_zip_name;
        }
        if (isset($order_item->legacy_values['csd_generated_data'])) {
            wc_add_order_item_meta($item_id, 'csd_data', $order_item->legacy_values['csd_generated_data']);
        }
    }

    /**
     *
     * @global type $csd_settings
     * @param type $item_id Item id.
     * @param type $item Item object.
     * @param type $_product Product object.
     * @return type
     */
    function csd_get_order_custom_admin_data($item_id, $item, $_product) {
        global $csd_settings;
        $output_options = get_proper_value($csd_settings, 'csd-output-options', array());
        $design_composition_visible = get_proper_value($output_options, 'design-composition', 'no');
        $output = '';
        if (!is_object($item)) {
            return;
        }
        $uploaded_data = wc_get_order_item_meta($item_id, 'csd_data_upl');
        if ($uploaded_data) {
            $output .= "<div class='csd_order_item' data-item='$item_id'>";
            foreach ($uploaded_data as $design_url) {
                $output .= "<a class='button' href='" . $design_url . "' download='" . basename($design_url) . "'>" . __('Download custom design', 'csd') . '</a> ';
            }
            $output .= '</div>';
        } elseif (isset($item['csd_data'])) {
            $upload_dir = wp_upload_dir();
            $output .= "<div class='csd_order_item' data-item='$item_id'>";
            $unserialized_data = $item['csd_data'];
            $design_data = $item['csd_data']['output']['files'];

            $customization_list = $item['csd_data'];

            if (class_exists('Ofb')) {
                if (isset($customization_list['output']['form_fields'])) {
                    $form_fields = $customization_list['output']['form_fields'];
                    foreach ($form_fields as $key => $value) {
                        if (!is_array($value)) {
                            $output .= '<p>' . $key . ' : ' . $value . '</p>';
                        } else {
                            $output .= '<p>' . $key . ' : ';
                            foreach ($value as $item => $data) {
                                $output .= $data . ' ';
                            }
                            $output .= ' </p>';
                        }
                    }
                }
            }

            $design_details = '';
            if (isset($unserialized_data['output']['tpl']) && !empty($unserialized_data['output']['tpl'])) {
                $tpl_name = get_the_title($unserialized_data['output']['tpl']);
                $design_details = "<br><b>Used Template</b>: $tpl_name<br>";
            }
            /* if ($design_composition_visible == 'yes') {
              $design_details .= $this->get_desing_details_from_json($unserialized_data);
              } */
            if (count($item['item_meta']['csd_data']) > 1) {
                foreach ($design_data as $data_key => $data) {
                    $tmp_dir = $unserialized_data['output']['working_dir'];
                    $generation_url = $upload_dir['baseurl'] . "/CSD/$tmp_dir/$data_key/";
                    if (is_admin()) {
                        $img_src = $generation_url . $data['image'];
                    } else {
                        if (isset($data['preview'])) {
                            $img_src = $generation_url . $data['preview'];
                        } else {
                            $img_src = $generation_url . $data['image'];
                        }
                    }
                    $original_part_img_url = $unserialized_data[$data_key]['original_part_img'];
                    $modal_id = uniqid() . "_$item_id" . "_$data_key";
                    $output .= '<span><a class="o-modal-trigger button" data-toggle="o-modal" data-target="#' . $modal_id . '">' . ucfirst($data_key) . '</a></span>';
                    $output .= '<div class="omodal fade o-modal wpc_part" id="' . $modal_id . '" tabindex="-1" role="dialog" aria-labelledby="myModalLabel" aria-hidden="true">
                                <div class="omodal-dialog">
                                  <div class="omodal-content">
                                    <div class="omodal-header">
                                      <button type="button" class="close" data-dismiss="omodal" aria-hidden="true">&times;</button>
                                      <h4 class="omodal-title" id="myModalLabel' . $modal_id . '">Preview</h4>
                                    </div>
                                    <div class="omodal-body">
                                        <div style="background-image:url(' . $original_part_img_url . ')"><img src="' . $img_src . '"></div>
                                    </div>
                                  </div>
                                </div>
                              </div>';
                }
            }
            $zip_file = $unserialized_data['output']['zip'];
            if (!empty($zip_file)) {
                $output .= "<a class='button' href='" . $upload_dir['baseurl'] . "/CSD/$tmp_dir/$zip_file' download='" . basename($zip_file) . "'>" . __('Download design', 'csd') . '</a> ';
            }
            if (isset($item['csd_design_pricing_options'])) {
                $output .= $item['csd_design_pricing_options'];
            }

            $output .= $design_details;
            $output .= '</div>';
        }

        echo $output;
    }

    /**
     * Creates a compressed zip file
     *
     * @param type $source Input directory path to zip
     * @param type $destination Output file path
     * @return boolean
     */
    private function generate_design_archive($source, $destination) {

        if (!extension_loaded('zip') || !file_exists($source)) {
            return false;
        }

        $zip = new ZipArchive();
        if (!$zip->open($destination, ZIPARCHIVE::CREATE)) {
            return false;
        }

        $source = str_replace('\\', DIRECTORY_SEPARATOR, realpath($source));

        if (is_dir($source) === true) {
            $files = new RecursiveIteratorIterator(new RecursiveDirectoryIterator($source), RecursiveIteratorIterator::SELF_FIRST);

            foreach ($files as $file) {
                $file = str_replace('\\', DIRECTORY_SEPARATOR, $file);

                // Ignore "." and ".." folders
                if (in_array(substr($file, strrpos($file, '/') + 1), array('.', '..'), true)) {
                    continue;
                }

                $file = realpath($file);

                if (is_dir($file) === true) {
                    $zip->addEmptyDir(str_replace($source . DIRECTORY_SEPARATOR, '', $file . DIRECTORY_SEPARATOR));
                } elseif (is_file($file) === true) {
                    $zip->addFromString(str_replace($source . DIRECTORY_SEPARATOR, '', $file), $this->url_get_contents($file));
                }
            }
        } elseif (is_file($source) === true) {
            $zip->addFromString(basename($source), $this->url_get_contents($source));
        }

        return $zip->close();
    }

    /**
     * Replacement just in case file_get_contents fails
     *
     * @param type $Url
     * @return type
     */
    public function url_get_contents($Url) {
        // If it's a path, we prefer use the file_get_contents.
        if (function_exists('curl_init') && !file_exists($Url)) {
            $ch = curl_init();
            curl_setopt($ch, CURLOPT_URL, $Url);
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
            $output = curl_exec($ch);
            curl_close($ch);
        } else {
            $output = file_get_contents($Url);
        }

        return $output;
    }

    /**
     * Function to display user design on account page.
     *
     * @global type $csd_settings
     * @param type $item_id Item id.
     * @param type $item Cart item object.
     * @param type $order Order object.
     * @return type
     */
    public function csd_get_user_account_products_meta($item_id, $item, $order) {
        if (!is_account_page()) {
            return;
        }
        global $csd_settings;
        $options = $csd_settings['csd-general-options'];
        $output = '';
        $download_btn = get_proper_value($options, 'csd-user-account-download-btn', '');
        $invalid_statuses = array('wc-cancelled', 'wc-refunded', 'wc-failed');
        if (!in_array($order->get_status(), $invalid_statuses) && $download_btn !== '0' && isset($item['variation_id']) && (!empty($item['variation_id']) || $item['variation_id'] == '0' )) {
            $product = wc_get_product($item['variation_id']);
            $item_id = uniqid();
            ob_start();
            $this->csd_get_order_custom_admin_data($item_id, $item, $product);
            $admin_data = ob_get_contents();
            ob_end_clean();
            $output .= $admin_data;
        }
        echo $output;
    }

    /**
     * Function to handle design integration in mail as attachement.
     *
     * @global type $csd_settings
     * @param type $attachments Files attached to mail.
     * @param type $status Order status.
     * @param type $order Order object.
     * @return type
     */
    public function csd_add_order_design_to_mail($attachments, $status, $order) {

        global $csd_settings;
        $options = $wpd_settings['csd-general-options'];
        $download_btn = $options['csd-send-design-mail'];
        if ($download_btn !== '0') {
            $allowed_statuses = array('new_order', 'customer_invoice', 'customer_processing_order', 'customer_completed_order');
            if (isset($status) && in_array($status, $allowed_statuses)) {
                $items = $order->get_items();
                foreach ($items as $order_item_id => $item) {
                    $upload_dir = wp_upload_dir();
                    if (isset($item['csd_data'])) {
                        $unserialized_data = $item['csd_data'];
                        $tmp_dir = $unserialized_data['output']['working_dir'];
                        array_push($attachments, $upload_dir['basedir'] . "/CSD/$tmp_dir/" . $unserialized_data['output']['zip']);
                    } elseif (isset($item['csd_data_upl'])) {
                        // Looks like the structure changed for latest versions of WC (tested on 2.3.7)
                        $design_url = $item['item_meta']['csd_data_upl'][0];
                        if (is_serialized($design_url)) {
                            $unserialized_urls = $design_url;
                            foreach ($unserialized_urls as $design_url) {
                                $design_path = str_replace($upload_dir['baseurl'], $upload_dir['basedir'], $design_url);
                                array_push($attachments, $design_path);
                            }
                        } else {
                            $design_path = str_replace($upload_dir['baseurl'], $upload_dir['basedir'], $design_url);
                            array_push($attachments, $design_path);
                        }
                    }
                }
            }
            $attachments = str_replace('"', '', $attachments);
        }
        return $attachments;
    }

    /**
     * Function to calculate cart item price
     *
     * @param type $cart Woocommerce cart.
     */
    public function csd_get_cart_item_price($cart) {
        foreach ($cart->cart_contents as $key => $cart_item) {
            if (isset($cart_item['variation_id']) && 0 !== $cart_item['variation_id']) {
                $csd_product = new CSD_Product($cart_item['variation_id']);
            } else {
                $csd_product = new CSD_Product($cart_item['product_id']);
            }
            if (isset($csd_product) && $csd_product->is_customizable()) {
                $price = (double) $csd_product->product->get_price();

                if (isset($csd_product->settings)) {
                    $csd_metas = $csd_product->settings;
                    $bg_palette = get_proper_value($csd_metas, 'bg-colors-palette', '');
                    $text_palette = get_proper_value($csd_metas, 'text-colors-palette', '');

                    $pricing_rules = get_proper_value($csd_metas, 'pricing-rules', '');
                }
                if (isset($cart_item) && isset($cart_item['csd_generated_data'])) {
                    $csd_generated_data = $cart_item['csd_generated_data'];
                }
                if (isset($bg_palette) && !empty($bg_palette) && isset($csd_generated_data)) {
                    foreach ($bg_palette as $key => $value) {
                        if (isset($value['hex']) && isset($csd_generated_data['canvas_color']) && strtoupper($value['hex']) === strtoupper($csd_generated_data['canvas_color'])) {
                            $price += (double) $value['price'];
                        }
                    }
                }

                if (isset($text_palette) && !empty($text_palette) && isset($csd_generated_data)) {
                    foreach ($text_palette as $key => $value) {
                        if (isset($value['hex']) && isset($csd_generated_data['text_color']) && strtoupper($value['hex']) === strtoupper($csd_generated_data['text_color'])) {
                            $price += (double) $value['price'];
                        }
                    }
                }

                $additionnal_price_per_item = 0;
                $additionnal_price_on_design = 0;
                $additionnal_price_per_additional_item = 0;

                if (isset($pricing_rules) && !empty($pricing_rules) && isset($csd_generated_data)) {
                    foreach ($pricing_rules as $key => $rules) {
                        $verify_result = $this->csd_condition_verify($rules, $csd_generated_data);
                        if ($verify_result) {
                            if ($verify_result['verify']) {
                                $additionnal_price_per_additional_item = $additionnal_price_per_additional_item + (double) $verify_result['additionnal_price'];
                                if (isset($rules['a_price'])) {
                                    if ($rules['scope'] === 'design' || $verify_result['param'] === 'if_selected_font') {
                                        $additionnal_price_on_design = $additionnal_price_on_design + (double) $rules['a_price'];
                                    }
                                }
                            }
                        }
                    }
                }

                if (isset($cart_item['csd_generated_data'])) {
                    $form_fields_data = array();
                    $total_price_form = 0;
                    $data = $cart_item['csd_generated_data'];
                    if (class_exists('Ofb')) {
                        if (isset($csd_metas['form-builder'])) {
                            if ($csd_metas['form-builder'] != '') {
                                if (isset($data['output']['total_price_form'])) {
                                    $total_price_form = $data['output']['total_price_form'];
                                }
                            }
                        }
                    }
                }
                $price += $additionnal_price_per_additional_item + $additionnal_price_on_design + $total_price_form;
                $cart_item['data']->set_price($price);
            }
        }
    }

    /**
     * Function that verify rules conditions
     *
     * @param type $rules Array of pricing rules.
     * @param type $csd_generated_data Generatedd data.
     * @return boolean
     */
    public function csd_condition_verify($rules, $csd_generated_data) {
        $data = $csd_generated_data;
        foreach ($data as $part => $part_data) {

            $raw_json = $part_data["json"];
            $json = str_replace("\n", "|n", $raw_json);
            $unslashed_json = stripslashes_deep($json);
            $decoded_json = json_decode($unslashed_json);
            $json_a = json_decode($json);
            if (!is_object($decoded_json)) {
                continue;
            }

            $comparaison_result = [];
            if (isset($rules) && !empty($rules)) {
                if (isset($rules['rules']) && !empty($rules['rules'])) {
                    foreach ($rules['rules'] as $key => $rule) {
                        switch ($rule['param']) {
                            case 'txt_nb_chars':
                                foreach ($decoded_json->objects as $key => $object) {
                                    $object_type = $object->type;
                                    if ($object_type == "text") {
                                        $object_text = $object->text;
                                        if (isset($object_text)) {
                                            $text_size = sizeof(str_split($object_text));
                                            $result = $this->csd_comparaison($rule['operator'], $text_size, $rule['value'], $rules['scope']);
                                            $result['param'] = $rule['param'];
                                            $comparaison_result[] = $result;
                                        }
                                    }
                                }
                                break;
                            case 'txt_nb_lines':
                                foreach ($decoded_json->objects as $key => $object) {
                                    $object_type = $object->type;
                                    if ($object_type == "text") {
                                        $object_text = $object->text;
                                        if (isset($object_text)) {
                                            $nb_line = mb_substr_count($object_text, "\n");
                                            $nb_line = $nb_line + 1;
                                            $result = $this->csd_comparaison($rule['operator'], $nb_line, $rule['value'], $rules['scope']);
                                            $result['param'] = $rule['param'];
                                            $comparaison_result[] = $result;
                                        }
                                    }
                                }
                                break;
                            case 'if_canvas_square_unit':
                                if (isset($data['canvas_w']) && isset($data['canvas_h'])) {
                                    $canvas_square = (int) $data['canvas_w'] * (int) $data['canvas_h'];
                                    $result = $this->csd_comparaison($rule['operator'], $canvas_square, $rule['value'], $rules['scope']);
                                    $result['param'] = $rule['param'];
                                    $comparaison_result[] = $result;
                                }
                                break;
                            case 'if_selected_font':
                                foreach ($decoded_json->objects as $key => $object) {
                                    $object_type = $object->type;
                                    if ($object_type == "text") {
                                        $object_font = $object->fontFamily;
                                        if (isset($object_font)) {
                                            $result = $this->csd_comparaison($rule['operator'], $object_font, $rule['value'], $rules['scope']);
                                            $result['param'] = $rule['param'];
                                            $comparaison_result[] = $result;
                                        }
                                    }
                                }
                                break;
                            default:
                                $comparaison_result = apply_filters('csd_custom_palette_tpl', $comparaison_result, $rules, $csd_generated_data);
                                break;
                        }
                    }

                    $verify = 0;
                    foreach ($comparaison_result as $key => $value) {
                        if ($value['result']) {
                            $verify++;
                        }
                    }

                    $additionnal_price_per_item = 0;
                    $result = array('verify' => false);

                    if (sizeof($comparaison_result) === $verify && isset($value) && !empty($value)) {
                        $result['verify'] = true;
                        $result['param'] = $value['param'];
                        foreach ($comparaison_result as $key => $value) {
                            if ($rules['scope'] === 'additional_item' || $rules['scope'] === 'item' && isset($value['remaining_number'])) {
                                $additionnal_price_per_item = $additionnal_price_per_item + ( $value['remaining_number'] * $rules['a_price'] );
                            } else {
                                $additionnal_price_per_item = $additionnal_price_per_item + 0;
                            }
                        }
                        $result['additionnal_price'] = $additionnal_price_per_item;
                    }
                    return $result;
                }
            }
        }
    }

    public function csd_get_design_price() {
        $serialized_parts = (array) json_decode(stripslashes_deep($_POST['serialized_parts']));
        if (isset($serialized_parts['product_id'])) {
            $variation_id = $serialized_parts['product_id'];
            $csd_product = new CSD_Product($variation_id);
            $wpc_metas = $csd_product->settings;
            $get_form_data = 0;
            if (class_exists('Ofb')) {
                if (isset($wpc_metas['form-builder'])) {
                    if ($wpc_metas['form-builder'] != '') {
                        $form_fields = $serialized_parts['form_fields'];

                        $data = array();
                        if (isset($form_fields)) {
                            foreach ($form_fields as $key => $value) {
                                $data[$key] = $value;
                            }
                        }

                        $get_form_data = get_form_data($wpc_metas['form-builder'], $data);
                    }
                }
            }
        }
        echo json_encode(
                array(
                    'prices' => $get_form_data,
                )
        );
        die();
    }

    /**
     * Function that compares values
     *
     * @param type $operator Given operator.
     * @param type $given_value Given value.
     * @param type $rule_value Rule value.
     * @return type
     */
    public function csd_comparaison($operator, $given_value, $rule_value, $rule_scope) {
        $data = [];
        if ('>' === $operator) {
            $data['result'] = $given_value > $rule_value;
            if ($given_value > $rule_value) {
                if ('additional_item' === $rule_scope) {
                    $data['remaining_number'] = $given_value - $rule_value;
                } elseif ('item' === $rule_scope) {
                    $data['remaining_number'] = $given_value;
                }
            }
            return $data;
        } elseif ('>=' === $operator) {
            $data['result'] = $given_value >= $rule_value;
            if ($given_value >= $rule_value) {
                if ('additional_item' === $rule_scope) {
                    $data['remaining_number'] = ( $given_value - $rule_value ) + 1;
                } elseif ('item' === $rule_scope) {
                    $data['remaining_number'] = $given_value;
                }
            }
            return $data;
        } elseif ('<' === $operator && 'additional_item' !== $rule_scope) {
            $data['result'] = $given_value < $rule_value;
            if ($given_value < $rule_value) {
                if ('item' === $rule_scope) {
                    $data['remaining_number'] = $given_value;
                }
            }
            return $data;
        } elseif ('<=' === $operator && 'additional_item' !== $rule_scope) {
            $data['result'] = $given_value <= $rule_value;
            if ($given_value < $rule_value) {
                if ('item' === $rule_scope) {
                    $data['remaining_number'] = $given_value;
                }
            }
            return $data;
        } elseif ('==' == $operator && 'additional_item' !== $rule_scope) {
            $data['result'] = $given_value == $rule_value;
            if (is_numeric($given_value) && is_numeric($rule_value) && $given_value == $rule_value) {
                if ('item' === $rule_scope) {
                    $data['remaining_number'] = $given_value;
                }
            }
            return $data;
        } else {
            $data['result'] = $given_value != $rule_value;
            if (is_numeric($given_value) && is_numeric($rule_value) && $given_value !== $rule_value) {
                $data['remaining_number'] = $given_value;
            }
            return $data;
        }
    }

    /**
     * Save the file as pdf
     *
     * @param type $variation_id
     * @param type $input_file
     * @param type $output_file
     * @param type $pdf_fonts
     */
    private function csd_save_pdf_output($output_file, $data) {
        $product_id = $data['product_id'];
        $config = csd_get_product_config($product_id);
        $product_metas = $config->settings;
        $product_output_settings = get_proper_value($product_metas, 'output-settings', array());
        $text = $data['text'];

        $font_ttf = $data['font-ttf'];

        if ($data['style'] === 'bold') {
            $fonturl = $data['font-url'] . ':700';
        } elseif ($data['style'] === 'italic') {
            $fonturl = $data['font-url'] . ':400i';
        } elseif ($data['style'] === 'both') {
            $fonturl = $data['font-url'] . ':700i';
        } else {
            $fonturl = $data['font-url'];
        }

        if ('' !== $fonturl && '' === $font_ttf) {
            $font_ttf = csd_retrieve_ttf_from_url($fonturl);
        }
        $pdf_format = get_proper_value($product_output_settings, 'pdf-format', 'A0');
        $pdf_unit = 'mm';

        /*
         * We convert font size in point and all other values to mm
         */
        if ($data['unit'] !== 'mm') {
            if ($data['unit'] !== 'pt') {
                $font_size_in_pixels = csd_convert_unit_to_px($data['font-size'], $data['unit']);
                $font_size = csd_convert_px_to_unit($font_size_in_pixels, 'pt');
            } else {
                $font_size = $data['font-size'];
            }

            if ($pdf_format == 'custom') {
                $pdf_width = csd_convert_px_to_unit(csd_convert_unit_to_px(get_proper_value($product_output_settings, 'pdf-width', 0), $data['unit']), 'mm');
                $pdf_height = csd_convert_px_to_unit(csd_convert_unit_to_px(get_proper_value($product_output_settings, 'pdf-height', 0), $data['unit']), 'mm');
            } else {
                $dimensions = TCPDF_STATIC::getPageSizeFromFormat($product_output_settings['pdf-format']);
                $pdf_width = csd_convert_px_to_unit(csd_convert_unit_to_px($dimensions[0], $data['unit']), 'mm');
                $pdf_height = csd_convert_px_to_unit(csd_convert_unit_to_px($dimensions[1], $data['unit']), 'mm');
            }

            $custom_width = csd_convert_px_to_unit(csd_convert_unit_to_px($data['custom-width'], $data['unit']), 'mm');
            $custom_height = csd_convert_px_to_unit(csd_convert_unit_to_px($data['custom-height'], $data['unit']), 'mm');
            $pdf_margin_lr = csd_convert_px_to_unit(csd_convert_unit_to_px(get_proper_value($product_output_settings, 'pdf-margin-lr', 0), $data['unit']), 'mm');
            $pdf_margin_tb = csd_convert_px_to_unit(csd_convert_unit_to_px(get_proper_value($product_output_settings, 'pdf-margin-tb', 0), $data['unit']), 'mm');
        } else {
            $custom_width = $data['custom-width'];
            $custom_height = $data['custom-height'];
            $font_size_in_pixels = csd_convert_unit_to_px($data['font-size'], $data['unit']);
            $font_size = csd_convert_px_to_unit($font_size_in_pixels, 'pt');
            if ($pdf_format == 'custom') {
                $pdf_width = get_proper_value($product_output_settings, 'pdf-width', 0);
                $pdf_height = get_proper_value($product_output_settings, 'pdf-height', 0);
            } else {
                $dimensions = TCPDF_STATIC::getPageSizeFromFormat($product_output_settings['pdf-format']);
                $pdf_width = $dimensions[0];
                $pdf_height = $dimensions[1];
            }
            $pdf_margin_lr = get_proper_value($product_output_settings, 'pdf-margin-lr', 0);
            $pdf_margin_tb = get_proper_value($product_output_settings, 'pdf-margin-tb', 0);
        }

        $shadow = get_proper_value($data, 'shadow', 0);
        $outline = get_proper_value($data, 'outline', 0);

        if (strpos($pdf_format, ',') !== false) {
            $formats = explode(',', $pdf_format);
            $pdf_format = $formats[0];
        }

        if ($pdf_width < $custom_width) {
            $pdf_width = $custom_width;
        }

        if ($pdf_height < $custom_height) {
            $pdf_height = $custom_height;
        }

        $output_format = array($pdf_width, $pdf_height);

        // set pdf orientation
        $pdf_orientation = get_proper_value($product_output_settings, 'pdf-orientation', 'P');

        $pdf = new TCPDF($pdf_orientation, $pdf_unit, $output_format, true, 'UTF-8', false);

        // set pdf unit
        $pdf->setPageUnit($pdf_unit);
        $pdf->setPageOrientation($pdf_orientation);
        $pdf->SetCreator('Lettering and signs Designer by ORION');
        $pdf->SetAuthor('Lettering and signs Designer by ORION');
        $pdf->SetTitle('PDF output');
        $pdf->setPrintHeader(false);
        $pdf->setPrintFooter(false);

        // set margins
        $pdf->SetMargins($pdf_margin_lr, $pdf_margin_tb, -1, true);
        $pdf->SetHeaderMargin($pdf_margin_tb);
        $pdf->SetFooterMargin($pdf_margin_tb);

        // convert TTF font to TCPDF format and store it on the fonts folder
        if ($font_ttf === '') {
            // set default monospaced font
            $pdf->SetDefaultMonospacedFont(PDF_FONT_MONOSPACED);
        } else {
            $fontname = $pdf->addTTFfont($font_ttf, 'TrueTypeUnicode', '', 96);
            $pdf->SetFont($fontname, '', $font_size, '', 'default', false);
        }

        $text_col = csd_hex_to_rgb($data['text_color']);

        $pdf->SetTextColor($text_col['0'], $text_col['1'], $text_col['2']);

        $pdf->AddPage();

        if ($shadow > 0) {
            $shadow_color = csd_hex_to_rgb($data['shadow_color']);
            // set text shadow effect
            $pdf->setTextShadow(
                    array(
                        'enabled' => true,
                        'depth_w' => $shadow,
                        'depth_h' => $data['shadow'],
                        'color' => $shadow_color,
                        'opacity' => 1,
                        'blend_mode' => 'Normal',
                    )
            );
        }

        if ($outline > 0) {
            $outline_color = csd_hex_to_rgb($data['outline_color']);
            $pdf->SetDrawColor($outline_color['0'], $outline_color['1'], $outline_color['2']);
        }

        // Write text
        $pdf->Text(0, 0, $text, $outline, false, true, 0, 0, '', false, '', 0, false, 'T', 'T', false);

        $pdf->Output($output_file, 'F');
    }

    /**
     * Execute imagick command.
     *
     * @param type $cmd The actual command to be executed
     * @return array
     */
    private function csd_exec($cmd) {
        $output = array();
        exec("$cmd 2>&1", $output);
        return $output;
    }

    /**
     * Export data to archive
     *
     * @param string                                $generation_dir Working directory path
     * @param array data Data to export
     * @param int variation_id Product/Variation ID
     * @return boolean|string
     */
    private function export_data_to_files($generation_dir, $data, $product_id, $zip_name, $pdf_watermark = false) {
        global $csd_settings;
        $global_output_settings = $csd_settings['csd-output-options'];
        // Set this value to true to have the pdf generated.
        $generate_pdf = false;
        $generate_zip = false;
        $wpc_cmyk_conversion = 'yes';

        $output_arr = array();

        $product_id = $product_id;
        $config = csd_get_product_config($product_id);
        $product_metas = $config->settings;
        $product_output_settings = get_proper_value($product_metas, 'output-settings', array());

        if ($product_output_settings['output-format'] == 'pdf+png' || $product_output_settings['output-format'] == 'pdf+jpg' || $product_output_settings['output-format'] == 'pdf+svg') {
            $generate_pdf = true;
            $format = explode('+', $product_output_settings['output-format']);
            $csd_img_format = $format[1];
        } else {
            $csd_img_format = $product_output_settings['output-format'];
        }

        if ($product_output_settings['zip-output'] == 'yes') {
            $generate_zip = true;
        }

        $allowed_extensions = array('png', 'jpg');
        if (!in_array($csd_img_format, $allowed_extensions)) {
            return false;
        }

        $output_arr = array();

        foreach ($data as $part_key => $part_data) {
            $part_dir = "$generation_dir/$part_key";

            if (!wp_mkdir_p($part_dir)) {
                echo "Can't create part directory...";
            }

            // Part image
            $name = $part_key;

            $output_file_path = $part_dir . "/$name.$csd_img_format";

            $moved = move_uploaded_file($_FILES[$name]['tmp_name']['image'], $output_file_path);

            $output_arr[$name]['image'] = "$name.$csd_img_format";

            $output_arr[$name]['preview'] = $output_arr[$name]['image'];

            if (!$generate_pdf && !$generate_zip) {
                $output_arr[$name]['file'] = "$name.$csd_img_format";
            }

            $fonts = array();
            // SVG
            if ($product_output_settings['output-format'] == 'svg' || $product_output_settings['output-format'] == 'pdf+svg') {
                $svg_path = $part_dir . "/$name.svg";

                file_put_contents($svg_path, stripcslashes($data['svg']), FILE_APPEND | LOCK_EX);
                $this->embbed_images_in_svg($svg_path, $svg_path);
                $output_file_path = $svg_path;

                // Fonts extraction
                $raw_json = $data['json'];
                $json = str_replace("\n", '|n', $raw_json);
                $unslashed_json = stripslashes_deep($json);
                $decoded_json = json_decode($unslashed_json);

                $map = array_map(create_function('$o', 'return $o->type;'), $decoded_json->objects);
                foreach ($decoded_json->objects as $object) {
                    $object_type = $object->type;
                    if ($object_type == 'text') {
                        if (!in_array($object->fontFamily, $fonts)) {
                            array_push($fonts, $object->fontFamily);
                        }
                    }
                }

                $output_arr[$name]['file'] = "$name.svg";
            }

            // Part pdf
            if ($generate_pdf) {

                $output_pdf_file_path = $part_dir . "/$name.pdf";

                $pdf_generation_method = apply_filters('csd_pdf_generation_method', array($this, 'save_pdf_output'));

                if ($pdf_watermark && $watermark_id) {
                    call_user_func($pdf_generation_method, $product_id, $preview_file_path, $output_pdf_file_path, $fonts);
                } else {
                    $preview_file_path = $output_file_path;
                    call_user_func($pdf_generation_method, $product_id, $output_file_path, $output_pdf_file_path, $fonts);
                }

                if (!$generate_zip) {
                    $output_arr[$name]['file'] = "$name.pdf";
                }
            }
        }
        if ($generate_zip)
            $this->generate_design_archive($generation_dir, "$generation_dir/$zip_name");
        return $output_arr;
    }

    private function embbed_images_in_svg($input, $output) {
        $xdoc = new DomDocument();
        $xdoc->Load($input);
        $images = $xdoc->getElementsByTagName('image');
        for ($i = 0; $i < $images->length; $i++) {
            $tagName = $xdoc->getElementsByTagName('image')->item($i);
            $attribNode = $tagName->getAttributeNode('xlink:href');
            $img_src = $attribNode->value;
            if (strpos($img_src, 'data:image') !== false) {
                continue;
            }

            $type = pathinfo($img_src, PATHINFO_EXTENSION);
            $data = $this->url_get_contents($img_src);
            $base64 = 'data:image/' . $type . ';base64,' . base64_encode($data);

            $tagName->setAttribute('xlink:href', $base64);

            $new_svg = $xdoc->saveXML();
            file_put_contents($output, $new_svg);
        }
    }

    private function save_pdf_output($variation_id, $input_file, $output_file, $pdf_fonts = array()) {
        $wpd_product = new CSD_Product($variation_id);
        $product_metas = $wpd_product->settings;
        $variation_output_settings = get_proper_value($product_metas, 'output-settings', array());
        $pdf_format = get_proper_value($variation_output_settings, 'pdf-format', 'A0');

        // Some formats are grouped together and separated by ,
        if (strpos($pdf_format, ',') !== false) {
            $formats = explode(',', $pdf_format);
            $pdf_format = $formats[0];
        }

        $pdf_orientation = get_proper_value($variation_output_settings, 'pdf-orientation', 'P');
        $pdf_margin_lr = get_proper_value($variation_output_settings, 'pdf-margin-lr', 0);
        $pdf_margin_tb = get_proper_value($variation_output_settings, 'pdf-margin-tb', 0);

        $pdf_unit = PDF_UNIT;
        if ($pdf_format == 'custom') {
            $pdf_width = get_proper_value($variation_output_settings, 'pdf-width', 0);
            $pdf_height = get_proper_value($variation_output_settings, 'pdf-height', 0);
            $pdf_unit = get_proper_value($variation_output_settings, 'pdf-unit', 'in');
            if ($pdf_unit == 'px') {
                $pdf_unit = 'mm';
                $pdf_width = $pdf_width * 0.264583333;
                $pdf_height = $pdf_height * 0.264583333;
            }
            $pdf_format = array($pdf_width, $pdf_height);
        } else {
            $dimensions = TCPDF_STATIC::getPageSizeFromFormat($variation_output_settings['pdf-format']);
            $pdf_width = $dimensions[0];
            $pdf_height = $dimensions[1];
        }

        $pdf = new TCPDF($pdf_orientation, $pdf_unit, $pdf_format, true, 'UTF-8', false);

        $pdf->SetCreator('Woocommerce Products Designer by ORION');
        $pdf->SetAuthor('Woocommerce Products Designer by ORION');
        $pdf->SetTitle('Output');
        $pdf->setPrintHeader(false);
        $pdf->setPrintFooter(false);

        // set default monospaced font
        $pdf->SetDefaultMonospacedFont(PDF_FONT_MONOSPACED);

        // set margins
        $pdf->SetMargins($pdf_margin_lr, $pdf_margin_tb, -1, true);
        $pdf->SetHeaderMargin($pdf_margin_tb);
        $pdf->SetFooterMargin($pdf_margin_tb);

        // set auto page breaks
        $pdf->SetAutoPageBreak(true, $pdf_margin_tb);

        // set image scale factor
        $pdf->setImageScale(PDF_IMAGE_SCALE_RATIO);

        // set some language-dependent strings (optional)
        if (@file_exists(dirname(__FILE__) . '/lang/eng.php')) {
            require_once dirname(__FILE__) . '/lang/eng.php';
            $pdf->setLanguageArray($l);
        }

        $pdf->AddPage();

        $path_parts = pathinfo($input_file);
        $ext = $path_parts['extension'];
        if ($ext == 'svg') {
            // We write the fonts
            $wpd_fonts = get_option('wpc-fonts');
            if (empty($wpd_fonts)) {
                $wpd_fonts = wpd_get_default_fonts();
            }
            $real_fonts_names = array();
            foreach ($wpd_fonts as $font) {
                $font_label = $font[0];
                if (in_array($font_label, $pdf_fonts)) {
                    // If the font file has been defined
                    if (isset($font[2]) && !empty($font[2])) {
                        $font_files = $font[2];
                        foreach ($font_files as $font_file_row) {
                            $styles = implode('', $font_file_row['styles']);

                            $font_file_path = get_attached_file($font_file_row['file_id']);
                            if (!file_exists($font_file_path)) {
                                continue;
                            }
                            // convert TTF font to TCPDF format and store it on the fonts folder
                            $fontname = TCPDF_FONTS::addTTFfont($font_file_path, 'TrueTypeUnicode', $styles, 96);
                            $real_fonts_names[$font_label] = $fontname;
                            // use the font
                            $pdf->SetFont($fontname, '', 14, '', false);
                        }
                    }
                }
            }
            $this->replace_fonts_names_in_svg($input_file, $real_fonts_names);
            $pf = TCPDF_STATIC::getPageSizeFromFormat($pdf_format);
            $pdf->ImageSVG($file = $input_file, $x = '', $y = '', $w = $pf[0], $h = 0, $link = '', $align = '', $palign = 'C', $border = 0, $fitonpage = true);
        } else {
            $pdf->Image($file = $input_file, $x = '', $y = '', $w = 0, $h = 0, $type = '', $link = '', $align = '', $resize = false, $dpi = 300, $palign = 'C', $ismask = false, $imgmask = false, $border = 0, $fitbox = false, $hidden = false, $fitonpage = false, $alt = false, $altimgs = array());
        }

        // Crop marks
        $crop_marks = get_proper_value($variation_output_settings, 'crop-marks', array());
        if (!empty($crop_marks) && is_array($crop_marks)) {
            foreach ($crop_marks as $crop_mark) {
                $pdf->cropMark($crop_mark['x'], $crop_mark['y'], $crop_mark['w'], $crop_mark['h'], $crop_mark['type']);
            }
        }

        // Zones
        $zones = get_proper_value($variation_output_settings, 'zones', array());
        if (!empty($zones) && is_array($zones)) {
            foreach ($zones as $zone) {
                $zone_w = $pdf_width - $zone['h'] * 2;
                $zone_h = $pdf_height - $zone['v'] * 2;
                $dash = get_proper_value($zone, 'dash-spacing', 0);
                $hex_color = get_proper_value($zone, 'color', '#FF0000');
                $color = wpd_hex_to_rgb($hex_color);
                $border_width = get_proper_value($zone, 'border-width', 0);
                $style3 = array(
                    'width' => $border_width,
                    'cap' => 'butt',
                    'join' => 'miter',
                    'dash' => $dash,
                    'color' => $color,
                );
                $pdf->Rect($zone['h'], $zone['v'], $zone_w, $zone_h, 'D', array('all' => $style3));
            }
        }

        $pdf->Output($output_file, 'F');
    }

    private function replace_fonts_names_in_svg($input, $fonts) {
        $xdoc = new DomDocument();
        $xdoc->Load($input);
        $text_elements = $xdoc->getElementsByTagName('text');
        for ($i = 0; $i < $text_elements->length; $i++) {
            $tagName = $xdoc->getElementsByTagName('text')->item($i);
            $attribNode = $tagName->getAttributeNode('font-family');
            $font_family = $attribNode->value;

            if (!isset($fonts[$font_family])) {
                continue;
            }
            $tagName->setAttribute('font-family', $fonts[$font_family]);

            $new_svg = $xdoc->saveXML();
            file_put_contents($input, $new_svg);
        }
    }

    function get_watermarked_preview() {
        $wpc_img_format = $_POST['format'];
        $allowed_extensions = array('png', 'jpg');
        if (!in_array($wpc_img_format, $allowed_extensions)) {
            return false;
        }
        $upload_dir = wp_upload_dir();
        $preview_filename = uniqid('preview_') . ".$wpc_img_format";
        $output_file_path = $upload_dir['basedir'] . '/WPC/' . $preview_filename;

        if (move_uploaded_file($_FILES['image']['tmp_name'], $output_file_path)) {
            $url = $upload_dir['baseurl'] . '/WPC/' . $preview_filename;
            echo json_encode(array('url' => $url));
        } else {
            echo "<div class='wpc_notification failure'>" . __('An error has occured. Please try again later or contact the administrator.', 'wpd');
        }
        die();
    }

    public function csd_save_user_choice(){
        session_start();

        if(isset($_POST["csd_data_attr"]))
        {
            $csd_attr = json_decode(filter_input(INPUT_POST, 'csd_data_attr'), true);
            $variation_id = filter_input(INPUT_POST, 'variation_id');
            $_SESSION["csd-save-user-choice-".$variation_id] = $csd_attr;

        }

        if(isset($_POST["csd_pricing_rules"]))
        {
            $pricing_rules = filter_input(INPUT_POST, 'csd_pricing_rules');
            setcookie("csd_pricing_rules", $pricing_rules, time() + 3600, COOKIEPATH, COOKIE_DOMAIN);
        }
        else setcookie("csd_pricing_rules", null, -1, COOKIEPATH, COOKIE_DOMAIN);
        
        echo "success";

        session_write_close();

        die;
    }
}
